Attribute VB_Name = "VFinger"
'********************************************************
'*                                                      *
'*                  VeriFinger 5.0 SDK                  *
'*                                                      *
'*  VFinger.bas                                         *
'*    VeriFinger interface declarations                 *
'*                                                      *
'*  Copyright (C) 1998-2006 Neurotechnologija           *
'*                                                      *
'********************************************************

Option Explicit
Option Base 0

' -----------------------------------------------------------------------
' Error handling
' -----------------------------------------------------------------------

' Error codes

' General
Public Const VFE_OK = 0
Public Const VFE_FAILED = -1
Public Const VFE_OUT_OF_MEMORY = -2
Public Const VFE_NOT_INITIALIZED = -3
Public Const VFE_ARGUMENT_NULL = -4
Public Const VFE_INVALID_ARGUMENT = -5
' Registration
Public Const VFE_NOT_REGISTERED = -2000
Public Const VFE_INVALID_SERIAL_NUMBER = -2001
Public Const VFE_INVALID_REGISTRATION_KEY = -2002
Public Const VFE_SCANNER_DRIVER_ERROR = -2003
Public Const VFE_REGISTRATION_NOT_NEEDED = -2004
Public Const VFE_NO_SCANNER = -2005
Public Const VFE_MORE_THAN_ONE_SCANNER = -2006
Public Const VFE_LM_CONNECTION_ERROR = -2007
Public Const VFE_LM_NO_MORE_LICENSES = -2008
' Parameters
Public Const VFE_INVALID_PARAMETER = -10
Public Const VFE_PARAMETER_READ_ONLY = -11
' Features extraction
Public Const VFE_ILLEGAL_IMAGE_RESOLUTION = -101
Public Const VFE_ILLEGAL_IMAGE_SIZE = -102
Public Const VFE_LOW_QUALITY_IMAGE = -103
' VeriFinger specific
Public Const VFE_INVALID_MODE = -1000

' -----------------------------------------------------------------------
' Initialization
' -----------------------------------------------------------------------

Public Declare Function VFInitialize Lib "vfvbp.dll" Alias "VBVFInitialize" () As Long
Public Declare Function VFFinalize Lib "vfvbp.dll" Alias "VBVFFinalize" () As Long

' -----------------------------------------------------------------------
' Registration
' -----------------------------------------------------------------------

Public Declare Function VFIsRegistered Lib "vfvbp.dll" Alias "VBVFIsRegistered" () As Long

' -----------------------------------------------------------------------
' Context
' -----------------------------------------------------------------------

Public Const VF_DEFAULT_CONTEXT = 0

' Contexts functions
Public Declare Function VFCreateContext Lib "vfvbp.dll" Alias "VBVFCreateContext" () As Long
Public Declare Function VFFreeContext Lib "vfvbp.dll" Alias "VBVFFreeContext" (ByVal context As Long) As Long

' -----------------------------------------------------------------------
' Parameters
' -----------------------------------------------------------------------

' Parameter types
Public Const VF_TYPE_VOID = 0
Public Const VF_TYPE_BYTE = 1       ' "Byte" data type in Visual Basic
Public Const VF_TYPE_SBYTE = 2      ' "Byte" data type in Visual Basic
Public Const VF_TYPE_WORD = 3       ' "Integer" data type in Visual Basic
Public Const VF_TYPE_SHORT = 4      ' "Integer" data type in Visual Basic
Public Const VF_TYPE_DWORD = 5      ' "Long" data type in Visual Basic
Public Const VF_TYPE_INT = 6        ' "Long" data type in Visual Basic
Public Const VF_TYPE_BOOL = 10      ' "Boolean" data type in Visual Basic
Public Const VF_TYPE_CHAR = 20      ' "Byte" data type in Visual Basic
Public Const VF_TYPE_STRING = 100   ' "String" data type in Visual Basic

' Parameters

' General
Public Const VFP_TYPE = 0           ' Get | Pass parameter identifier (VFP_XXX) as value | Returns one of VF_TYPE_XXX constants
Public Const VFP_NAME = 10           ' Get | String
Public Const VFP_VERSION_HIGH = 11   ' Get | DWord
Public Const VFP_VERSION_LOW = 12    ' Get | DWord
Public Const VFP_COPYRIGHT = 13      ' Get | String

' Features extraction
Public Const VFP_EXTRACT_FEATURES = 110                               ' Get/Set | Int | Can be one of the following
    Public Const VF_ALL_FEATURES = 0
    Public Const VF_OMIT_BLOCKED_ORIENTATIONS = 1
    Public Const VF_OMIT_BLOCKED_ORIENTATIONS_AND_CURVATURES = 2

' Features Matching
Public Const VFP_MATCHING_THRESHOLD = 200         ' Get/Set | Int | >= 0
Public Const VFP_MAXIMAL_ROTATION = 201           ' Get/Set | Int | DIR_0..DIR_180
Public Const VFP_MATCH_FEATURES = 210             ' Get/Set | Int | The values are the same as for the VFP_EXTRACT_FEATURES
Public Const VFP_MATCHING_SPEED = 220             ' Get/Set | Int | Can be one of the following
    Public Const VF_MATCHING_SPEED_LOW = 0
    Public Const VF_MATCHING_SPEED_HIGH = 256

' Features generalization
Public Const VFP_GENERALIZATION_THRESHOLD = 300   ' Get/Set | Int | >= 0

' VeriFinger specific
Public Const VFP_MODE = 1000                                   ' Get/Set | Int | Can be one of the following
    Public Const VF_MODE_GENERAL = 0
    Public Const VF_MODE_DIGITALPERSONA_URU = 100
    Public Const VF_MODE_BIOMETRIKA_FX2000 = 200
    Public Const VF_MODE_KEYTRONIC_SECUREDESKTOP = 300
    Public Const VF_MODE_IDENTIX_TOUCHVIEW = 400
    Public Const VF_MODE_PRECISEBIOMETRICS_100CS = 500
    Public Const VF_MODE_STMICROELECTRONICS_TOUCHCHIP = 600
    Public Const VF_MODE_IDENTICATORTECHNOLOGY_DF90 = 700
    Public Const VF_MODE_AUTHENTEC_AFS2 = 800
    Public Const VF_MODE_AUTHENTEC_AES4000 = 810
    Public Const VF_MODE_ATMEL_FINGERCHIP = 900
    Public Const VF_MODE_BMF_BLP100 = 1000
    Public Const VF_MODE_SECUGEN_HAMSTER = 1100
    Public Const VF_MODE_ETHENTICA = 1200
    Public Const VF_MODE_CROSSMATCH_VERIFIER300 = 1300

' Parameters functions
Public Declare Function VFGetParameterType Lib "vfvbp.dll" Alias "VBVFGetParameterType" (ByVal Parameter As Long) As Long
Public Declare Function VFGetParameter Lib "vfvbp.dll" Alias "VBVFGetParameter" (ByVal Parameter As Long, ByRef value As Variant, ByVal context As Long) As Long
Public Declare Function VFSetParameter Lib "vfvbp.dll" Alias "VBVFSetParameter" (ByVal Parameter As Long, ByVal value As Variant, ByVal context As Long) As Long

Public Declare Function VFGetParameterRaw Lib "vfvbp.dll" Alias "VBVFGetParameterRaw" (ByVal Parameter As Long, ByVal value As Any, ByVal context As Long) As Long
Public Declare Function VFSetParameterRaw Lib "vfvbp.dll" Alias "VBVFSetParameterRaw" (ByVal Parameter As Long, ByVal value As Any, ByVal context As Long) As Long

' -----------------------------------------------------------------------
' Fingerprint features
' -----------------------------------------------------------------------

Public Const VF_MAX_FEATURES_SIZE = 10000

Public Const VF_IMAGE_RESOLUTION = 500

Public Const VF_MIN_IMAGE_RESOLUTION = 50
Public Const VF_MAX_IMAGE_RESOLUTION = 5000

Public Const VF_MIN_IMAGE_DIMENSION = 16
Public Const VF_MAX_IMAGE_DIMENSION = 2048

' Features extraction function
Public Declare Function VFExtract Lib "vfvbp.dll" Alias "VBVFExtract" (ByVal width As Long, ByVal height As Long, Image As Variant, ByVal resolution As Long, features As Variant, ByRef Size As Long, ByVal context As Long) As Long

' -----------------------------------------------------------------------
' Identification
' -----------------------------------------------------------------------

Public Const VF_MAX_MINUTIA_COUNT = 1024

Public Const VF_MATCHDETAILS_SIZE = 20
Public Const VF_MATCHDETAILSEX_SIZE = 4120

Public Type VFMATCHDETAILS
    Size As Long ' DWORD
    similarity As Long ' INT
    rotation As Long ' INT
    trans_x As Long ' INT
    trans_y As Long ' INT
End Type

Public Type VFMATCHDETAILSEX
    Size As Long ' DWORD
    similarity As Long ' INT
    rotation As Long ' INT
    trans_x As Long ' INT
    trans_y As Long ' INT
    mm_count As Long ' INT
    mm(VF_MAX_MINUTIA_COUNT - 1, 1) As Integer ' SHORT
End Type

' Identification functions
Public Declare Function VFIdentifyStart Lib "vfvbp.dll" Alias "VBVFIdentifyStart" (test_features As Variant, ByVal context As Long) As Long
Public Declare Function VFIdentifyNext Lib "vfvbp.dll" Alias "VBVFIdentifyNext" (sample_features As Variant, md As Any, ByVal context As Long) As Long
Public Declare Function VFIdentifyEnd Lib "vfvbp.dll" Alias "VBVFIdentifyEnd" (ByVal context As Long) As Long

' -----------------------------------------------------------------------
' Verification
' -----------------------------------------------------------------------

' Verification function
Public Declare Function VFVerify Lib "vfvbp.dll" Alias "VBVFVerify" (features1 As Variant, features2 As Variant, md As Any, ByVal context As Long) As Long

' -----------------------------------------------------------------------
' Generalization
' -----------------------------------------------------------------------

Public Const VF_GENERALIZE_COUNT = 3

' Features generalization function
Public Declare Function VFGeneralize Lib "vfvbp.dll" Alias "VBVFGeneralize" (ByVal count As Long, gen_features As Variant, features As Variant, ByRef Size As Long, ByVal context As Long) As Long

' -----------------------------------------------------------------------
' Fingerprint features manipulation
' -----------------------------------------------------------------------

' Singular point types
Public Const VFFEAT_SP_DELTA = &H3
Public Const VFFEAT_SP_CORE = &H1
Public Const VFFEAT_SP_DOUBLECORE = &H2

' Directions
Public Const VFDIR_0 = 0
Public Const VFDIR_45 = 30
Public Const VFDIR_90 = (VFDIR_45 * 2)
Public Const VFDIR_135 = (VFDIR_45 * 3)
Public Const VFDIR_180 = (VFDIR_45 * 4)
Public Const VFDIR_225 = (VFDIR_45 * 5)
Public Const VFDIR_270 = (VFDIR_45 * 6)
Public Const VFDIR_315 = (VFDIR_45 * 7)
Public Const VFDIR_360 = (VFDIR_45 * 8)
Public Const VFDIR_UNKNOWN = 127
Public Const VFDIR_BACKGROUND = 255

' features structure definition

Public Type VFMINUTIAE
    count As Long ' int
    X(VF_MAX_MINUTIA_COUNT - 1) As Long ' int
    Y(VF_MAX_MINUTIA_COUNT - 1) As Long ' int
    D(VF_MAX_MINUTIA_COUNT - 1) As Byte ' BYTE
    C(VF_MAX_MINUTIA_COUNT - 1) As Byte ' BYTE
End Type

Public Const VF_MAX_SINGULAR_POINT_COUNT = 64

Public Type VFSINGULARPOINTS
    count As Long ' int
    X(VF_MAX_SINGULAR_POINT_COUNT - 1) As Long ' int
    Y(VF_MAX_SINGULAR_POINT_COUNT - 1) As Long ' int
    t(VF_MAX_SINGULAR_POINT_COUNT - 1) As Byte ' BYTE
    D(VF_MAX_SINGULAR_POINT_COUNT - 1) As Byte ' BYTE
End Type

Public Const VF_BLOCK_SIZE = 16

Public Const VF_MAX_BLOCKED_ORIENTS_DIMENSION = (VF_MAX_IMAGE_DIMENSION / VF_BLOCK_SIZE)

Public Type VFBLOCKEDORIENTS
    width As Long ' int
    height As Long ' int
    bits(VF_MAX_BLOCKED_ORIENTS_DIMENSION - 1, VF_MAX_BLOCKED_ORIENTS_DIMENSION - 1) As Byte ' BYTE
End Type

Public Type VFFEATURES
    G As Byte ' BYTE
    m As VFMINUTIAE
    sp As VFSINGULARPOINTS
    bo As VFBLOCKEDORIENTS
End Type

' Compression/decompression functions
Public Declare Function VFDecompressFeatures Lib "vfvbp.dll" Alias "VBVFDecompressFeatures" (features As Variant, structure As VFFEATURES) As Long
Public Declare Function VFCompressFeatures Lib "vfvbp.dll" Alias "VBVFCompressFeatures" (structure As VFFEATURES, features As Variant, ByRef Size As Long) As Long

' -----------------------------------------------------------------------
' Helper functions
' -----------------------------------------------------------------------

Public Const VF_PALLETE_GREEN = 0
Public Const VF_PALLETE_GRAY = 1

Public Declare Function ImageToHandle Lib "vfvbp.dll" Alias "VBImageToHandle" (ByVal width As Long, ByVal height As Long, Image As Variant, ByRef handle As Long, ByVal pallete As Long) As Long
Public Declare Function HandleToImage Lib "vfvbp.dll" Alias "VBHandleToImage" (ByVal handle As Long, ByRef width As Long, ByRef height As Long, Image As Variant) As Long
Public Declare Function LoadImageFromFile Lib "vfvbp.dll" Alias "VBLoadImageFromFile" (ByVal filename As String, ByRef width As Long, ByRef height As Long, Image As Variant) As Long
Public Declare Function SaveImageToFile Lib "vfvbp.dll" Alias "VBSaveImageToFile" (ByVal filename As String, ByVal width As Long, ByVal height As Long, Image As Variant) As Long

Public Const VF_DRAW_MINUTEA = 1
Public Const VF_DRAW_BLOCKED_ORIENTATIONS = 2
Public Const VF_DRAW_SINGULAR_POINTS = 4

Public Declare Function VBDrawFeatures Lib "vfvbp.dll" Alias "VBVFDrawFeatures" (structure As VFFEATURES, ByVal width As Long, ByVal height As Long, Image As Variant, ByVal pallete As Long, ByVal parameters As Long, ByRef handle As Long) As Long

' -----------------------------------------------------------------------
' Support functions
' -----------------------------------------------------------------------

Public Function VFGetG(features As Variant) As Byte
    VFGetG = features(LBound(features) + 1)
End Function

Public Function VFErrorToString(ByVal code As Long) As String
    Select Case code
        ' General
        Case VFE_OK
            VFErrorToString = "No error"
        Case VFE_FAILED
            VFErrorToString = "Failed!"
        Case VFE_OUT_OF_MEMORY
            VFErrorToString = "Out of memory"
        Case VFE_NOT_INITIALIZED
            VFErrorToString = "Not initialized"
        Case VFE_ARGUMENT_NULL
            VFErrorToString = "Argument NULL"
        Case VFE_INVALID_ARGUMENT
            VFErrorToString = "Invalid argument"
        ' Registration
        Case VFE_NOT_REGISTERED
            VFErrorToString = "Not registered"
        Case VFE_INVALID_SERIAL_NUMBER
            VFErrorToString = "Invalid serial number"
        Case VFE_INVALID_REGISTRATION_KEY
            VFErrorToString = "Invalid registration key"
        Case VFE_SCANNER_DRIVER_ERROR
            VFErrorToString = "Scanner driver error"
        Case VFE_REGISTRATION_NOT_NEEDED
            VFErrorToString = "Registration not needed"
        Case VFE_NO_SCANNER
            VFErrorToString = "No scanner"
        Case VFE_MORE_THAN_ONE_SCANNER
            VFErrorToString = "More than one scanner"
        Case VFE_LM_CONNECTION_ERROR
            VFErrorToString = "Communication with License Manager error"
        Case VFE_LM_NO_MORE_LICENSES
            VFErrorToString = "License Manager error: no more licenses"
        ' Parameters
        Case VFE_INVALID_PARAMETER
            VFErrorToString = "Invalid parameter"
        Case VFE_PARAMETER_READ_ONLY
            VFErrorToString = "Parameter read-only"
        ' Features extraction
        Case VFE_ILLEGAL_IMAGE_RESOLUTION
            VFErrorToString = "Illegal image resolution"
        Case VFE_ILLEGAL_IMAGE_SIZE
            VFErrorToString = "Illegal image size"
        Case VFE_LOW_QUALITY_IMAGE
            VFErrorToString = "Low quality image"
        ' VeriFinger specific
        Case VFE_INVALID_MODE
            VFErrorToString = "Invalid mode"
    End Select
End Function

Public Function VFResultToString(ByVal result As Long) As String
    If VFFailed(result) Then
        VFResultToString = VFErrorToString(result)
    Else
        If result = VFE_OK Then
            VFResultToString = "OK"
        Else
            VFResultToString = "OK (" & CStr(result) & ");"
        End If
    End If
End Function

Public Sub VFCheckResult(ByVal result As Long)
    If VFFailed(result) Then
        err.Raise result, "VeriFinger", "VeriFinger error:" & vbNewLine & vbNewLine & vbTab & "Code = " & CStr(result) & _
                          vbNewLine & vbNewLine & vbTab & " Description = " & VFErrorToString(result)
    End If
End Sub

Public Function VFFailed(ByVal result As Long) As Boolean
    VFFailed = result < 0
End Function

Public Function VFSucceeded(ByVal result As Long) As Boolean
    VFSucceeded = result >= 0
End Function

Public Function VFDirToRad(dir As Byte) As Double
    VFDirToRad = dir * 3.14 / VFDIR_180
End Function

Public Function VFRadToDir(a As Double) As Byte
    VFRadToDir = Round(a * VFDIR_180 / 3.14)
End Function

Public Function VFGetName() As String
    Dim str As Variant
    VFGetParameter VFP_NAME, str, VF_DEFAULT_CONTEXT
    VFGetName = str
End Function

Public Function VFGetVersionHigh() As Long
    Dim value As Variant
    VFGetParameter VFP_VERSION_HIGH, value, VF_DEFAULT_CONTEXT
    VFGetVersionHigh = value
End Function

Public Function VFGetVersionLow() As Long
    Dim value As Variant
    VFGetParameter VFP_VERSION_LOW, value, VF_DEFAULT_CONTEXT
    VFGetVersionLow = value
End Function

Public Function VFGetCopyright() As String
    Dim str As Variant
    VFGetParameter VFP_COPYRIGHT, str, VF_DEFAULT_CONTEXT
    VFGetCopyright = str
End Function

