unit VFinger;

interface

uses SysUtils, Math;

const
  VFingerDLL = 'VFinger.dll';

  { Constants }

  VF_IMAGE_RESOLUTION = 500;
  VF_FEATURES_RESOLUTION = 500;
  VF_BLOCK_SIZE = 16;
  VF_MAX_FEATURES_SIZE = 10000;

  VF_GENERALIZE_COUNT = 3;

  { Directions }

  VFDIR_0   =  0;
  VFDIR_45  = 30;
  VFDIR_90  = VFDIR_45 * 2;
  VFDIR_135 = VFDIR_45 * 3;
  VFDIR_180 = VFDIR_45 * 4;
  VFDIR_225 = VFDIR_45 * 5;
  VFDIR_270 = VFDIR_45 * 6;
  VFDIR_315 = VFDIR_45 * 7;
  VFDIR_360 = VFDIR_45 * 8;
  VFDIR_UNKNOWN = 127;
  VFDIR_BACKGROUND = 255;

  { Registration types }

  VF_RT_NOT_PROTECTED = 0;
  VF_RT_UNREGISTERED  = 6;
  VF_RT_HASP          = 1;
  VF_RT_PC            = 2;
  VF_RT_UAREU         = 4;
  VF_RT_LAN           = 8;

  { Parameter types }

  VF_TYPE_VOID   = 0;
  VF_TYPE_BYTE   = 1;
  VF_TYPE_SBYTE  = 2;
  VF_TYPE_WORD   = 3;
  VF_TYPE_SHORT  = 4;
  VF_TYPE_DWORD  = 5;
  VF_TYPE_INT    = 6;
  VF_TYPE_BOOL   = 10;
  VF_TYPE_CHAR   = 20;
  VF_TYPE_STRING = 100;

  { Error codes }

  // General
  VFE_OK               = 0;
  VFE_FAILED           = -1;
  VFE_OUT_OF_MEMORY    = -2;
  VFE_NOT_INITIALIZED  = -3;
  VFE_ARGUMENT_NULL    = -4;
  VFE_INVALID_ARGUMENT = -5;
  VFE_NOT_IMPLEMENTED  = -9;
  // Registration
  VFE_NOT_REGISTERED           = -2000;
  VFE_INVALID_SERIAL_NUMBER    = -2001;
  VFE_INVALID_REGISTRATION_KEY = -2002;
  VFE_SCANNER_DRIVER_ERROR     = -2003;
  VFE_REGISTRATION_NOT_NEEDED  = -2004;
  VFE_NO_SCANNER               = -2005;
  VFE_MORE_THAN_ONE_SCANNER    = -2006;
  VFE_LM_CONNECTION_ERROR      = -2007;
  VFE_LM_NO_MORE_LICENSES      = -2008;

  // Parameters
  VFE_INVALID_PARAMETER   = -10;
  VFE_PARAMETER_READ_ONLY = -11;
  // Features extraction
  VFE_ILLEGAL_IMAGE_RESOLUTION = -101;
  VFE_ILLEGAL_IMAGE_SIZE       = -102;
  VFE_LOW_QUALITY_IMAGE        = -103;
  // VeriFinger specific
  VFE_INVALID_MODE = -1000;
  // Features
  VFE_INVALID_FEATURES_FORMAT = -3000;

  { Parameters }

  // General
  VFP_TYPE         =  0; // Get | Pass parameter identifier ((DWORD)VFP_xxx) as value | Returns one of VF_TYPE_XXX constants
  VFP_NAME         = 10; // Get | String
  VFP_VERSION_HIGH = 11; // Get | DWord
  VFP_VERSION_LOW  = 12; // Get | DWord
  VFP_COPYRIGHT    = 13; // Get | String

  // Features extraction
  VFP_EXTRACT_FEATURES = 110; // Get/Set | Int | Obsolete. Can be one of the following
    VF_ALL_FEATURES                             = 0;
    VF_OMIT_BLOCKED_ORIENTATIONS                = 1;
    VF_OMIT_BLOCKED_ORIENTATIONS_AND_CURVATURES = 2;

  VFP_RETURNED_IMAGE = 10002; // Get/Set | Int | Can be one of the following
    VF_RETURNED_IMAGE_NONE         = 0;
    VF_RETURNED_IMAGE_BINARIZED    = 1;
    VF_RETURNED_IMAGE_SKELETONIZED = 2;

  // Features matching
  VFP_MATCHING_THRESHOLD = 200; // Get/Set | Int | >= 0
  VFP_MAXIMAL_ROTATION   = 201; // Get/Set | Int | DIR_0..DIR_180
  VFP_MATCH_FEATURES     = 210; // Get/Set | Int | Obsolete. The values are the same as for the VFP_EXTRACT_FEATURES
  VFP_MATCHING_SPEED     = 220; // Get/Set | Int | Can be one of the following
    VF_MATCHING_SPEED_LOW  = 0;
    VF_MATCHING_SPEED_HIGH = 256;

  // Features generalization
  VFP_GENERALIZATION_THRESHOLD = 300; // Get/Set | Int | >= 0
  VFP_GEN_MAXIMAL_ROTATION     = 301; // Get/Set | Int | DIR_0..DIR_180

  // VeriFinger specific
  VFP_MODE = 1000; // Get/Set | Int | Can be one of the following
    VF_MODE_GENERAL                      = 0;
    VF_MODE_DIGITALPERSONA_UAREU         = 100;
    VF_MODE_DIGITALPERSONA_URU           = VF_MODE_DIGITALPERSONA_UAREU;
    VF_MODE_BIOMETRIKA_FX2000            = 200;
    VF_MODE_KEYTRONIC_SECUREDESKTOP      = 300;
    VF_MODE_IDENTIX_TOUCHVIEW            = 400;
    VF_MODE_PRECISEBIOMETRICS_100CS      = 500;
    VF_MODE_STMICROELECTRONICS_TOUCHCHIP = 600;
    VF_MODE_IDENTICATORTECHNOLOGY_DF90   = 700;
    VF_MODE_AUTHENTEC_AFS2               = 800;
    VF_MODE_AUTHENTEC_AES4000            = 810;
    VF_MODE_ATMEL_FINGERCHIP             = 900;
    VF_MODE_BMF_BLP100                   = 1000;
    VF_MODE_SECUGEN_HAMSTER              = 1100;
    VF_MODE_ETHENTICA                    = 1200;
    VF_MODE_CROSSMATCH_VERIFIER300       = 1300;

type
  PPByte = ^PByte;

  HVFCONTEXT = Pointer;

  { Features }

  TVFSingularPointType = (vfsptUnknown = 0, vfsptCore = 1, vfsptDoubleCore = 2, vfsptDelta = 3);

  PVFSingularPoint = ^TVFSingularPoint;
  TVFSingularPoint = record
    X: Integer;
    Y: Integer;
    T: TVFSingularPointType;
    Dummy1, Dummy2, Dummy3: Byte; // because enumeration have to be 4 bytes
    D: Byte;
  end;

  TVFMinutiaType = (vfmtUnknown = 0, vfmtEnd = 1, vfmtBifurcation = 2);

  PVFMinutia = ^TVFMinutia;
  TVFMinutia = record
    X: Integer;
    Y: Integer;
    T: TVFMinutiaType;
    Dummy1, Dummy2, Dummy3: Byte; // because enumeration have to be 4 bytes
    D: Byte;
    C: Byte;
    G: Byte;
  end;

  { Match details }

  PVFMatchDetails = ^TVFMatchDetails;
  TVFMatchDetails = record
    Size: LongWord;

    Similarity: Integer;
    Rotation: Integer;
    TranslationX: Integer;
    TranslationY: Integer;
  end;

const
  VF_MAX_MATCHED_MINUTIA_COUNT = 1024;

type
  PVFMatchedMinutiae = ^TVFMatchedMinutiae;
  TVFMatchedMinutiae = record
    Count: Integer;
    Test: array[0..VF_MAX_MATCHED_MINUTIA_COUNT - 1] of SmallInt;
    Sample: array[0..VF_MAX_MATCHED_MINUTIA_COUNT - 1] of SmallInt;
  end;

  PVFMatchDetailsEx = ^TVFMatchDetailsEx;
  TVFMatchDetailsEx = record
    Size: LongWord;

    Similarity: Integer;
    Rotation: Integer;
    TranslationX: Integer;
    TranslationY: Integer;

    MM: TVFMatchedMinutiae;
  end;

{ VeriFinger library functions }

// Registration
function VFRegistrationType: Integer; stdcall;
function VFGenerateId(Serial, Id: PChar): Integer; stdcall;
function VFRegister(Serial, Key: PChar): Integer; stdcall;

// Initialization
function VFInitialize: Integer; stdcall;
function VFFinalize: Integer; stdcall;

// Contexts
function VFCreateContext: HVFCONTEXT; stdcall;
function VFFreeContext(Context: HVFCONTEXT): Integer; stdcall;

// Parameters
function VFGetParameter(Parameter: Integer; Value: Pointer; Context: HVFCONTEXT): Integer; stdcall;
function VFSetParameter(Parameter: Integer; Value: LongWord; Context: HVFCONTEXT): Integer; stdcall;

// Features extraction
function VFExtract(Width, Height: Integer; Image: PByte; Resolution: Integer; Features: PByte; var Size: LongWord; Context: HVFCONTEXT): Integer; stdcall;

// Features generalization
function VFGeneralize(Count: Integer; GenFeatures: PPByte; Features: PByte; var Size: LongWord; Context: HVFCONTEXT): Integer; stdcall;

// Verification
function VFVerify(Features1, Features2: PByte; MD: PVFMatchDetails; Context: HVFCONTEXT): Integer; stdcall;

// Identification
function VFIdentifyStart(TestFeatures: PByte; Context: HVFCONTEXT): Integer; stdcall;
function VFIdentifyNext(SampleFeatures: PByte; MD: PVFMatchDetails; Context: HVFCONTEXT): Integer; stdcall;
function VFIdentifyEnd(Context: HVFCONTEXT): Integer; stdcall;

// Features compression
function VFFeatSet(G: Byte; MCount: Integer; M: PVFMinutia;
	SPCount: Integer; SP: PVFSingularPoint; BOWidth, BOHeight: Integer; BO: PByte;
	Features: PByte): Integer; stdcall;

// Features decompression
function VFFeatGetG(Features: PByte): Integer; stdcall;
function VFFeatGetMinutiaCount(Features: PByte): Integer; stdcall;
function VFFeatGetMinutiae(Features: PByte; M: PVFMinutia): Integer; stdcall;
function VFFeatGetSPCount(Features: PByte): Integer; stdcall;
function VFFeatGetSP(Features: PByte; SP: PVFSingularPoint): Integer; stdcall;
function VFFeatGetBOSize(Features: PByte; var Width, Height: Integer): Integer; stdcall;
function VFFeatGetBO(Features: PByte; BO: PByte): Integer; stdcall;

{ Error and result functions }

function VFSucceeded(Res: Integer): Boolean;
function VFFailed(Res: Integer): Boolean;

function VFErrorToString(Err: Integer): string;
function VFResultToString(Res: Integer): string;

procedure VFRaiseError(Err: Integer);
procedure VFCheckResult(Res: Integer);

{ Directions functions }

// Conversion from VFDIR_XXX to degrees and vice a versa
function VFDirToDeg(Dir: Byte): Integer;
function VFDirToDegF(Dir: Byte): Double;
function VFDegToDir(Deg: Integer): Byte;
// Conversion from VFDIR_XXX to radians and vice a versa
function VFDirToRad(Dir: Byte): Double;
function VFRadToDir(A: Double): Byte;
// Orientation stuff
function VFIsBadArea(Orient: Byte): Boolean;
function VFIsGoodArea(Orient: Byte): Boolean;
function VFTheOrient(Orient: Byte): Byte;
function VFIsUnknown(Orient: Byte): Boolean;
function VFIsOrient(Orient: Byte): Boolean;

{ Parameters functions }

function VFParameterOfType(Parameter, ParameterType: Integer): Boolean;
procedure VFCheckParameterType(Parameter, ParameterType: Integer);

// Get
function VFGetByteParameter(Parameter: Integer; Context: HVFCONTEXT = nil): Byte;
function VFGetSByteParameter(Parameter: Integer; Context: HVFCONTEXT = nil): ShortInt;
function VFGetWordParameter(Parameter: Integer; Context: HVFCONTEXT = nil): Word;
function VFGetShortParameter(Parameter: Integer; Context: HVFCONTEXT = nil): SmallInt;
function VFGetDWordParameter(Parameter: Integer; Context: HVFCONTEXT = nil): LongWord;
function VFGetIntParameter(Parameter: Integer; Context: HVFCONTEXT = nil): Integer;
function VFGetBoolParameter(Parameter: Integer; Context: HVFCONTEXT = nil): Boolean;
function VFGetCharParameter(Parameter: Integer; Context: HVFCONTEXT = nil): Char;
function VFGetStringParameter(Parameter: Integer; Context: HVFCONTEXT = nil): string;

// Set
procedure VFSetByteParameter(Parameter: Integer; Value: Byte; Context: HVFCONTEXT = nil);
procedure VFSetSByteParameter(Parameter: Integer; Value: ShortInt; Context: HVFCONTEXT = nil);
procedure VFSetWordParameter(Parameter: Integer; Value: Word; Context: HVFCONTEXT = nil);
procedure VFSetShortParameter(Parameter: Integer; Value: SmallInt; Context: HVFCONTEXT = nil);
procedure VFSetDWordParameter(Parameter: Integer; Value: LongWord; Context: HVFCONTEXT = nil);
procedure VFSetIntParameter(Parameter: Integer; Value: Integer; Context: HVFCONTEXT = nil);
procedure VFSetBoolParameter(Parameter: Integer; Value: Boolean; Context: HVFCONTEXT = nil);
procedure VFSetCharParameter(Parameter: Integer; Value: Char; Context: HVFCONTEXT = nil);
procedure VFSetStringParameter(Parameter: Integer; Value: string; Context: HVFCONTEXT = nil);

// Get general
function VFGetParameterType(Parameter: Integer): Integer;
function VFGetName: string;
function VFGetVersionHigh: LongWord;
function VFGetVersionLow: LongWord;
function VFGetCopyright: string;

{ FAR and matching threshold functions }

function VFMatchingThresholdToFAR(Th: Integer): Double;
function VFFARToMatchingThreshold(F: Double): Integer;

implementation

const
  ThFARLogRatio: Double = 12;


{ VeriFinger library functions }

function VFRegistrationType; external VFingerDLL;
function VFGenerateId; external VFingerDLL;
function VFRegister; external VFingerDLL;
function VFInitialize; external VFingerDLL;
function VFFinalize;  external VFingerDLL;
function VFCreateContext; external VFingerDLL;
function VFFreeContext; external VFingerDLL;
function VFGetParameter; external VFingerDLL;
function VFSetParameter; external VFingerDLL;
function VFExtract; external VFingerDLL;
function VFGeneralize; external VFingerDLL;
function VFVerify; external VFingerDLL;
function VFIdentifyStart; external VFingerDLL;
function VFIdentifyNext; external VFingerDLL;
function VFIdentifyEnd; external VFingerDLL;
function VFFeatSet; external VFingerDLL;
function VFFeatGetG; external VFingerDLL;
function VFFeatGetMinutiaCount; external VFingerDLL;
function VFFeatGetMinutiae; external VFingerDLL;
function VFFeatGetSPCount; external VFingerDLL;
function VFFeatGetSP; external VFingerDLL;
function VFFeatGetBOSize; external VFingerDLL;
function VFFeatGetBO; external VFingerDLL;

{ Error and result functions }

function VFSucceeded(Res: Integer): Boolean;
begin
  Result := Res >= 0;
end;

function VFFailed(Res: Integer): Boolean;
begin
  Result := Res < 0;
end;

function VFErrorToString(Err: Integer): string;
begin
  if Err >= 0 then Result := EmptyStr
  else
    case Err of
      VFE_FAILED: Result := 'Failed';
      VFE_OUT_OF_MEMORY: Result := 'Out of memory';
      VFE_NOT_INITIALIZED: Result := 'Not initialized';
      VFE_ARGUMENT_NULL: Result := 'Argument null';
      VFE_INVALID_ARGUMENT: Result := 'Invalid argument';
      VFE_NOT_IMPLEMENTED: Result := 'Not implemented';
      VFE_NOT_REGISTERED: Result := 'Not registered';
      VFE_INVALID_SERIAL_NUMBER: Result := 'Invalid serial number';
      VFE_INVALID_REGISTRATION_KEY: Result := 'Invalid registration key';
      VFE_SCANNER_DRIVER_ERROR: Result := 'Scanner driver error';
      VFE_REGISTRATION_NOT_NEEDED: Result := 'Registration not needed';
      VFE_NO_SCANNER: Result := 'No scanner connected';
      VFE_MORE_THAN_ONE_SCANNER: Result := 'More than one scanner connected';
      VFE_LM_CONNECTION_ERROR: Result := 'Communication with License Manager error';
      VFE_LM_NO_MORE_LICENSES: Result := 'License Manager error: no more licenses';
      VFE_INVALID_PARAMETER: Result := 'Invalid parameter';
      VFE_PARAMETER_READ_ONLY: Result := 'Parameter is read only';
      VFE_ILLEGAL_IMAGE_RESOLUTION: Result := 'Illegal image resolution';
      VFE_ILLEGAL_IMAGE_SIZE: Result := 'Illegal image size';
      VFE_LOW_QUALITY_IMAGE: Result := 'Low quality image';
      VFE_INVALID_MODE: Result := 'Invalid mode';
      VFE_INVALID_FEATURES_FORMAT: Result := 'Invalid features format';
    else Result := IntToStr(Err);
    end;
end;

function VFResultToString(Res: Integer): string;
begin
  if VFFailed(Res) then Result := VFErrorToString(Res)
  else if Res = VFE_OK then Result := 'OK' else Result := Format('OK (%d)', [Res]);
end;

procedure VFRaiseError(Err: Integer);
begin
  raise Exception.CreateFmt('VeriFinger error:'#13#9'Code = %d'#13#9'Description = %s',
    [Err, VFErrorToString(Err)]);
end;

procedure VFCheckResult(Res: Integer);
begin
  if VFFailed(Res) then VFRaiseError(Res);
end;

{ Directions functions }

function VFDirToDeg(Dir: Byte): Integer;
begin
  Result := (Dir * 180 + VFDIR_90) div VFDIR_180;
end;

function VFDirToDegF(Dir: Byte): Double;
begin
  Result := Dir * 180 / VFDIR_180;
end;

function VFDegToDir(Deg: Integer): Byte;
var
  P: Integer;
begin
  if Deg < 0 then P := -90
  else P := 90;
 Result := (Deg * VFDIR_180 + P) div 180;
end;

function VFDirToRad(Dir: Byte): Double;
begin
  Result := Dir * Pi / VFDIR_180;
end;

function VFRadToDir(A: Double): Byte;
begin
  Result := Round(A * VFDIR_180 / PI);
end;

function VFIsBadArea(Orient: Byte): Boolean;
begin
  Result := Orient and 128 <> 0;
end;

function VFIsGoodArea(Orient: Byte): Boolean;
begin
  Result := Orient and 128 = 0;
end;

function VFTheOrient(Orient: Byte): Byte;
begin
  Result := Orient and 127;
end;

function VFIsUnknown(Orient: Byte): Boolean;
begin
  Result := VFTheOrient(Orient) = VFDIR_UNKNOWN;
end;

function VFIsOrient(Orient: Byte): Boolean;
begin
  Result := VFTheOrient(Orient) < VFDIR_180;
end;

{ Parameters functions }

function VFParameterOfType(Parameter, ParameterType: Integer): Boolean;
begin
  Result := VFGetParameterType(Parameter) = ParameterType;
end;

procedure VFCheckParameterType(Parameter, ParameterType: Integer);
begin
  if not VFParameterOfType(Parameter, ParameterType) then
    raise Exception.Create('VeriFinger parameter is of wrong type');
end;

// Get

function VFGetByteParameter(Parameter: Integer; Context: HVFCONTEXT = nil): Byte;
begin
  VFCheckParameterType(Parameter, VF_TYPE_BYTE);
  VFCheckResult(VFGetParameter(Parameter, @Result, Context));
end;

function VFGetSByteParameter(Parameter: Integer; Context: HVFCONTEXT = nil): ShortInt;
begin
  VFCheckParameterType(Parameter, VF_TYPE_SBYTE);
  VFCheckResult(VFGetParameter(Parameter, @Result, Context));
end;

function VFGetWordParameter(Parameter: Integer; Context: HVFCONTEXT = nil): Word;
begin
  VFCheckParameterType(Parameter, VF_TYPE_WORD);
  VFCheckResult(VFGetParameter(Parameter, @Result, Context));
end;

function VFGetShortParameter(Parameter: Integer; Context: HVFCONTEXT = nil): SmallInt;
begin
  VFCheckParameterType(Parameter, VF_TYPE_SHORT);
  VFCheckResult(VFGetParameter(Parameter, @Result, Context));
end;

function VFGetDWordParameter(Parameter: Integer; Context: HVFCONTEXT = nil): LongWord;
begin
  VFCheckParameterType(Parameter, VF_TYPE_DWORD);
  VFCheckResult(VFGetParameter(Parameter, @Result, Context));
end;

function VFGetIntParameter(Parameter: Integer; Context: HVFCONTEXT = nil): Integer;
begin
  VFCheckParameterType(Parameter, VF_TYPE_INT);
  VFCheckResult(VFGetParameter(Parameter, @Result, Context));
end;

function VFGetBoolParameter(Parameter: Integer; Context: HVFCONTEXT = nil): Boolean;
var
  Value: LongBool;
begin
  VFCheckParameterType(Parameter, VF_TYPE_BOOL);
  VFCheckResult(VFGetParameter(Parameter, @Value, Context));
  Result := Value;
end;

function VFGetCharParameter(Parameter: Integer; Context: HVFCONTEXT = nil): Char;
var
  Value: AnsiChar;
begin
  VFCheckParameterType(Parameter, VF_TYPE_CHAR);
  VFCheckResult(VFGetParameter(Parameter, @Value, Context));
  Result := Value;
end;

function VFGetStringParameter(Parameter: Integer; Context: HVFCONTEXT = nil): string;
var
  L: Integer;
  Value: AnsiString;
begin
  VFCheckParameterType(Parameter, VF_TYPE_STRING);
  L := VFGetParameter(Parameter, nil, Context);
  VFCheckResult(L);
  SetLength(Value, L);
  VFCheckResult(VFGetParameter(Parameter, PAnsiChar(Value), Context));
  Result := Value;
end;

// Set

procedure VFSetByteParameter(Parameter: Integer; Value: Byte; Context: HVFCONTEXT = nil);
begin
  VFCheckParameterType(Parameter, VF_TYPE_BYTE);
  VFCheckResult(VFSetParameter(Parameter, LongWord(Value), Context));
end;

procedure VFSetSByteParameter(Parameter: Integer; Value: ShortInt; Context: HVFCONTEXT = nil);
begin
  VFCheckParameterType(Parameter, VF_TYPE_SBYTE);
  VFCheckResult(VFSetParameter(Parameter, LongWord(Value), Context));
end;

procedure VFSetWordParameter(Parameter: Integer; Value: Word; Context: HVFCONTEXT = nil);
begin
  VFCheckParameterType(Parameter, VF_TYPE_WORD);
  VFCheckResult(VFSetParameter(Parameter, LongWord(Value), Context));
end;

procedure VFSetShortParameter(Parameter: Integer; Value: SmallInt; Context: HVFCONTEXT = nil);
begin
  VFCheckParameterType(Parameter, VF_TYPE_SHORT);
  VFCheckResult(VFSetParameter(Parameter, LongWord(Value), Context));
end;

procedure VFSetDWordParameter(Parameter: Integer; Value: LongWord; Context: HVFCONTEXT = nil);
begin
  VFCheckParameterType(Parameter, VF_TYPE_DWORD);
  VFCheckResult(VFSetParameter(Parameter, Value, Context));
end;

procedure VFSetIntParameter(Parameter: Integer; Value: Integer; Context: HVFCONTEXT = nil);
begin
  VFCheckParameterType(Parameter, VF_TYPE_INT);
  VFCheckResult(VFSetParameter(Parameter, LongWord(Value), Context));
end;

procedure VFSetBoolParameter(Parameter: Integer; Value: Boolean; Context: HVFCONTEXT = nil);
begin
  VFCheckParameterType(Parameter, VF_TYPE_BOOL);
  VFCheckResult(VFSetParameter(Parameter, LongWord(LongBool(Value)), Context));
end;

procedure VFSetCharParameter(Parameter: Integer; Value: Char; Context: HVFCONTEXT = nil);
begin
  VFCheckParameterType(Parameter, VF_TYPE_CHAR);
  VFCheckResult(VFSetParameter(Parameter, LongWord(AnsiChar(Value)), Context));
end;

procedure VFSetStringParameter(Parameter: Integer; Value: string; Context: HVFCONTEXT = nil);
var
  V: AnsiString;
begin
  VFCheckParameterType(Parameter, VF_TYPE_STRING);
  V := Value;
  VFCheckResult(VFSetParameter(Parameter, LongWord(PAnsiChar(V)), Context));
end;

// Get general

function VFGetParameterType(Parameter: Integer): Integer;
begin
  Result := VFGetParameter(VFP_TYPE, Pointer(Parameter), nil);
  VFCheckResult(Result);
end;

function VFGetName: string;
begin
  Result := VFGetStringParameter(VFP_NAME);
end;

function VFGetVersionHigh: LongWord;
begin
  Result := VFGetDWordParameter(VFP_VERSION_HIGH);
end;

function VFGetVersionLow: LongWord;
begin
  Result := VFGetDWordParameter(VFP_VERSION_LOW);
end;

function VFGetCopyright: string;
begin
  Result := VFGetStringParameter(VFP_COPYRIGHT);
end;

{ FAR and matching threshold functions }

function VFMatchingThresholdToFAR(Th: Integer): Double;
begin
  if Th < 0 then Th := 0;
  Result := Power(10.0, -Th / ThFARLogRatio + 2);
end;

function VFFARToMatchingThreshold(F: Double): Integer;
begin
  if F > 100.0 then F := 100.0
  else if F <= 0.0 then F := 1E-100;
  Result := Round((-Log10(F) + 2) * ThFARLogRatio);
end;

end.

