#include "StdAfx.h"
#include "VFImage.h"


//////////////
//          //
// CVFImage //
//          //
//////////////

CVFImage::CVFImage()
	: width(0), height(0), size(0), bits(NULL)
{
}

CVFImage::CVFImage(int width, int height)
	: width(0), height(0), size(0), bits(NULL)
{
	SetSize(width, height);
}

CVFImage::~CVFImage()
{
	Clear();
}

void CVFImage::Clear()
{
	if (!IsEmpty())
	{
		width = 0;
		height = 0;
		size = 0;
		free(bits);
		bits = NULL;
	}
}

void CVFImage::SetSize(int width, int height)
{
	if (width <= 0 || height <= 0) { width = 0; height = 0; }
	DWORD size = width * height;
	if(size != this->size)
	{
		bits = (BYTE*)realloc(bits, size);
		this->size = size;
	}
	this->width = width;
	this->height = height;
}

BYTE CVFImage::GetPixel(int x, int y)
{
	CheckPixelIndex(x, y);
	return bits[y * width + x];
}

void CVFImage::SetPixel(int x, int y, BYTE value)
{
	CheckPixelIndex(x, y);
	bits[y * width + x] = value;
}

void CVFImage::Assign(const CVFImage * image)
{
	Assign(image->width, image->height, image->bits);
}

void CVFImage::Assign(int width, int height, const BYTE * bits)
{
	SetSize(width, height);
	memcpy(this->bits, bits, size);
}

HBITMAP CVFImage::GetBitmap(COLORREF color_min, COLORREF color_max)
{
	if (IsEmpty()) return NULL;
	BITMAPINFO *bi = (BITMAPINFO*)malloc(sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD) * 256);
	bi->bmiHeader.biBitCount = 8;
	bi->bmiHeader.biClrImportant = 0;
	bi->bmiHeader.biClrUsed = 0;
	bi->bmiHeader.biCompression = BI_RGB;
	bi->bmiHeader.biPlanes = 1;
	bi->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	bi->bmiHeader.biXPelsPerMeter = 0;
	bi->bmiHeader.biYPelsPerMeter = 0;
	bi->bmiHeader.biHeight = height;
	bi->bmiHeader.biWidth = width;
	int blue_min = GetBValue(color_min);
	int green_min = GetGValue(color_min);
	int red_min = GetRValue(color_min);
	int blue_max = GetBValue(color_max);
	int green_max = GetGValue(color_max);
	int red_max = GetRValue(color_max);
	int blue_delta = blue_max - blue_min;
	int green_delta = green_max - green_min;
	int red_delta = red_max - red_min;
	for (int i = 0; i < 256; i++)
	{
		bi->bmiColors[i].rgbBlue = (BYTE)(i * blue_delta / 255 + blue_min);
		bi->bmiColors[i].rgbGreen = (BYTE)(i * green_delta / 255 + green_min);
		bi->bmiColors[i].rgbRed = (BYTE)(i * red_delta / 255 + red_min);
		bi->bmiColors[i].rgbReserved = 0;
	}
	BYTE *bits;
	HBITMAP bmp = CreateDIBSection(NULL, bi, DIB_RGB_COLORS, (VOID**)&bits, NULL, 0);
	if (bmp)
	{
		DWORD width_bytes = (width / 4 + (width % 4 ? 1 : 0)) * 4;
		BYTE *img = this->bits + (height - 1) * width;
		for(int i = 0; i < height; i++, bits += width_bytes, img -= width)
			memcpy(bits, img, width);
	}
	free(bi);
	return bmp;
}

HBITMAP CVFImage::GetBitmap(int bitmapWidth, int bitmapHeight, COLORREF color_min, COLORREF color_max)
{
	if (IsEmpty()) return NULL;
	BITMAPINFO *bi = (BITMAPINFO*)malloc(sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD) * 256);
	bi->bmiHeader.biBitCount = 8;
	bi->bmiHeader.biClrImportant = 0;
	bi->bmiHeader.biClrUsed = 0;
	bi->bmiHeader.biCompression = BI_RGB;
	bi->bmiHeader.biPlanes = 1;
	bi->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	bi->bmiHeader.biXPelsPerMeter = 0;
	bi->bmiHeader.biYPelsPerMeter = 0;
	bi->bmiHeader.biHeight = bitmapHeight;
	bi->bmiHeader.biWidth = bitmapWidth;
	int blue_min = GetBValue(color_min);
	int green_min = GetGValue(color_min);
	int red_min = GetRValue(color_min);
	int blue_max = GetBValue(color_max);
	int green_max = GetGValue(color_max);
	int red_max = GetRValue(color_max);
	int blue_delta = blue_max - blue_min;
	int green_delta = green_max - green_min;
	int red_delta = red_max - red_min;
	for (int i = 0; i < 256; i++)
	{
		bi->bmiColors[i].rgbBlue = (BYTE)(i * blue_delta / 255 + blue_min);
		bi->bmiColors[i].rgbGreen = (BYTE)(i * green_delta / 255 + green_min);
		bi->bmiColors[i].rgbRed = (BYTE)(i * red_delta / 255 + red_min);
		bi->bmiColors[i].rgbReserved = 0;
	}
	BYTE *bits;
	HBITMAP bmp = CreateDIBSection(NULL, bi, DIB_RGB_COLORS, (VOID**)&bits, NULL, 0);
	if (bmp)
	{
		DWORD widthBytes = (bitmapWidth / 4 + (bitmapWidth % 4 ? 1 : 0)) * 4;
		bits += (bitmapHeight - 1) * widthBytes;
		for (int i = 0, ii; i < bitmapHeight; i++, bits -= widthBytes)
		{
			BYTE *p = bits;
			ii = (i * height / bitmapHeight) * width;
			for (int j = 0, jj; j < bitmapWidth; j++, p++)
			{
				jj = j * width / bitmapWidth;
				*p = *(this->bits + ii + jj);
			}
		}
	}
	free(bi);
	return bmp;
}
