#include "System.h"
#include "FileListForm.h"
#include "VFImage.h"
#include "Utils.h"
#include "FileListEntryForm.h"

///////////////////
//               //
// CFileListForm //
//               //
///////////////////

CFileListForm::CFileListForm()
	: CForm(), filelist(NULL), modified(false), resolution(VF_IMAGE_RESOLUTION), finger_id(default_file_finger_id),
	changing(false)
{
	SetMinimizeBox(false);
	SetMaximizeBox(false);
	SetToolWindow(true);
	SetText(STRING("File list editor"));
	SetSize(500, 400);
	SetFormPosition(fpOwnerCenter);

	open_dialog = new COpenFileDialog();
	open_dialog->Filter = GetLoadVFFileListFilterString();
	save_dialog = new CSaveFileDialog();
	save_dialog->Filter = GetSaveVFFileListFilterString();
	save_dialog->DefaultExtension = GetExtensionFromFileFilter(save_dialog->Filter);
	add_dialog = new COpenFileDialog();
	add_dialog->Title = TEXT("Add");
	add_dialog->Filter = CVFImageIO::GetLoadFileFilterString();
	add_dialog->AllowMultiSelect = true;

	listview = new CListView();
	listview->SetSingleSel(false);
	listview->SetFullRowSelect(true);
	listview->SetNoSortHeader(true);
	listview->SetShowSelAlways(true);
	listview->SetOwnerData(true);
	listview->AddColumn(STRING("Finger ID"), 130);
	listview->AddColumn(STRING("Resolution"), 70, taRight);
	listview->AddColumn(STRING("Filename"), 260);
	
	statusbar = new CStatusBar();
	statusbar->AddPart(60);
	statusbar->AddPart(-1);

	AddControl(listview);
	AddControl(statusbar);

	SetActiveControl(listview);
}

CFileListForm::~CFileListForm()
{
	delete add_dialog;
	delete save_dialog;
	delete open_dialog;
}

LRESULT CFileListForm::OnSize(DWORD resizing_type, WORD client_width, WORD client_height)
{
	if(resizing_type == SIZE_RESTORED)
	{
		statusbar->SendMessage(WM_SIZE, 0, 0);
		listview->SetSize(client_width, client_height - statusbar->Height());
		return 0;
	}
	else return CForm::OnSize(resizing_type, client_width, client_height);
}

void CFileListForm::OnCommand(WORD id, WORD event, CControl *control)
{
	if(!control)
		switch(id)
		{
			case ID_FILE_NEW: OnFileNew(); break;
			case ID_FILE_OPEN: OnFileOpen(); break;
			case ID_FILE_SAVE: OnFileSave(); break;
			case ID_FILE_ADD: OnFileAdd(); break;
			case ID_FILE_EXIT: OnFileExit(); break;
			case ID_EDIT_ADD: OnEditAdd(); break;
			case ID_EDIT_ADDDIRECTORY: OnEditAddDirectory(); break;
			case ID_EDIT_NEWENTRY: OnEditNewEntry(); break;
			case ID_EDIT_EDITENTRY: OnEditEditEntry(); break;
			case ID_EDIT_DELETE: OnEditDelete(); break;
			case ID_EDIT_MOVEUP: OnEditMoveUp(); break;
			case ID_EDIT_MOVEDOWN: OnEditMoveDown(); break;
			case ID_EDIT_CLEARFINGERID: OnEditClearFingerID(); break;
			case ID_EDIT_SELECTALL: OnEditSelectAll(); break;
			case ID_EDIT_SELECTNONE: OnEditSelectNone(); break;
			case ID_EDIT_GOTO: OnEditGoTo(); break;
			default: CForm::OnCommand(id, event, control);
		}
}

LRESULT CFileListForm::OnNotify(LPNMHDR nmh, CControl *control)
{
	if(control)
	{
		if(control == listview)
		{
			if(nmh->code == LVN_GETDISPINFO)
			{
				NMLVDISPINFO *di = (NMLVDISPINFO*)nmh;
				if(di->item.mask & LVIF_TEXT)
				{
					di->item.mask = LVIF_TEXT;
					VFFileListItem &item = (*filelist)[di->item.iItem];
					string s;
					switch(di->item.iSubItem)
					{
						case 0: s = item.finger_id; break;
						case 1: s = IntToString(item.resolution); break;
						case 2: s = item.filename; break;
					}
					s.Copy(di->item.pszText, 0, di->item.cchTextMax);
					return 0;
				}
			}
			else if(nmh->code == LVN_ITEMCHANGED)
			{
				if(!changing)
				{
					NMLISTVIEW * lw = (NMLISTVIEW *)nmh;
					if((lw->uChanged & LVIF_STATE) && ((lw->uNewState ^ lw->uOldState) & LVIS_SELECTED))
					{
						UpdateSelection();
						return 0;
					}
				}
			}
			else if(nmh->code == LVN_ODSTATECHANGED)
			{
				NMLVODSTATECHANGE * sc = (NMLVODSTATECHANGE *)nmh;
				if((sc->uNewState ^ sc->uOldState) & LVIS_SELECTED)
				{
					UpdateSelection();
					return 0;
				}
			}
		}
	}
	return CForm::OnNotify(nmh, control);
}

void CFileListForm::UpdateFileList()
{
	listview->SetItemCount(filelist->Length());
}

bool CFileListForm::ClearPrompt()
{
	if(modified)
		switch(MessageBox(this, STRING("File list modified. Save changes?"), Text(), MB_YESNOCANCEL | MB_ICONQUESTION))
		{
			case IDYES: return OnFileSave();
			case IDNO: return true;
			default: return false;
		}
	else return true;
}

void CFileListForm::OnFileNew()
{
	if(ClearPrompt())
	{
		filelist->Clear();
		listview->ClearItems();
		modified = false;
		UpdateSelection();
	}
}

void CFileListForm::OnFileOpen()
{
	if(ClearPrompt())
	{
		open_dialog->FileName.Clear();
		if(open_dialog->Show(this) == IDOK)
		{
			CWaitCursor * waitCursor = new CWaitCursor();
			bool ok = LoadVFFileList(open_dialog->FileName, filelist);
			if(ok)
			{
				UpdateFileList();
				modified = false;
				UpdateSelection();
			}
			delete waitCursor;
			if(!ok) MessageBox(this, Format(TEXT("Can not load from file \"%s\""), (LPCTSTR)open_dialog->FileName), Text(), MB_OK | MB_ICONERROR);
		}
	}
}

bool CFileListForm::OnFileSave()
{
	save_dialog->FileName.Clear();
	if(save_dialog->Show(this) == IDOK)
	{
		CWaitCursor * waitCursor = new CWaitCursor();
		bool ok = SaveVFFileList(save_dialog->FileName, filelist);
		if(ok)
		{
			modified = false;
			UpdateSelection();
		}
		delete waitCursor;
		if(!ok) MessageBox(this, Format(TEXT("Can not save to file \"%s\""), (LPCTSTR)save_dialog->FileName), Text(), MB_OK | MB_ICONERROR);
		return ok;
	}
	else return false;
}

void CFileListForm::OnFileAdd()
{
	open_dialog->FileName.Clear();
	open_dialog->Title = "Add file list";
	if(open_dialog->Show(this) == IDOK)
	{
		CWaitCursor * waitCursor = new CWaitCursor();
		VFFileList fl;
		bool ok = LoadVFFileList(open_dialog->FileName, &fl);
		if(ok)
		{
			filelist->Add(fl);
			UpdateFileList();
			UpdateSelection();
		}
		delete waitCursor;
		if(!ok) MessageBox(this, Format(TEXT("Can not add from file \"%s\""), (LPCTSTR)open_dialog->FileName), Text(), MB_OK | MB_ICONERROR);
	}
	open_dialog->Title.Clear();
}

void CFileListForm::OnFileExit()
{
	Close();
}

void CFileListForm::OnEditAdd()
{
	add_dialog->FileName.Clear();
	if(add_dialog->Show(this) == IDOK)
	{
		string filename = add_dialog->FileName;
		DWORD flags = FLEB_NEW_ENTRY;
		SetFlag(flags, FLEB_MANY_ENTRIES, add_dialog->FileNames.Length() > 1);
		if(FileListEntryBox(this, flags, filename, resolution, finger_id))
		{
			CWaitCursor waitCursor;
			listview->SetRedrawState(false);
			for(int i = 0; i < add_dialog->FileNames.Length(); i++)
				AddFile(add_dialog->FileNames[i]);
			modified = true;
			UpdateSelection();
			listview->SetRedrawState(true);
		}
	}
}

void CFileListForm::OnEditAddDirectory()
{
	string path;
	string filter = CVFImageIO::LoadFileFilter();
	if(BrowseForFolderBox(this, string(TEXT("Files with ") + filter + TEXT(" extensions from this folder and all its subfolders will be added")), path))
	{
		string filename = FindFile(path, filter);
		DWORD flags = FLEB_NEW_ENTRY | FLEB_MANY_ENTRIES;
		if(FileListEntryBox(this, flags, filename, resolution, finger_id))
		{
			CWaitCursor waitCursor;
			listview->SetRedrawState(false);
			if(AddFromDirectory(path, filter) > 0)
			{
				modified = true;
				UpdateSelection();
			}
			listview->SetRedrawState(true);
		}
	}
}

int CFileListForm::AddFromDirectory(const string &path, const string &filter)
{
	int c = 0;
	string cur_path = path;
	AddTrailingBackslash(cur_path);
	WIN32_FIND_DATA fd;
	string fn = cur_path + TEXT('*');
	HANDLE h = FindFirstFile(fn, &fd);
	if(h != INVALID_HANDLE_VALUE)
	{
		do
		{
			if(!(fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
			{
				string filename = fd.cFileName;
				if(FileFilterHasExtension(filter, GetFileExtension(filename)))
				{
					AddFile(cur_path + filename);
					c++;
				}
			}
		} while(FindNextFile(h, &fd));
		FindClose(h);
	}
	h = FindFirstFile(fn, &fd);
	if(h != INVALID_HANDLE_VALUE)
	{
		do
		{
			if(fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY
				&& !StringsEqual(fd.cFileName, TEXT(".")) && !StringsEqual(fd.cFileName, TEXT("..")))
				c += AddFromDirectory(cur_path + fd.cFileName, filter);
		} while(FindNextFile(h, &fd));
		FindClose(h);
	}
	return c;
}

void CFileListForm::OnEditDelete()
{
	if(listview->SelCount() > 0)
	{
		CWaitCursor waitCursor;
		listview->SetRedrawState(false);
		changing = true;
		for(int i = filelist->Length() - 1; i >= 0; i--)
		{
			if(listview->ItemSelected(i))
			{
				filelist->Remove(i);
				listview->RemoveItem(i);
			}
		}
		listview->SetRedrawState(true);
		modified = true;
		changing = false;
		UpdateSelection();
	}
}

void CFileListForm::OnEditMoveUp()
{
	int i, first = 0;
	for(i = first; i < filelist->Length(); i++)
	{
		if(listview->ItemSelected(i))
		{
			first = i;
			break;
		}
	}
	if(first > 0)
	{
		CWaitCursor waitCursor;
		listview->SetRedrawState(false);
		changing = true;
		for(i = first; i < filelist->Length(); i++)
		{
			if(listview->ItemSelected(i))
			{
				VFFileListItem item = (*filelist)[i];
				filelist->Remove(i);
				listview->RemoveItem(i);
				filelist->Insert(i - 1, item);
				listview->InsertItem(i - 1);
				listview->SetItemSelected(i - 1, true);
			}
		}
		listview->SetRedrawState(true);
		modified = true;
		changing = false;
		UpdateSelection();
	}
}

void CFileListForm::OnEditMoveDown()
{
	int i, last = filelist->Length() - 1;
	for(i = last; i >= 0 ; i--)
	{
		if(listview->ItemSelected(i))
		{
			last = i;
			break;
		}
	}
	if(last < filelist->Length() - 1)
	{
		CWaitCursor waitCursor;
		listview->SetRedrawState(false);
		changing = true;
		for(i = last; i >= 0; i--)
		{
			if(listview->ItemSelected(i))
			{
				VFFileListItem item = (*filelist)[i];
				filelist->Remove(i);
				listview->RemoveItem(i);
				filelist->Insert(i + 1, item);
				listview->InsertItem(i + 1);
				listview->SetItemSelected(i + 1, true);
			}
		}
		listview->SetRedrawState(true);
		modified = true;
		changing = false;
		UpdateSelection();
	}
}

void CFileListForm::OnEditClearFingerID()
{
	if(listview->SelCount() > 0)
	{
		CWaitCursor waitCursor;
		for(int i = 0; i < filelist->Length(); i++)
			if(listview->ItemSelected(i))
				(*filelist)[i].finger_id = emptyString;
		modified = true;
		UpdateSelection();
		listview->RedrawItems();
	}
}

void CFileListForm::OnEditNewEntry()
{
	string filename;
	DWORD flags = FLEB_NEW_ENTRY;
	if(FileListEntryBox(this, flags, filename, resolution, finger_id))
	{
		AddFile(filename);
		modified = true;
		UpdateSelection();
	}
}

void CFileListForm::OnEditEditEntry()
{
	int c = listview->SelCount();
	if(!c) return;
	string filename;
	int resolution = 0;
	string finger_id;
	bool dif_res = false, dif_fid = false;
	if(c == 1)
	{
		VFFileListItem &item = (*filelist)[listview->SelIndex()];
		filename = item.filename;
		resolution = item.resolution;
		finger_id = item.finger_id;
	}
	else
	{
		bool first = true;
		for(int i = 0; i < filelist->Length() && (!dif_res || !dif_fid); i++)
		{
			if(listview->ItemSelected(i))
			{
				VFFileListItem &item = (*filelist)[i];
				if(first)
				{
					filename = item.filename;
					resolution = item.resolution;
					finger_id = item.finger_id;
					first = false;
				}
				else
				{
					if(!dif_res && item.resolution != resolution) dif_res = true;
					if(!dif_fid && item.finger_id != finger_id) dif_fid = true;
				}
			}
		}
	}
	DWORD flags = 0;
	SetFlag(flags, FLEB_MANY_ENTRIES, c > 1);
	SetFlag(flags, FLEB_DIF_RES, dif_res);
	SetFlag(flags, FLEB_DIF_FID, dif_fid);
	if(FileListEntryBox(this, flags, filename, resolution, finger_id))
	{
		dif_res = IsFlagSet(flags, FLEB_DIF_RES);
		dif_fid = IsFlagSet(flags, FLEB_DIF_FID);
		if(!dif_res) this->resolution = resolution;
		if(!dif_fid) this->finger_id = finger_id;
		if(c == 1)
		{
			SetFile(listview->SelIndex(), filename);
			listview->RedrawItems(listview->SelIndex());
		}
		else if(!dif_res || !dif_fid)
		{
			CWaitCursor waitCursor;
			for(int i = 0; i < filelist->Length(); i++)
			{
				if(listview->ItemSelected(i))
				{
					VFFileListItem &item = (*filelist)[i];
					if(!dif_res) item.resolution = resolution;
					if(!dif_fid) item.finger_id = MakeFingerID(finger_id, item.filename, false);
				}
			}
			listview->RedrawItems();
		}
		if(c == 1 || !dif_res || !dif_fid)
		{
			modified = true;
			UpdateSelection();
		}
	}
}

void CFileListForm::OnEditSelectAll()
{
	if(listview->SelCount() != listview->ItemCount())
	{
		listview->SelectAll();
		//UpdateSelection();
	}
}

void CFileListForm::OnEditSelectNone()
{
	if(listview->SelCount())
	{
		listview->SelectNone();
		//UpdateSelection();
	}
}

void CFileListForm::OnEditGoTo()
{
	string v;
	if(listview->SelIndex() != -1) v = IntToString(listview->SelIndex() + 1);
	if(InputBox(this, STRING("Go to"), STRING("Entry number:"), v))
		try
		{
			int index = StringToInt(v) - 1;
			if(index < 0 || index >= filelist->Length())
				throw exception("Entry number out of bounds");
			listview->SetSelIndex(index);
			listview->EnsureVisible(index);
		}
		catch(exception &e)
		{
			ErrorBox(this, e, TEXT("Go to"));
		}
}

string FileCountToString(int value, const string &no = emptyString)
{
	return IntToString(value, STRING("file"), STRING("files"), no);
}

void CFileListForm::UpdateSelection()
{
	int c = listview->SelCount();
	EnableMenuItem(ID_EDIT_EDITENTRY, c > 0);
	SetMenuItemText(ID_EDIT_EDITENTRY, STRING("&Edit ") + (c > 1 ? STRING("entries") : STRING("entry")) + STRING("..."));
	EnableMenuItem(ID_EDIT_DELETE, c > 0);
	int i, first = 0, last = filelist->Length() - 1;
	for(i = first; i < filelist->Length(); i++)
	{
		if(listview->ItemSelected(i))
		{
			first = i;
			break;
		}
	}
	for(i = last; i >= 0 ; i--)
	{
		if(listview->ItemSelected(i))
		{
			last = i;
			break;
		}
	}
	EnableMenuItem(ID_EDIT_MOVEUP, first > 0);
	EnableMenuItem(ID_EDIT_MOVEDOWN, last < filelist->Length() - 1);
	EnableMenuItem(ID_EDIT_CLEARFINGERID, c > 0);
	EnableMenuItem(ID_EDIT_SELECTNONE, c > 0);
	EnableMenuItem(ID_EDIT_SELECTALL, c != listview->ItemCount());
	statusbar->SetPartText(0, modified ? STRING("Modified") : emptyString);
	statusbar->SetPartText(1, Format(TEXT("%s selected, %s total"),
		(LPCTSTR)FileCountToString(listview->SelCount(), STRING("No")),
		(LPCTSTR)FileCountToString(listview->ItemCount())));
}

void CFileListForm::OnCreate()
{
	CForm::OnCreate();
	UpdateSelection();
}

bool CFileListForm::OnCloseQuery()
{
	return ClearPrompt();
}

void CFileListForm::SetFileList(VFFileList *value)
{
	if(filelist != value)
	{
		filelist = value;
		UpdateFileList();
	}
}

void CFileListForm::SetResolution(int value)
{
	resolution = value;
}

void CFileListForm::SetFingerID(const string &value)
{
	finger_id = value;
}

/////////////////
//             //
// FileListBox //
//             //
/////////////////

void FileListBox(CForm *owner)
{
	CFileListForm *form = new CFileListForm();
	VFFileList filelist;
	form->SetFileList(&filelist);
	form->SetOwner(owner);
	form->ShowModal();
	delete form;
}
