#pragma once

#include "VFingerX.h"

// OLE DB
#include <OleDB.h>
#include "msado15.tlh"
using namespace ADODB;
#include "icrsint.h"

#define SOHEAP_MAX_BLOCK_COUNT 1024

class CSOHeap
{
private:
	DWORD blockSize;
	int blockCount;
	BYTE * blocks[SOHEAP_MAX_BLOCK_COUNT];
	DWORD sizes[SOHEAP_MAX_BLOCK_COUNT];

public:
	CSOHeap(DWORD blockSize = 1048576)
		: blockCount(0)
	{
		this->blockSize = blockSize;
	}

	~CSOHeap()
	{
		Clear();
	}

	void Clear()
	{
		for(int i = 0; i < blockCount; i++)
			free(blocks[i]);
		blockCount = 0;
	}

	BYTE* Alloc(WORD size)
	{
		if(!size) return NULL;
		DWORD memSize = size + sizeof(WORD);
		BYTE *block = NULL;
		int j = -1;
		for(int i = 0; i < blockCount; i++)
		{
			if(blockSize - sizes[i] >= memSize)
			{
				block = blocks[i];
				j = i;
				break;
			}
		}
		if(!block)
		{
			if(blockCount == SOHEAP_MAX_BLOCK_COUNT) throw exception("SOHeap: max block count riched");
			block = new BYTE[blockSize];
			blocks[blockCount] = block;
			sizes[blockCount] = 0;
			j = blockCount;
			blockCount++;
		}
		block += sizes[j];
		*(WORD*)block = size; block += sizeof(WORD);
		sizes[j] += memSize;
		return block;
	}

	void Free(BYTE * /*object*/)
	{
		// Do nothing - memory will be allocated until heap is cleared
	}

};

#define FINGER_ID_LENGTH 50

class CFingerprintsRs: public CADORecordBinding
{

BEGIN_ADO_BINDING(CFingerprintsRs)
	
	ADO_FIXED_LENGTH_ENTRY(1, adInteger, id, idStatus, FALSE)

	ADO_VARIABLE_LENGTH_ENTRY(2, adBinary, features, sizeof(features), featuresStatus, featuresLength, TRUE)

#ifdef _UNICODE
	ADO_VARIABLE_LENGTH_ENTRY(3, adWChar, fingerId, sizeof(fingerId), fingerIdStatus, fingerIdLength, TRUE)
#else
	ADO_VARIABLE_LENGTH_ENTRY(3, adChar, fingerId, sizeof(fingerId), fingerIdStatus, fingerIdLength, TRUE)
#endif

END_ADO_BINDING()

public:
	int id;
	ULONG idStatus;
	BYTE features[VF_MAX_FEATURES_SIZE];
	ULONG featuresStatus;
	DWORD featuresLength;
	TCHAR fingerId[FINGER_ID_LENGTH + 1];
	ULONG fingerIdStatus;
	DWORD fingerIdLength;
};

struct VFDBItem
{
	int index;
	int gIndex;
	int id;
	BYTE *features;
	DWORD featuresSize;
	DWORD fingerIdLength;
	TCHAR fingerId[FINGER_ID_LENGTH + 1];
};

interface IVFDBOpenCallback
{
	virtual void OnOpenState(const string &state) = 0;
	virtual bool OnOpenProgress(bool started, int value) = 0;
};

struct VFDBFindData
{
	int testG;
	int g;
	int index;
};

class CVFDatabase
{
private:
	CSOHeap heap;

	bool connected;
	bool opened;

	string recordsetName;

	_ConnectionPtr connection;

	_RecordsetPtr recordset;

	CFingerprintsRs record;

	IADORecordBinding * recordBinding;

	VFDBItem **items[256], **allItems;
	int capacity[256], allCapacity;
	int count[256], allCount;

	string provider;
	string dataSource;

	void OpenRecordset(CursorTypeEnum cursor_type = adOpenKeyset, bool bind = true,
		LockTypeEnum lockType = adLockOptimistic, CursorLocationEnum cursorLocation = adUseServer);
	void CloseRecordset();

	static void CopyString(LPTSTR dst, DWORD &dstLength, LPCTSTR src, DWORD srcLength, DWORD maxLength);
	static void CopyFingerID(LPTSTR dst, DWORD &dstLength, LPCTSTR src, DWORD srcLength);

	void ClearItems();
	int AddItem(int id, const BYTE * features, DWORD size,
		const LPCTSTR fingerId, DWORD fingerIdLength);

	void CheckIndex(int index);

	void SetRecord(int index);
	bool FindRecord(int id);
	void UpdateRecord(int index);

public:
	CVFDatabase();
	~CVFDatabase();

	void Open(const string &provider, const string &dataSource,
		const string &userId = emptyString, const string &password = emptyString,
		IVFDBOpenCallback * callback = NULL);
	void Close();
	bool Opened() { return opened; };

	const string & Provider() { return provider; }
	const string & DataSource() { return dataSource; }

	int get_Count() { return allCount; }
	__declspec(property(get = get_Count)) int Count;
	
	bool get_IsEmpty() { return allCount != 0; }
	__declspec(property(get = get_IsEmpty)) bool IsEmpty;

	int ID(int index);
	const BYTE * Features(int index);
	DWORD FeaturesSize(int index);
	string FingerID(int index);

	void SetFeatures(int index, const BYTE * features, DWORD size);
	void SetFingerID(int index, const string &fingerId);

	int Add(const BYTE * features, DWORD size, LPCTSTR fingerId);
	void Remove(int index);
	void Clear();

	void FindStart(const BYTE * features, VFDBFindData &fd);
	int FindNext(VFDBFindData &fd);
	void FindEnd(VFDBFindData &fd);
};

extern const string defaultDatabaseProvider;
extern const string defaultDatabaseFileExtension;
string GetVFDBDefaultFilter();
string GetVFDBDefaultFilterName();
string GetVFDBDefaultFilterString(bool filterInName = true);
