import java.awt.Frame;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import neurotechnologija.utils.Utils;

import ScanMan.ScannerEventListener;

/**
 * Dialog used to accept images from enrollment and recognition from fingerprint scanner.
 *
 * @version 1.0 02/07/2002
 */
public class FingerprintDlg
        extends JDialog
        implements  ScannerEventListener, ActionListener {

        // members
        private Image[] images = null;
        private String[] id;
        private int imageCount = 0;
        private int numImages = 0;

        /**
         * Constructor
         * @param owner owning frame
         * @param title dialog title
         * @param imageCount number of images to retrieve from scanner
         * @param id array which will contain image id's entered by user.
         * Pass null if you need no id's.
         * */
        public FingerprintDlg(
                Frame owner,
                String title,
                int imageCount,
                String[] id) {
                super(owner, true);
                this.setTitle(title);
                this.imageCount = imageCount;
                this.images = new Image[imageCount];
                this.id = id;
                if (id == null) {
                        idLbl.setVisible(false);
                        idFld.setVisible(false);
                }

                try {
                        initGui();
                }
                catch (Exception e) {
                        e.printStackTrace();
                }
        }

        /**
         * Handle gui actions
         * */
        public void actionPerformed(ActionEvent e) {
                Object source = e.getSource();
                if (source == cancelBtn) {
                        dispose();
                }
        }

        /**
         * Return retrieved images
         * */
        public void getImages(Image[] images) {
                setVisible(true);

                System.arraycopy(this.images, 0, images, 0, this.images.length);
        }

        /**
         * Create user interface
         * */
        private void initGui() throws Exception {
                cancelBtn.setBounds(new Rectangle(312, 17, 79, 27));
                cancelBtn.setText("Cancel");
                cancelBtn.addActionListener(this);
                this.getContentPane().setLayout(null);
                infoLbl.setText("Please, put your finger on scanner");
                infoLbl.setBounds(new Rectangle(15, 66, 215, 17));
                idLbl.setText("Fingerprint id:");
                idLbl.setBounds(new Rectangle(16, 21, 85, 17));
                idFld.setBounds(new Rectangle(96, 18, 181, 21));
                statusLbl.setBorder(BorderFactory.createLoweredBevelBorder());
                statusLbl.setText("Status");
                statusLbl.setBounds(new Rectangle(1, 123, 400, 21));
                this.getContentPane().add(idLbl, null);
                this.getContentPane().add(idFld, null);
                this.getContentPane().add(statusLbl, null);
                this.getContentPane().add(cancelBtn, null);
                this.getContentPane().add(infoLbl, null);
                this.setSize(408, 170);
                this.setResizable(false);
                Utils.centerParent(this);
        }

        // components
        JButton cancelBtn = new JButton();
        JLabel infoLbl = new JLabel();
        JLabel idLbl = new JLabel();
        JTextField idFld = new JTextField();
        JLabel statusLbl = new JLabel();

        /**
         * Image received from fingerprint scanner
         * @see ScanMan.ScannerEventListener#imageEvent(int, int, byte[], int)
         */
        public void imageEvent(
                int width,
                int height,
                byte[] image,
                int resolution) {

                if (this.id != null && idFld.getText().equals("")) {
                        JOptionPane.showMessageDialog(this, "You have to specify fingerprint id!");
                        return;
                }
                // create image from grayscale value array
                this.images[numImages] = Utils.createImage(image, width, height);
                if (this.id != null) {
                        this.id[numImages] = idFld.getText();
                        idFld.setEnabled(false);
                }
                Settings.imageDpi = resolution;
                numImages++;
                statusLbl.setText(
                        "Image " + numImages + " of " + imageCount + " retrieved");
                infoLbl.setText("Please, put your finger on scanner");
                if (numImages == imageCount)
                        setVisible(false);

        }

        /**
         * Status event received from fingerprint scanner
         * @see ScanMan.ScannerEventListener#statusEvent(int)
         */
        public void statusEvent(int state) {
                String[] events = {"", "SM_STATE_FINGER_DOWN", "SM_STATE_FINGER_UP"};
                statusLbl.setText("Scanner event " + events[state]);
        }

}
