import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.FileNotFoundException;
import java.util.Hashtable;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import neurotechnologija.utils.Utils;

/**
 * VeriFinger Demo application: Properties dialog
 *
 * @version 1.0 02/07/2002
 */
public class PropertiesDlg extends JDialog implements ActionListener {

        /**
         * User Interface action handler
         * */
        public void actionPerformed(ActionEvent event) {
                Object source = event.getSource();

                if (source == okBtn) {
                        try {
                                updateSettings();
                                Settings.save(VFingerJava.CONFIG_FILE);
                        }
                        catch (FileNotFoundException e) {
                                System.out.println(e.getMessage());
                                e.printStackTrace();
                        }
                        dispose();
                }
                else if (source == cancelBtn) {
                        dispose();
                }
                else if (source == browseDbBtn) {
                        String fname = Utils.selectFile(this, "Select file");
                        if (fname != null)
                                fingerprintDbEdt.setText(fname);
                }
        }

        /**
         * Update settings storage with current control values
         * */
        private void updateSettings() {
                Settings.dbFile = fingerprintDbEdt.getText();
                Settings.falseAcceptanceRate = qualitySld.getValue();
                Settings.matchingSpeed = speedSld.getValue();
                Settings.imageDpi = Integer.parseInt(imageDpiEdt.getText());
                Settings.vfMode = veriFingerMode.getSelectedIndex();
                Settings.maximalRotation = Integer.parseInt(maxRotationEdt.getText());
                Settings.useBlockedOrientation = showBlockOrientations.isSelected();
        }

        /**
         * Constructor
         * */
        public PropertiesDlg(Frame owner) {
                super(owner, true);
                addWindowListener(new WindowAdapter() {
                        public void windowClosing(WindowEvent e) {
                                dispose();
                        }
                });

                getContentPane().setLayout(null);
                String[] modes = {
                        "General",
                        "DigitalPersona U.are.U",
                        "BiometriKa FX2000",
                        "Keytronic SecureDesktop",
                        "Identix TouchView",
                        "Precise Biometrics 100CS",
                        "STMicroelectronics TouchChip",
                        "IdenticatorTechnology DF90",
                        "Authentec AFS2",
                        "Authentec AES4000",
                        "Atmel FingerChip",
                        "BMF BLP100",
                        "Secugen Hamster",
                        "Ethentica"};

                veriFingerMode = new JComboBox(modes);
                getContentPane().add(veriFingerMode);
        veriFingerMode.setBounds(10, 20, 200, 20);
        veriFingerMode.addActionListener(this);

                getContentPane().add(showBlockOrientations);
                showBlockOrientations.setText("Show block orientations");
                showBlockOrientations.setBounds(240, 20, 200, 20);

        getContentPane().add(imageDpiLbl);
        imageDpiLbl.setText("Input image DPI:");
        imageDpiLbl.setBounds(10, 80, 110, 16);

        getContentPane().add(imageDpiEdt);
        imageDpiEdt.setBounds(120, 80, 60, 20);


        maxRotationLbl.setText("Maximal rotation:");
        getContentPane().add(maxRotationLbl);
        maxRotationLbl.setBounds(200, 80, 110, 16);

        getContentPane().add(maxRotationEdt);
        maxRotationEdt.setBounds(310, 80, 60, 20);

        featuresExtractionPnl.setLayout(null);

        featuresExtractionPnl.setBorder(new TitledBorder("VeriFinger options:"));
        speedLbl.setText("Matching speed:");
        featuresExtractionPnl.add(speedLbl);
        speedLbl.setBounds(10, 30, 100, 16);
                Hashtable labels = new Hashtable();
                labels.put(new Integer(0), new JLabel("Low"));
                labels.put(new Integer(1), new JLabel("High"));
                speedSld.setLabelTable(labels);
                speedSld.setMajorTickSpacing(1);
        speedSld.setMaximum(1);
        speedSld.setMinimum(0);
        speedSld.setPaintLabels(true);
        speedSld.setPaintTicks(true);
        speedSld.setSnapToTicks(true);
        featuresExtractionPnl.add(speedSld);
        speedSld.setBounds(110, 20, 120, 50);

        qualityLbl.setText("FAR:");
        featuresExtractionPnl.add(qualityLbl);
        qualityLbl.setBounds(250, 30, 50, 16);

        qualitySld.setMajorTickSpacing(1);
                Hashtable labels2 = new Hashtable();
                labels2.put(new Integer(0), new JLabel("0.1%"));
                labels2.put(new Integer(1), new JLabel("0.01%"));
                labels2.put(new Integer(2), new JLabel("0.001%"));
                qualitySld.setLabelTable(labels2);

        qualitySld.setMaximum(2);
        qualitySld.setMinimum(0);
        qualitySld.setPaintLabels(true);
        qualitySld.setPaintTicks(true);
        qualitySld.setSnapToTicks(true);
        featuresExtractionPnl.add(qualitySld);
        qualitySld.setBounds(280, 20, 180, 50);

        getContentPane().add(featuresExtractionPnl);
        featuresExtractionPnl.setBounds(0, 110, 480, 80);

            fingerprintDbEdt.setBounds(140, 200, 300, 20);
        fingerprintDbEdt.setText(Settings.dbFile);

        browseDbBtn.setText("...");
        browseDbBtn.setBounds(440, 200, 30, 20);
        browseDbBtn.addActionListener(this);

        fingerprintDbLbl.setText("Fingerprint database:");
        fingerprintDbLbl.setBounds(10, 200, 120, 16);

        getContentPane().add(fingerprintDbEdt);
        getContentPane().add(browseDbBtn);
        getContentPane().add(fingerprintDbLbl);

        okBtn.setText("OK");
        okBtn.setPreferredSize(new java.awt.Dimension(73, 26));
        getContentPane().add(okBtn);
        okBtn.setBounds(310, 240, 80, 26);
        okBtn.addActionListener(this);

        cancelBtn.setText("Cancel");
        getContentPane().add(cancelBtn);
        cancelBtn.setBounds(410, 240, 73, 26);
        cancelBtn.addActionListener(this);

                this.setSize(500, 300);
                Utils.centerParent(this);
                this.setTitle("Properties");
                this.setResizable(false);

                resetControls();
        }

        private void resetControls() {
                imageDpiEdt.setText(Integer.toString(Settings.imageDpi));
                maxRotationEdt.setText(Integer.toString(Settings.maximalRotation));
                qualitySld.setValue(Settings.falseAcceptanceRate);
                speedSld.setValue(Settings.matchingSpeed);
                veriFingerMode.setSelectedIndex(Settings.vfMode);
                showBlockOrientations.setSelected(Settings.useBlockedOrientation);
        }

        // components
        JComboBox veriFingerMode;
        JCheckBox showBlockOrientations = new JCheckBox();
        JCheckBox showRecognDlgCbx = new JCheckBox();
        JCheckBox showEnrollDlgCbx = new JCheckBox();
        JTextField fingerprintDbEdt = new JTextField();
        JButton browseDbBtn = new JButton();
        JLabel fingerprintDbLbl = new JLabel();
        JLabel imageScalingLbl = new JLabel();
        JLabel imageDpiLbl = new JLabel();
        JTextField imageDpiEdt = new JTextField();
        JLabel maxRotationLbl = new JLabel();
        JTextField maxRotationEdt = new JTextField();
        JButton okBtn = new JButton();
        JButton cancelBtn = new JButton();
        JPanel FilesPnl = new JPanel();
        JSlider speedSld = new JSlider();
        JSlider qualitySld = new JSlider();
        JPanel featuresExtractionPnl = new JPanel();
        JLabel speedLbl = new JLabel();
        JLabel qualityLbl = new JLabel();
}
