unit FormFileList;

interface

{$WARN UNIT_PLATFORM OFF}
{$WARN SYMBOL_PLATFORM OFF}

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ActnList, Menus, ComCtrls, VFFileList, VFImage, Utils, VFinger,
  FormFileListEntry, FileCtrl;

const
  UM_SELCHANGED = WM_USER;

type
  TFileListForm = class(TForm)
    Actions: TActionList;
    FileNew: TAction;
    FileOpen: TAction;
    FileSave: TAction;
    FileExit: TAction;
    EditAdd: TAction;
    EditAddDirectory: TAction;
    EditNewEntry: TAction;
    EditEditEntry: TAction;
    EditDelete: TAction;
    EditMoveUp: TAction;
    EditMoveDown: TAction;
    EditClearFingerId: TAction;
    EditSelectAll: TAction;
    EditSelectNone: TAction;
    MainMenu: TMainMenu;
    miFile: TMenuItem;
    miFileNew: TMenuItem;
    miFileOpen: TMenuItem;
    miFileSave: TMenuItem;
    msFile1: TMenuItem;
    miFileExit: TMenuItem;
    miEdit: TMenuItem;
    miEditAdd: TMenuItem;
    miEditAddDirectory: TMenuItem;
    msEdit1: TMenuItem;
    miEditNewEntry: TMenuItem;
    miEditEditEntry: TMenuItem;
    miEditDelete: TMenuItem;
    msEdit2: TMenuItem;
    miEditMoveUp: TMenuItem;
    miEditMoveDown: TMenuItem;
    msEdit3: TMenuItem;
    miEditClearFingerId: TMenuItem;
    msEdit4: TMenuItem;
    miEditSelectAll: TMenuItem;
    miEditSelectNone: TMenuItem;
    ListView: TListView;
    StatusBar: TStatusBar;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    AddDialog: TOpenDialog;
    procedure FormCreate(Sender: TObject);
    procedure ListViewData(Sender: TObject; Item: TListItem);
    procedure FileOpenExecute(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FileNewExecute(Sender: TObject);
    procedure FileSaveExecute(Sender: TObject);
    procedure FileExitExecute(Sender: TObject);
    procedure ListViewClick(Sender: TObject);
    procedure ListViewKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure EditSelectAllExecute(Sender: TObject);
    procedure EditSelectNoneExecute(Sender: TObject);
    procedure EditAddExecute(Sender: TObject);
    procedure EditAddDirectoryExecute(Sender: TObject);
    procedure EditDeleteExecute(Sender: TObject);
    procedure EditMoveUpExecute(Sender: TObject);
    procedure EditMoveDownExecute(Sender: TObject);
    procedure EditClearFingerIdExecute(Sender: TObject);
    procedure EditNewEntryExecute(Sender: TObject);
    procedure EditEditEntryExecute(Sender: TObject);
  private
    FModified: Boolean;
    FResolution: Integer;
    FFingerId: string;
    FFileList: TVFFileList;
    FOldCursor: TCursor;
    procedure SetFileList(const Value: TVFFileList);
    procedure SetFingerId(const Value: string);
    procedure SetResolution(const Value: Integer);
    procedure SetFileListItem(Index: Integer);
    procedure AddFileListItem;
    procedure UpdateFileList;
    procedure SetFile(Index: Integer; Filename: string);
    procedure AddFile(Filename: string);
    function ClearPrompt: Boolean;
    procedure UpdateSelection;
    procedure SetWaitCursor;
    procedure RemoveWaitCursor;
    function SaveFile: Boolean;
    function AddFromDirectory(Path, Filter: string): Integer;
    procedure UMSelChanged(var Message: TMessage); message UM_SELCHANGED;
  public
    property FileList: TVFFileList read FFileList write SetFileList;
    property Resolution: Integer read FResolution write SetResolution;
    property FingerId: string read FFingerId write SetFingerId;
  end;

procedure FileListBox(Owner: TForm);

implementation

{$R *.dfm}

{ FileListBox }

procedure FileListBox(Owner: TForm);
var
  Form: TFileListForm;
  FileList: TVFFileList;
begin
  Form := TFileListForm.Create(Owner);
  FileList := TVFFileList.Create;
  Form.FileList := FileList;
  Form.ShowModal;
  FileList.Free;
  Form.Free;
end;

{ TFileListForm }

procedure TFileListForm.AddFile(Filename: string);
begin
  FFileList.Add(Filename, FResolution, MakeFingerID(FFingerId, Filename, False));
  AddFileListItem;
end;

procedure TFileListForm.AddFileListItem;
begin
  ListView.Items.Add;
end;

procedure TFileListForm.SetFile(Index: Integer; Filename: string);
var
  Item: PVFFileListItem;
begin
  Item := FFileList[Index];
  Item.Filename := Filename;
  Item.Resolution := Resolution;
  Item.FingerId := FingerId;
  SetFileListItem(Index);
end;

procedure TFileListForm.SetFileList(const Value: TVFFileList);
begin
  if FFileList <> Value then
  begin
    FFileList := Value;
    UpdateFileList;
  end;
end;

procedure TFileListForm.SetFileListItem(Index: Integer);
begin
end;

procedure TFileListForm.SetFingerId(const Value: string);
begin
  FFingerId := Value;
end;

procedure TFileListForm.SetResolution(const Value: Integer);
begin
  FResolution := Value;
end;

procedure TFileListForm.UpdateFileList;
begin
  with ListView.Items do
  begin
    BeginUpdate;
    Count := FFileList.Count;
    EndUpdate;
  end;
end;

procedure TFileListForm.FormCreate(Sender: TObject);
begin
  FFileList := nil;
  FModified := False;
  FResolution := VF_IMAGE_RESOLUTION;
  FFingerId := DefaultFileFingerId;
  OpenDialog.Filter := TVFFileList.LoadFilterString;
  SaveDialog.Filter := TVFFileList.SaveFilterString;
  SaveDialog.DefaultExt := GetExtensionFromFileFilter(SaveDialog.Filter);
  AddDialog.Filter := ImageIO.GetLoadFileFilterString;

  UpdateSelection;
end;

procedure TFileListForm.ListViewData(Sender: TObject; Item: TListItem);
var
  I: PVFFileListItem;
begin
  I := FFileList[Item.Index];
  Item.Caption := I.FingerId;
  Item.SubItems.Add(IntToStr(I.Resolution));
  Item.SubItems.Add(I.Filename);
end;

procedure TFileListForm.FileOpenExecute(Sender: TObject);
var
  Ok: Boolean;
begin
  if ClearPrompt then
  begin
    OpenDialog.FileName := EmptyStr;
    if OpenDialog.Execute then
    begin
      SetWaitCursor;
      Ok := FFileList.Load(OpenDialog.FileName);
      if Ok then
      begin
        UpdateFileList;
        FModified := False;
        UpdateSelection;
      end;
      RemoveWaitCursor;
      if not Ok then MessageDlg(Format('Can not load from file ''%s''', [OpenDialog.FileName]), mtError, [mbOk], 0);
    end;
  end;
end;

function TFileListForm.ClearPrompt: Boolean;
begin
  if FModified then
    case MessageDlg('File list modified. Save changes?', mtConfirmation, [mbYes, mbNo, mbCancel], 0) of
      mrYes: Result := SaveFile;
      mrNo: Result := True;
    else
      Result := False;
    end
  else Result := True;
end;

procedure TFileListForm.UpdateSelection;
var
  C: Integer;
  I, First, Last: Integer;

  function FileCountToString(Value: Integer; No: string = ''): string;
  begin
    Result := IntToString(Value, 'file', 'files', No);
  end;

begin
  C := ListView.SelCount;
  EditEditEntry.Enabled := C > 0;
  EditEditEntry.Caption := 'Edit ';
  if C > 1 then EditEditEntry.Caption := EditEditEntry.Caption + 'entries'
  else EditEditEntry.Caption := EditEditEntry.Caption + 'entry';
  EditEditEntry.Caption := EditEditEntry.Caption + '...';
  EditDelete.Enabled := C > 0;
  First := 0;
  Last := ListView.Items.Count - 1;
  for I := First to ListView.Items.Count - 1 do
  begin
    if ListView.Items[I].Selected then
    begin
      First := I;
      Break;
    end;
  end;
  for I := Last downto 0 do
  begin
    if ListView.Items[I].Selected then
    begin
      Last := I;
      Break;
    end;
  end;
  EditMoveUp.Enabled := First > 0;
  EditMoveDown.Enabled := Last < ListView.Items.Count - 1;
  EditClearFingerId.Enabled := C > 0;
  EditSelectNone.Enabled := C > 0;
  EditSelectAll.Enabled := C <> ListView.Items.Count;
  if FModified then StatusBar.Panels[0].Text := 'Modified'
  else StatusBar.Panels[0].Text := EmptyStr;
  StatusBar.Panels[1].Text := Format('%s selected, %s total',
    [FileCountToString(C, 'No'), FileCountToString(ListView.Items.Count)]);
end;

procedure TFileListForm.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose := ClearPrompt;
end;

procedure TFileListForm.RemoveWaitCursor;
begin
  Screen.Cursor := FOldCursor;
end;

procedure TFileListForm.SetWaitCursor;
begin
  FOldCursor := Screen.Cursor;
  Screen.Cursor := crHourGlass;
end;

procedure TFileListForm.FileNewExecute(Sender: TObject);
begin
  if ClearPrompt then
  begin
    FFileList.Clear;
    with ListView.Items do
    begin
      BeginUpdate;
      Count := 0;
      EndUpdate;
      FModified := False;
      UpdateSelection;
    end;
  end;
end;

procedure TFileListForm.FileSaveExecute(Sender: TObject);
begin
  SaveFile;
end;

function TFileListForm.SaveFile: Boolean;
begin
  SaveDialog.FileName := EmptyStr;
  if SaveDialog.Execute then
  begin
    SetWaitCursor;
    Result := FFileList.Save(SaveDialog.FileName);
    if Result then
    begin
      FModified := False;
      UpdateSelection;
    end;
    RemoveWaitCursor;
    if not Result then MessageDlg(Format('Can not save to file ''%s''', [SaveDialog.FileName]), mtError, [mbOk], 0);
  end
  else Result := False;
end;

procedure TFileListForm.FileExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TFileListForm.ListViewClick(Sender: TObject);
begin
  PostMessage(Handle, UM_SELCHANGED, 0, 0);
end;

procedure TFileListForm.ListViewKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  PostMessage(Handle, UM_SELCHANGED, 0, 0);
end;

procedure TFileListForm.UMSelChanged(var Message: TMessage);
begin
  UpdateSelection;
end;

procedure TFileListForm.EditSelectAllExecute(Sender: TObject);
begin
  ListView.SelectAll;
  UpdateSelection;
end;

procedure TFileListForm.EditSelectNoneExecute(Sender: TObject);
begin
  ListView.Selected := nil;
  UpdateSelection;
end;

procedure TFileListForm.EditAddExecute(Sender: TObject);
var
  Filename: string;
  Flags: LongWord;
  I: Integer;
begin
  AddDialog.FileName := EmptyStr;
  if AddDialog.Execute then
  begin
    Filename := AddDialog.FileName;
    Flags := FLEB_NEW_ENTRY;
    SetFlag(Flags, FLEB_MANY_ENTRIES, AddDialog.Files.Count > 1);
    if FileListEntryBox(Self, Flags, Filename, FResolution, FFingerId) then
      with ListView.Items do
      begin
        SetWaitCursor;
        BeginUpdate;
        for I := 0 to AddDialog.Files.Count - 1 do
          AddFile(AddDialog.Files[I]);
        FModified := True;
        UpdateSelection;
        EndUpdate;
        RemoveWaitCursor;
      end;
  end;
end;

procedure TFileListForm.EditAddDirectoryExecute(Sender: TObject);
var
  Path: string;
  Filter: string;
  Filename: string;
  Flags: LongWord;
begin
  Filter := ImageIO.LoadFileFilter;
  if SelectDirectory('Files with ' + Filter + ' extensions from this folder and all its subfolders will be added',
    '', Path) then
  begin
    Filename := FindFile(Path, Filter);
    Flags := FLEB_NEW_ENTRY or FLEB_MANY_ENTRIES;
    if FileListEntryBox(Self, Flags, Filename, FResolution, FFingerId) then
    begin
      SetWaitCursor;
      ListView.Items.BeginUpdate;
      if AddFromDirectory(Path, Filter) > 0 then
      begin
        FModified := True;
        UpdateSelection;
      end;
      ListView.Items.EndUpdate;
      RemoveWaitCursor;
    end;
  end;
end;

function TFileListForm.AddFromDirectory(Path, Filter: string): Integer;
var
  CurPath: string;
  FN: string;
  F: TSearchRec;
  Filename: string;
begin
  Result := 0;
  CurPath := Path;
  CurPath := IncludeTrailingBackslash(CurPath);
  FN := CurPath + '*';
  if FindFirst(FN, faAnyFile, F) = 0 then
  begin
    repeat begin
      if F.Attr and faDirectory = 0 then
      begin
        Filename := F.Name;
        if FileFilterHasExtension(Filter, ExtractFileExt(Filename)) then
        begin
          AddFile(CurPath + Filename);
          Inc(Result);
        end;
      end;
    end until FindNext(F) <> 0;
    FindClose(F);
  end;
  if FindFirst(FN, faDirectory, F) = 0 then
  begin
    repeat begin
      if (F.Name <> '.') and (F.Name <> '..') then
        Inc(Result, AddFromDirectory(CurPath + F.Name, Filter));
    end until FindNext(F) <> 0;
    FindClose(F);
  end;
end;

procedure TFileListForm.EditDeleteExecute(Sender: TObject);
var
  I, J: Integer;
begin
  if ListView.SelCount > 0 then
    with ListView.Items do
    begin
      SetWaitCursor;
      BeginUpdate;
      if ListView.ItemFocused <> nil then J := ListView.ItemFocused.Index
      else J := -1;
      for I := FFileList.Count - 1 downto - 1 do
      begin
        if (I >= 0) and Item[I].Selected then
        begin
          FFileList.Remove(I);
          Count := Count - 1;
          if (J <> -1) and (I < J) then Dec(J);
        end
      end;
      ListView.Selected := nil;
      if J = Count then Dec(J);
      if J <> -1 then ListView.ItemFocused := Item[J];
      EndUpdate;
      FModified := True;
      UpdateSelection;
      RemoveWaitCursor;
    end;
end;

procedure TFileListForm.EditMoveUpExecute(Sender: TObject);
var
  I, First: Integer;
begin
  First := 0;
  for I := First to FFileList.Count - 1 do
  begin
    if ListView.Items[I].Selected then
    begin
      First := I;
      Break;
    end;
  end;
  if First > 0 then
    with ListView.Items do
    begin
      SetWaitCursor;
      BeginUpdate;
      for I := First to FFileList.Count - 1 do
      begin
        if Item[I].Selected then
        begin
          FFileList.Move(I, I - 1);
          Item[I - 1].Selected := True;
          Item[I].Selected := False;
        end;
      end;
      EndUpdate;
      FModified := True;
      UpdateSelection;
      RemoveWaitCursor;
    end;
end;

procedure TFileListForm.EditMoveDownExecute(Sender: TObject);
var
  I, Last: Integer;
begin
  Last := FFileList.Count - 1;
  for I := Last downto 0 do
  begin
    if ListView.Items[I].Selected then
    begin
      Last := I;
      Break;
    end;
  end;
  if Last < FFileList.Count - 1 then
    with ListView.Items do
    begin
      SetWaitCursor;
      BeginUpdate;
      for I := Last downto 0 do
      begin
        if Item[I].Selected then
        begin
          FFileList.Move(I, I + 1);
          Item[I + 1].Selected := True;
          Item[I].Selected := False;
        end;
      end;
      EndUpdate;
      FModified := True;
      UpdateSelection;
      RemoveWaitCursor;
    end;
end;

procedure TFileListForm.EditClearFingerIdExecute(Sender: TObject);
var
  I: Integer;
begin
  if ListView.SelCount > 0 then
    with ListView.Items do
    begin
      SetWaitCursor;
      BeginUpdate;
      for I := 0 to FFileList.Count - 1 do
      begin
        if Item[I].Selected then
        begin
          FFileList[I].FingerId := EmptyStr;
        end;
      end;
      EndUpdate;
      FModified := True;
      UpdateSelection;
      RemoveWaitCursor;
    end;
end;

procedure TFileListForm.EditNewEntryExecute(Sender: TObject);
var
  Filename: string;
  Flags: LongWord;
begin
  Flags := FLEB_NEW_ENTRY;
  if FileListEntryBox(Self, Flags, Filename, FResolution, FFingerId) then
  begin
    AddFile(Filename);
    FModified := True;
    UpdateSelection;
  end;
end;

procedure TFileListForm.EditEditEntryExecute(Sender: TObject);
var
  C: Integer;
  Filename: string;
  Resolution: Integer;
  FingerId: string;
  DifRes, DifFID: Boolean;
  Item: PVFFileListItem;
  I: Integer;
  First: Boolean;
  Flags: LongWord;
begin
  C := ListView.SelCount;
  if C = 0 then Exit;
  Resolution := 0;
  DifRes := False; DifFID := False;
  if C = 1 then
  begin
    Item := FFileList[ListView.Selected.Index];
    Filename := Item.Filename;
    Resolution := Item.Resolution;
    FingerId := Item.FingerId;
  end
  else begin
    First := True;
    for I := 0 to FFileList.Count - 1 do
    begin
      if ListView.Items[I].Selected then
      begin
        Item := FFileList[I];
        if First then
        begin
          Filename := Item.Filename;
          Resolution := Item.Resolution;
          FingerId := Item.FingerId;
          First := False;
        end
        else begin
          if not DifRes and (Item.Resolution <> Resolution) then DifRes := True;
          if not DifFid and (Item.FingerId <> FingerId) then DifFID := True;
        end;
      end;
      if DifRes and DifFID then Break;
    end;
  end;
  Flags := 0;
  SetFlag(Flags, FLEB_MANY_ENTRIES, C > 1);
  SetFlag(Flags, FLEB_DIF_RES, DifRes);
  SetFlag(Flags, FLEB_DIF_FID, DifFID);
  if FileListEntryBox(Self, Flags, Filename, Resolution, FingerId) then
  begin
    DifRes := IsFlagSet(Flags, FLEB_DIF_RES);
    DifFid := IsFlagSet(Flags, FLEB_DIF_FID);
    if not DifRes then FResolution := Resolution;
    if not DifFID then FFingerId := FingerId;
    if C = 1 then
    begin
      ListView.Items.BeginUpdate;
      SetFile(ListView.Selected.Index, Filename);
      ListView.Items.EndUpdate;
    end
    else if not DifRes or not DifFid then
    begin
      SetWaitCursor;
      ListView.Items.BeginUpdate;
      for I := 0 to FFileList.Count - 1 do
      begin
        if ListView.Items[I].Selected then
        begin
          Item := FFileList[I];
          if not DifRes then Item.Resolution := Resolution;
          if not DifFID then Item.FingerId := MakeFingerID(FingerId, Item.Filename, False);
          SetFileListItem(I);
        end;
      end;
      ListView.Items.EndUpdate;
      RemoveWaitCursor;
    end;
    if (C = 1) or not DifRes or not DifFID then
    begin
      FModified := True;
      UpdateSelection;
    end;
  end;
end;

end.

