unit Log;

interface

uses Windows, Messages, SysUtils, Classes, StdCtrls, ComCtrls, SyncObjs;

type
  TLog = class
  private
    CS: TCriticalSection;
    FLastLine: string;
    FFilename: string;
    FStream: TStream;
    FControl: TRichEdit;

    procedure CloseFile();
    procedure OpenFile(Mode: Word = fmOpenRead);

    procedure AddText(Text: string);
    procedure AppendText(Text: string);

    procedure SetControl(const Value: TRichEdit);
    procedure SetFilename(const Value: string);

  public
    constructor Create;
    destructor Destroy; override;

    procedure Clear;
    procedure Add(Text: string); overload;
    procedure Add(Format: string; Args: array of const); overload;
    procedure Append(Text: string); overload;
    procedure Append(Format: string; Args: array of const); overload;
    procedure AddEmpty;

    property Control: TRichEdit read FControl write SetControl;
    property Filename: string read FFilename write SetFilename;

    property LastLine: string read FLastLine;

  end;

implementation

{ TLog }

procedure TLog.Add(Format: string; Args: array of const);
begin
  AddText(SysUtils.Format(Format, Args));
end;

procedure TLog.Add(Text: string);
begin
  AddText(Text);
end;

procedure TLog.AddEmpty;
begin
  AddText(EmptyStr);
end;

procedure TLog.AddText(Text: string);
var
  C: AnsiChar;
  S: AnsiString;
begin
  CS.Enter;
  if FControl <> nil then
  begin
    FControl.Lines.Add(Text);
    SendMessage(FControl.Handle, WM_VSCROLL, SB_BOTTOM, 0);
  end;
  if FFilename <> EmptyStr then
  begin
    OpenFile();
    if FStream.Size > 0 then
    begin
      C := #13;
      FStream.WriteBuffer(C, SizeOf(AnsiChar));
    end;
    S := Text;
    FStream.WriteBuffer(S[1], Length(S) * SizeOf(AnsiChar));
    CloseFile();
  end;
  FLastLine := Text;
  UniqueString(FLastLine);
  CS.Leave;
end;

procedure TLog.Append(Format: string; Args: array of const);
begin
  AppendText(SysUtils.Format(Format, Args));
end;

procedure TLog.Append(Text: string);
begin
  AppendText(Text);
end;

procedure TLog.AppendText(Text: string);
var
  I: Integer;
  S: AnsiString;
begin
  CS.Enter;
  if FControl <> nil then
  begin
    I := FControl.Lines.Count - 1;
    FControl.Lines[I] := FControl.Lines[I] + Text;
    SendMessage(FControl.Handle, WM_VSCROLL, SB_BOTTOM, 0);
  end;
  if FFilename <> EmptyStr then
  begin
    OpenFile();
    S := Text;
    FStream.WriteBuffer(S[1], Length(S) * SizeOf(AnsiChar));
    CloseFile;
  end;
  FLastLine := FLastLine + Text;
  CS.Leave;
end;

procedure TLog.Clear;
begin
  CS.Enter;
  if FControl <> nil then FControl.Lines.Clear;
  OpenFile(fmCreate);
  CloseFile;
  FLastLine := EmptyStr;
  CS.Leave;
end;

procedure TLog.CloseFile;
begin
  if FStream <> nil then
  begin
    FStream.Free;
    FStream := nil;
  end;
end;

constructor TLog.Create;
begin
  CS := TCriticalSection.Create;
  FControl := nil;
  FStream := nil;
end;

destructor TLog.Destroy;
begin
  CloseFile;
  CS.Free;
  inherited;
end;

procedure TLog.OpenFile(Mode: Word = fmOpenRead);
begin
  CloseFile;
  if FFilename <> EmptyStr then FStream := TFileStream.Create(FFilename, Mode or fmShareDenyWrite);
  if FStream <> nil then FStream.Seek(0, soFromEnd);
end;

procedure TLog.SetControl(const Value: TRichEdit);
begin
  FControl := Value;
end;

procedure TLog.SetFilename(const Value: string);
begin
  FFilename := Value;
  OpenFile(fmCreate);
  CloseFile;
end;

end.

