#include "stdafx.h"

#define SM_NAME "ScanMan Java Wrapper"

#define MAKEDWORD(high, low) (((DWORD)(high) << 16) | (DWORD)(low))
#define SM_VERSION_HIGH         MAKEDWORD(0, 1)
#define SM_VERSION_LOW          MAKEDWORD(0, 1)
#define SM_COPYRIGHT "Copyright (c) 2002-2006 Neurotechnologija"

// Number of times SMInitialize() was called. The same number of times SMFinalize() should be called
static int init_count = 0;

// Is monitor set
static bool monitoring = true;

// Currently set monitor
static struct Monitor
{
    jobject jobj; // ScannersMonitor
} monitor;

struct Capturing
{
    char*   scanner_id;
    HANDLE  handle;
    jobject jobj; // ScannerEventListener
};

typedef std::vector<Capturing*> Capturings;

// Started capturings
static Capturings capturings;

static int IToE(BSTR iid, char * eid, int len)
{
    int l = SysStringLen(iid);
    int result = 0;

    if(eid)
    {
        result = WideCharToMultiByte(CP_ACP, 0, iid, l, eid, len, NULL, NULL);
        if(eid && len > l) 
            eid[l] = '\0';
    }
    else 
        result = l+1;

    return result;
}

static BSTR EToI(const char *eid)
{
    int l = strlen(eid) + 1;
    int s = MultiByteToWideChar(CP_ACP, 0, eid, l, NULL, 0);
    OLECHAR *buf = new OLECHAR[s];
    MultiByteToWideChar(CP_ACP, 0, eid, l, buf, s);
    BSTR iid = SysAllocString(buf);
    delete[] buf;
    return iid;
}

void CallJavaMonitorEvent(jobject jobj, char* id, long lStatus)
{ 
    JavaVM *jvm = NULL;
    JNIEnv *env = NULL;    
    jint   size = 0;

    JNI_GetCreatedJavaVMs(&jvm, 1, &size);    
    if (jvm != NULL)
        jvm->AttachCurrentThread((void **)&env, NULL);

    if (jobj == NULL || env == NULL || jvm == NULL)
       return;    

    jclass cls = NULL;
    jmethodID method = NULL;
    jclass clsd = NULL;

    try
    {
        cls = env->GetObjectClass(jobj);
        if (cls == NULL)
            throw 1;

        method = env->GetMethodID(cls, "monitorEvent", "(Ljava/lang/String;I)V");
        if (method == NULL)
            throw 1;

        env->CallObjectMethod(jobj, method, env->NewStringUTF(id), lStatus);            
    }
    catch(...)
    {}

    if (jvm)
        jvm->DetachCurrentThread();
}

void CallJavaStatusEvent(jobject jobj, long lStatus)
{ 
    JavaVM *jvm = NULL;
    JNIEnv *env = NULL;    
    jint   size = 0;

    JNI_GetCreatedJavaVMs(&jvm, 1, &size);    
    if (jvm != NULL)
        jvm->AttachCurrentThread((void **)&env, NULL);

    if (jobj == NULL || env == NULL || jvm == NULL)
       return;    

    jclass cls = NULL;
    jmethodID method = NULL;
    jclass clsd = NULL;

    try
    {
        cls = env->GetObjectClass(jobj);
        if (cls == NULL)
            throw 1;

        method = env->GetMethodID(cls, "statusEvent", "(I)V");
        if (method == NULL)
            throw 1;

        env->CallObjectMethod(jobj, method, lStatus);            
    }
    catch(...)
    {
    }

    if (jvm)
        jvm->DetachCurrentThread();
}

void CallJavaImageEvent(JNIEnv *env, jobject jobj, jint width, jint height, jbyteArray image, jint resolution)
{ 
    if (jobj == NULL || env == NULL || image == NULL)
       return;    

    jclass cls = NULL;
    jmethodID method = NULL;
    jclass clsd = NULL;

    try
    {
        cls = env->GetObjectClass(jobj);
        if (cls == NULL)
        {
            throw 1;
        }            

        method = env->GetMethodID(cls, "imageEvent", "(II[BI)V");
        if (method == NULL)
        {
            throw 2;
        }

        env->CallObjectMethod(jobj, method, width, height, image, resolution);            
    }
    catch(...)
    {}
}

VOID SM_CALLBACK DeviceStatusCallBack(const CHAR * scanner_id, DWORD event, VOID * param)
{
    if(monitoring)
    {
        DWORD event = 0;
        switch(event)
        {
            case SM_EVENT_PLUGGED: 
                event = SM_EVENT_PLUGGED; 
                break;

            case SM_EVENT_UNPLUGGED: 
                event = SM_EVENT_UNPLUGGED; 
                break;

            case SM_EVENT_ERROR: 
                event = SM_EVENT_ERROR;
                break;
        }

        if(event)
        {
            Monitor *monitor = (Monitor*)param;

            if (monitor != NULL)
                CallJavaMonitorEvent(monitor->jobj, (char*) scanner_id, event);            
        }
    }
}

void SM_CALLBACK ImageCallBack(const CHAR * scanner_id, INT lWidth, INT lHeight, const BYTE * pbtImage, INT resolution, VOID * pParam)
{
    Capturing *capturing = (Capturing*)pParam;
    if((capturing != NULL) && (capturing->jobj != NULL))
    {
        JavaVM *jvm = NULL;
        JNIEnv *env = NULL;    
        jint   size = 0;
    
        JNI_GetCreatedJavaVMs(&jvm, 1, &size);    
        if (jvm != NULL)
            jvm->AttachCurrentThread((void **)&env, NULL);

        if (env)
        {
            jbyteArray jimage = env->NewByteArray(lWidth*lHeight);
            env->SetByteArrayRegion(jimage, 0, lWidth*lHeight, (jbyte*)pbtImage);

            if (jimage != NULL)
                CallJavaImageEvent(env, capturing->jobj, lWidth, lHeight, jimage, 500);

            env->DeleteLocalRef(jimage);
            jimage = NULL;
        }

        if (jvm)
            jvm->DetachCurrentThread();
    }
}

void SM_CALLBACK StatusCallBack(const CHAR * scanner_id, DWORD lStatus, VOID * pParam)
{
    Capturing *capturing = (Capturing*)pParam;
    if((capturing != NULL) && (capturing->jobj != NULL))
    {
        DWORD state = 0;
        switch(lStatus)
        {
            case SM_STATE_FINGER_DOWN: 
                state = SM_STATE_FINGER_DOWN; 
                break;

            case SM_STATE_FINGER_UP: 
                state = SM_STATE_FINGER_UP; 
                break;
        }
        
        if(state) 
            CallJavaStatusEvent(capturing->jobj, state);                    
    }
}

INT SCANMAN_API SMStopAllCapturing(JNIEnv * env)
{
    SMStopCapturing(NULL);
    return SME_OK;
}

char* ErrorToString(int code) 
{
    switch(code)
    {
        // General
        case SME_OK:
            return "No error";
        case SME_FAILED:
            return "Failed!";
        case SME_OUT_OF_MEMORY:
            return "Out of memory";
        case SME_NOT_INITIALIZED:
            return "Not initialized";
        case SME_ARGUMENT_NULL:
            return "Argument Null";
        case SME_INVALID_ARGUMENT:
            return "Invalid argument";        
        // Parameters
        case SME_INVALID_PARAMETER:
            return "Invalid parameter";        
        case SME_PARAMETER_READ_ONLY:
            return "Parameter read only";        
        // Capturing
        case SME_NOT_CAPTURING:
            return "Not capturing";        
        case SME_ALREADY_CAPTURING:
            return "Already capturing";                
    }

    return "Error";   
}

void ThrowJavaException(JNIEnv* env, char* msg = "Unexpected error in native code - VeriFinger or it wrapper")
{
    jthrowable exc = env->ExceptionOccurred();
    if (exc == NULL) 
    {
        jclass newExcCls = env->FindClass("java/lang/Exception");
        if (newExcCls != NULL) 
        {
            if (msg == NULL)
                env->ThrowNew(newExcCls, "");
            else
                env->ThrowNew(newExcCls, msg);

            env->DeleteLocalRef(newExcCls);           
        }
    }
}

void ThrowScanManException(JNIEnv* env, int code)
{
    jthrowable exc = env->ExceptionOccurred();
    if (exc == NULL) 
    {
        jclass newExcCls = env->FindClass("ScanMan/ScanManException");
        if (newExcCls != NULL) 
        {
            jmethodID constr = env->GetMethodID(newExcCls, "<init>", "(ILjava/lang/String;)V");
            jthrowable exception = (jthrowable)env->NewObject(newExcCls, constr, code, env->NewStringUTF(ErrorToString(code)));                

            env->Throw(exception);
                            
            env->DeleteLocalRef(newExcCls);           
            env->DeleteLocalRef(exception);                       
        }
    }
}

/*
 * Class:     ScanMan_ScanMan
 * Method:    SMInitialize
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_ScanMan_ScanMan_SMInitialize
  (JNIEnv *env, jclass)
{
    try
    {
		SMInitialize();
    }
    catch(...)
    {
        ThrowJavaException(env);
    }    
}

/*
 * Class:     ScanMan_ScanMan
 * Method:    SMFinalize
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_ScanMan_ScanMan_SMFinalize
  (JNIEnv * env, jclass)
{
    try
    {
        if(!init_count) 
        {
            ThrowScanManException(env, SME_NOT_INITIALIZED);                
            return;
        }
        
        if(!--init_count)
        {
            SMStopAllCapturing(env);
            env->DeleteGlobalRef(monitor.jobj);
            SMRemoveMonitor();
            monitoring = false;
            SMFinalize();
        }
        
        //return init_count;
    }
    catch(...)
    {
        ThrowJavaException(env);
    }    
}

/*
 * Class:     ScanMan_ScanMan
 * Method:    getParameterValue
 * Signature: (I)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_ScanMan_ScanMan_getParameterValue
  (JNIEnv * env, jclass, jint parameter)
{
    char result[1000];
    memset(result, 0, 1000);

    try
    {    
        if(!init_count) 
        {
            ThrowScanManException(env, SME_NOT_INITIALIZED);                
            return env->NewStringUTF(result);
        }

        switch(parameter)
        {
            case SMP_TYPE:

                ThrowScanManException(env, SME_INVALID_ARGUMENT);                                    
                break;
                    
            case SMP_NAME: 
                memcpy(result, SM_NAME, strlen(SM_NAME));
                break;

            case SMP_VERSION_HIGH: 
                sprintf(result, "%d", (DWORD) SM_VERSION_HIGH);
                break;               

            case SMP_VERSION_LOW: 
                sprintf(result, "%u", (DWORD) SM_VERSION_LOW);
                break;                

            case SMP_COPYRIGHT:                 
                memcpy(result, SM_COPYRIGHT, strlen(SM_COPYRIGHT));
                break;

            default: 
                
                ThrowScanManException(env, SME_INVALID_PARAMETER);
        }
    }
    catch(...)
    {
        ThrowJavaException(env);
    }    
    
    return env->NewStringUTF(result);
}

/*
 * Class:     ScanMan_ScanMan
 * Method:    setParameterValue
 * Signature: (ILjava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_ScanMan_ScanMan_setParameterValue
  (JNIEnv * env, jclass, jint parameter, jstring jvalue)
{
    try
    {    
        if(!init_count) 
        {
            ThrowScanManException(env, SME_NOT_INITIALIZED);                
            return;
        }
        
        switch(parameter)
        {
            case SMP_TYPE: 
            case SMP_NAME: 
            case SMP_VERSION_HIGH: 
            case SMP_VERSION_LOW: 
            case SMP_COPYRIGHT: 
                ThrowScanManException(env, SME_PARAMETER_READ_ONLY);
                return;
            default: 
                ThrowScanManException(env, SME_INVALID_PARAMETER);
                return;
        }
    }
    catch(...)
    {
        ThrowJavaException(env);
    }    
}

/*
 * Class:     ScanMan_ScanMan
 * Method:    getParameterType
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_ScanMan_ScanMan_getParameterType
  (JNIEnv * env, jclass, jint parameter)
{
    try
    {    
        switch(parameter)
        {
            case SMP_TYPE: 
                return SM_TYPE_VOID;

            case SMP_NAME:
            case SMP_COPYRIGHT:
                return SM_TYPE_STRING;

            case SMP_VERSION_HIGH:
            case SMP_VERSION_LOW:
                return SM_TYPE_DWORD;

            default: 
                ThrowScanManException(env, SME_INVALID_ARGUMENT);
                return SME_INVALID_ARGUMENT;
        }
    }
    catch(...)
    {
        ThrowJavaException(env);
    }    

    return SME_INVALID_ARGUMENT;
}

/*
 * Class:     ScanMan_ScanMan
 * Method:    SMGetScannerCount
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_ScanMan_ScanMan_SMGetScannerCount
(JNIEnv * env, jclass)
{
    int result = 0;

    try
    {
        if(!init_count) 
        {
            ThrowScanManException(env, SME_NOT_INITIALIZED);
            return result;
        }
        
        result = SMGetScannerCount();
    }
    catch(...)
    {
        ThrowJavaException(env);
    }    
    
    return result;
}

/*
 * Class:     ScanMan_ScanMan
 * Method:    SMGetScannerId
 * Signature: (I)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_ScanMan_ScanMan_SMGetScannerId
  (JNIEnv * env, jclass, jint index)
{
    char result[1000];
    memset(result, 0, 1000);
    int len = 1000;

    try
    {
        if(!init_count) 
        {
            ThrowScanManException(env, SME_NOT_INITIALIZED);
            return env->NewStringUTF(result);
        }
        
		SMGetScannerId(index, result, 1000);
        
        return env->NewStringUTF(result);
    }
    catch(...)
    {
        ThrowJavaException(env);
    }    

    return env->NewStringUTF(result);
}

/*
 * Class:     ScanMan_ScanMan
 * Method:    SMSetMonitor
 * Signature: (LScanMan/ScannersMonitor;)V
 */
JNIEXPORT void JNICALL Java_ScanMan_ScanMan_SMSetMonitor
  (JNIEnv * env, jclass, jobject jmonitor)
{
    try
    {
        if(!init_count) 
        {
            ThrowScanManException(env, SME_NOT_INITIALIZED);
            return;
        }

        if(jmonitor == NULL) 
        {
            ThrowScanManException(env, SME_ARGUMENT_NULL);
            return;
        }

        if(monitoring)
        {
            env->DeleteGlobalRef(monitor.jobj);
            SMRemoveMonitor();
            monitoring = false;
        }

        if(FAILED(SMSetMonitor((SM_MONITOR_PROC*)DeviceStatusCallBack, &monitor))) 
            ThrowScanManException(env, SME_FAILED);
        else
        {            
            monitor.jobj = env->NewGlobalRef(jmonitor);        
            monitoring = true;                
        }
    }
    catch(...)
    {
        ThrowJavaException(env);
    }    
}

/*
 * Class:     ScanMan_ScanMan
 * Method:    SMRemoveMonitor
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_ScanMan_ScanMan_SMRemoveMonitor
  (JNIEnv * env, jclass)
{
    try
    {
        if(!init_count) 
        {
            ThrowScanManException(env, SME_NOT_INITIALIZED);
            return;
        }

        if(monitoring)
        {
            env->DeleteGlobalRef(monitor.jobj);
            SMRemoveMonitor();
            monitoring = false;
        }
    }
    catch(...)
    {
        ThrowJavaException(env);
    }    
}

/*
 * Class:     ScanMan_ScanMan
 * Method:    SMStartCapturing
 * Signature: (Ljava/lang/String;LScanMan/ScannerEventListener;)V
 */
JNIEXPORT void JNICALL Java_ScanMan_ScanMan_SMStartCapturing
  (JNIEnv * env, jclass, jstring jid, jobject jobj)
{
    try
    {
        if(!init_count) 
        {
            ThrowScanManException(env, SME_NOT_INITIALIZED);            
            return;
        }

        if(jid == NULL) 
        {
            ThrowScanManException(env, SME_ARGUMENT_NULL);            
            return;
        }

        const char* scanner_id = env->GetStringUTFChars(jid, NULL);         

		Capturing *capturing = new Capturing;
		memset(capturing, 0, sizeof(Capturing));

        if(FAILED(SMStartCapturing(scanner_id, (SM_IMAGE_PROC*)ImageCallBack, (SM_STATE_PROC*)StatusCallBack, capturing)))
        {
            goto err;
        }
        
        capturing->jobj = env->NewGlobalRef(jobj);

        capturings.push_back(capturing);
        env->ReleaseStringUTFChars(jid, scanner_id);
        return;
err1:
        delete[] capturing->scanner_id;        
err:
        delete capturing;        
        env->ReleaseStringUTFChars(jid, scanner_id);
        ThrowScanManException(env, SME_FAILED);
    }
    catch(...)
    {
        ThrowJavaException(env);
    }    
}

/*
 * Class:     ScanMan_ScanMan
 * Method:    SMStopCapturing
 * Signature: (Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_ScanMan_ScanMan_SMStopCapturing
  (JNIEnv * env, jclass, jstring jid)
{
    try
    {
        if(!init_count) 
        {
            ThrowScanManException(env, SME_NOT_INITIALIZED);
            return;
        }

        if(jid == NULL) 
        {
            ThrowScanManException(env, SME_ARGUMENT_NULL);
            return;
        }        

        const char* scanner_id = env->GetStringUTFChars(jid, NULL);         

        if(!scanner_id) 
        {            
            SMStopAllCapturing(env);
            env->ReleaseStringUTFChars(jid, scanner_id);
            return;
        }

        if(strlen(scanner_id) == 0) 
        {            
            SMStopAllCapturing(env);
            env->ReleaseStringUTFChars(jid, scanner_id);
            return;
        }

        Capturings::iterator i;
        for(i = capturings.begin(); i < capturings.end(); i++)
            if(!strcmp(scanner_id, (*i)->scanner_id)) 
                break;

        if(i == capturings.end()) 
        {
            env->ReleaseStringUTFChars(jid, scanner_id);
            ThrowScanManException(env, SME_NOT_CAPTURING);
            return;
        }

		SMStopCapturing(scanner_id);

        if (env)
            env->DeleteGlobalRef((*i)->jobj);

        delete[] (*i)->scanner_id;
        delete *i;
        capturings.erase(i);

        env->ReleaseStringUTFChars(jid, scanner_id);
    }
    catch(...)
    {
        ThrowJavaException(env);
    }    
}
