#include "sm_null.h"
#include "ScanMan_ScanMan.h"
#include <string.h>



#define SM_NAME "FAKE ScanMan Java Wrapper"

#define MAKEDWORD(high, low) (((DWORD)(high) << 16) | (DWORD)(low))
#define SM_VERSION_HIGH         MAKEDWORD(0, 1)
#define SM_VERSION_LOW          MAKEDWORD(0, 1)
#define SM_COPYRIGHT "Copyright (c) 2002-2006 Neurotechnologija"

// if this is set initialize will fail
static int fail_initialize = 0;
// Number of times SMInitialize() was called. The same number of times SMFinalize() should be called
static int init_count = 0;


char* ErrorToString(int code)
{
    switch(code)
    {
        // General
        case SME_OK:
            return "No error";
        case SME_FAILED:
            return "Failed!";
        case SME_OUT_OF_MEMORY:
            return "Out of memory";
        case SME_NOT_INITIALIZED:
            return "Not initialized";
        case SME_ARGUMENT_NULL:
            return "Argument Null";
        case SME_INVALID_ARGUMENT:
            return "Invalid argument";
        // Parameters
        case SME_INVALID_PARAMETER:
            return "Invalid parameter";
        case SME_PARAMETER_READ_ONLY:
            return "Parameter read only";
        // Capturing
        case SME_NOT_CAPTURING:
            return "Not capturing";
        case SME_ALREADY_CAPTURING:
            return "Already capturing";
    }

    return "Error";
}

void ThrowJavaException(JNIEnv* env, char* msg = "Unexpected error in native code - VeriFinger or it wrapper")
{
    jthrowable exc = env->ExceptionOccurred();
    if (exc == NULL)
    {
        jclass newExcCls = env->FindClass("java/lang/Exception");
        if (newExcCls != NULL)
        {
            if (msg == NULL)
                env->ThrowNew(newExcCls, "");
            else
                env->ThrowNew(newExcCls, msg);

            env->DeleteLocalRef(newExcCls);
        }
    }
}

void ThrowScanManException(JNIEnv* env, int code)
{
    jthrowable exc = env->ExceptionOccurred();
    if (exc == NULL)
    {
        jclass newExcCls = env->FindClass("ScanMan/ScanManException");
        if (newExcCls != NULL)
        {
            jmethodID constr = env->GetMethodID(newExcCls, "<init>", "(ILjava/lang/String;)V");
            jthrowable exception = (jthrowable)env->NewObject(newExcCls, constr, code, env->NewStringUTF(ErrorToString(code)));

            env->Throw(exception);

            env->DeleteLocalRef(newExcCls);
            env->DeleteLocalRef(exception);
        }
    }
}

/*
 * Class:     ScanMan_ScanMan
 * Method:    SMInitialize
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_ScanMan_ScanMan_SMInitialize
  (JNIEnv *env, jclass)
{
    int fail_init = 1;

    try
    {
        if(!init_count)
        {

            if(fail_initialize)
            {
                ThrowScanManException(env, SME_FAILED);
            }
        }

        init_count++;
    }
    catch(...)
    {
        ThrowJavaException(env);
    }
}

/*
 * Class:     ScanMan_ScanMan
 * Method:    SMFinalize
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_ScanMan_ScanMan_SMFinalize
  (JNIEnv * env, jclass)
{
    try
    {
        if(!init_count)
        {
            ThrowScanManException(env, SME_NOT_INITIALIZED);
            return;
        }
    }
    catch(...)
    {
        ThrowJavaException(env);
    }
}

/*
 * Class:     ScanMan_ScanMan
 * Method:    getParameterValue
 * Signature: (I)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_ScanMan_ScanMan_getParameterValue
  (JNIEnv * env, jclass, jint parameter)
{
    char result[1000];
    memset(result, 0, 1000);

    try
    {
        if(!init_count)
        {
            ThrowScanManException(env, SME_NOT_INITIALIZED);
            return env->NewStringUTF(result);
        }

        switch(parameter)
        {
            case SMP_TYPE:

                ThrowScanManException(env, SME_INVALID_ARGUMENT);
                break;

            case SMP_NAME:
                memcpy(result, SM_NAME, strlen(SM_NAME));
                break;

            case SMP_VERSION_HIGH:
                sprintf(result, "%d", (DWORD) SM_VERSION_HIGH);
                break;

            case SMP_VERSION_LOW:
                sprintf(result, "%u", (DWORD) SM_VERSION_LOW);
                break;

            case SMP_COPYRIGHT:
                memcpy(result, SM_COPYRIGHT, strlen(SM_COPYRIGHT));
                break;

            default:

                ThrowScanManException(env, SME_INVALID_PARAMETER);
        }
    }
    catch(...)
    {
        ThrowJavaException(env);
    }

    return env->NewStringUTF(result);
}

/*
 * Class:     ScanMan_ScanMan
 * Method:    setParameterValue
 * Signature: (ILjava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_ScanMan_ScanMan_setParameterValue
  (JNIEnv * env, jclass, jint parameter, jstring jvalue)
{
    try
    {
        if(!init_count)
        {
            ThrowScanManException(env, SME_NOT_INITIALIZED);
            return;
        }

        switch(parameter)
        {
            case SMP_TYPE:
            case SMP_NAME:
            case SMP_VERSION_HIGH:
            case SMP_VERSION_LOW:
            case SMP_COPYRIGHT:
                ThrowScanManException(env, SME_PARAMETER_READ_ONLY);
                return;
            default:
                ThrowScanManException(env, SME_INVALID_PARAMETER);
                return;
        }
    }
    catch(...)
    {
        ThrowJavaException(env);
    }
}

/*
 * Class:     ScanMan_ScanMan
 * Method:    getParameterType
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_ScanMan_ScanMan_getParameterType
  (JNIEnv * env, jclass, jint parameter)
{
    try
    {
        switch(parameter)
        {
            case SMP_TYPE:
                return SM_TYPE_VOID;

            case SMP_NAME:
            case SMP_COPYRIGHT:
                return SM_TYPE_STRING;

            case SMP_VERSION_HIGH:
            case SMP_VERSION_LOW:
                return SM_TYPE_DWORD;

            default:
                ThrowScanManException(env, SME_INVALID_ARGUMENT);
                return SME_INVALID_ARGUMENT;
        }
    }
    catch(...)
    {
        ThrowJavaException(env);
    }

    return SME_INVALID_ARGUMENT;
}

/*
 * Class:     ScanMan_ScanMan
 * Method:    SMGetScannerCount
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_ScanMan_ScanMan_SMGetScannerCount
(JNIEnv * env, jclass)
{
    int result = 0;

    try
    {
        if(!init_count)
        {
            ThrowScanManException(env, SME_NOT_INITIALIZED);
            return result;
        }

	return 0;
//        ThrowScanManException(env, SME_FAILED);
    }
    catch(...)
    {
        ThrowJavaException(env);
    }

    return result;
}

/*
 * Class:     ScanMan_ScanMan
 * Method:    SMGetScannerId
 * Signature: (I)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_ScanMan_ScanMan_SMGetScannerId
  (JNIEnv * env, jclass, jint index)
{
    char result[1000];
    memset(result, 0, 1000);
    int len = 1000;

    try
    {
        if(!init_count)
        {
            ThrowScanManException(env, SME_NOT_INITIALIZED);
            return env->NewStringUTF(result);
        }

        ThrowScanManException(env, SME_FAILED);
    }
    catch(...)
    {
        ThrowJavaException(env);
    }

    return env->NewStringUTF(result);
}

/*
 * Class:     ScanMan_ScanMan
 * Method:    SMSetMonitor
 * Signature: (LScanMan/ScannersMonitor;)V
 */
JNIEXPORT void JNICALL Java_ScanMan_ScanMan_SMSetMonitor
  (JNIEnv * env, jclass, jobject jmonitor)
{
    try
    {
        if(!init_count)
        {
            ThrowScanManException(env, SME_NOT_INITIALIZED);
            return;
        }

        ThrowScanManException(env, SME_FAILED);
    }
    catch(...)
    {
        ThrowJavaException(env);
    }
}

/*
 * Class:     ScanMan_ScanMan
 * Method:    SMRemoveMonitor
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_ScanMan_ScanMan_SMRemoveMonitor
  (JNIEnv * env, jclass)
{
    try
    {
        if(!init_count)
        {
            ThrowScanManException(env, SME_NOT_INITIALIZED);
            return;
        }

        ThrowScanManException(env, SME_FAILED);
    }
    catch(...)
    {
        ThrowJavaException(env);
    }
}

/*
 * Class:     ScanMan_ScanMan
 * Method:    SMStartCapturing
 * Signature: (Ljava/lang/String;LScanMan/ScannerEventListener;)V
 */
JNIEXPORT void JNICALL Java_ScanMan_ScanMan_SMStartCapturing
  (JNIEnv * env, jclass, jstring jid, jobject jobj)
{
    try
    {
        if(!init_count)
        {
            ThrowScanManException(env, SME_NOT_INITIALIZED);
            return;
        }

        ThrowScanManException(env, SME_FAILED);
    }
    catch(...)
    {
        ThrowJavaException(env);
    }
}

/*
 * Class:     ScanMan_ScanMan
 * Method:    SMStopCapturing
 * Signature: (Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_ScanMan_ScanMan_SMStopCapturing
  (JNIEnv * env, jclass, jstring jid)
{
    try
    {
        if(!init_count)
        {
            ThrowScanManException(env, SME_NOT_INITIALIZED);
            return;
        }

        ThrowScanManException(env, SME_FAILED);
    }
    catch(...)
    {
        ThrowJavaException(env);
    }
}
