//#if defined(linux) || defined (__LINUX__)
//	#include "linux_include.h"
//#else
	#include "stdafx.h"
//#endif

typedef BYTE* BYTEPTR;
/*
 * Class:     VeriFinger_VeriFingerWrapper
 * Method:    VFGeneralize
 * Signature: (I[[B[BI)I
 *
 * public static native int VFGeneralize(int count, byte[][] gen_features,
                                         byte[] features, int context) throws VeriFingerException;
 *
 */
JNIEXPORT jobject JNICALL Java_VeriFinger_VeriFingerWrapper_VFGeneralize
  (JNIEnv * env, jclass jcls, jint count, jobjectArray jgen_features, jbyteArray jfeatures, jint context)
{
    int res = VFE_OK;

    try
    {
        // get features
        jbyte* features = env->GetByteArrayElements(jfeatures, NULL);
        if (features == NULL || jfeatures == NULL)
            ThrowJavaIllegalArgumentException(env);
        else
        {
            unsigned long size = 0;
            bool error = false;

            int i;

            BYTEPTR *gen_features = new BYTEPTR[count];
            jobject *obj = new jobject[count];

            for (i = 0; i < count; i++)
            {
                obj[i] = env->GetObjectArrayElement(jgen_features, i);
                if (obj[i] == NULL)
                {
                    error = true;
                    continue;
                }

                jbyte* el = env->GetByteArrayElements((jbyteArray)obj[i], NULL);
                gen_features[i] = (BYTE*)el;

                if (el == NULL)
                {
                    error = true;
                    continue;
                }
            }

            if (!error)
                res = VFGeneralize(count, gen_features, (BYTE *) features, &size, (HVFCONTEXT) context);
            else
                res = VFE_FAILED;

            for (i = 0; i < count; i++)
            {
                env->ReleaseByteArrayElements((jbyteArray)obj[i], (jbyte*)gen_features[i], 0);
                env->DeleteLocalRef(obj[i]);
            }

            delete[] gen_features;
            delete[] obj;

            if VFFailed(res)
                ThrowVeriFingerException(env, res);

            // release features
            env->ReleaseByteArrayElements(jfeatures, features, 0);

            // preparing result
            jclass newExcCls = env->FindClass("VeriFinger/VeriFingerGeneralizationResult");
            if (newExcCls == NULL)
            {
                ThrowJavaClassNotFoundException(env);
            }
            else
            {
                jmethodID constr = env->GetMethodID(newExcCls, "<init>", "(II)V");
                jobject result = (jobject)env->NewObject(newExcCls, constr, (jint) size, (jint) res);

                if (result == NULL)
                    ThrowJavaException(env);

                env->DeleteLocalRef(newExcCls);

                return result;
            }
        }
    }
    catch(...)
    {
        ThrowJavaException(env);
    }

    return NULL;
}
