//#if defined(linux) || defined (__LINUX__)
//	#include "linux_include.h"
//#else
	#include "stdafx.h"
//#endif

jobject FillMatchingResult(JNIEnv * env, VFMATCHDETAILSEX &md, jboolean extended_match_info)
{
    jclass newExcCls = env->FindClass("VeriFinger/VeriFingerMatchDetails");
    if (newExcCls == NULL)
    {
        ThrowJavaClassNotFoundException(env);
        return NULL;
    }

    jmethodID constr = env->GetMethodID(newExcCls, "<init>", "()V");
    jobject result = (jobject) env->NewObject(newExcCls, constr);

    if (result == NULL)
    {
        env->DeleteLocalRef(newExcCls);
        ThrowJavaException(env);

        return NULL;
    }

    jfieldID fid_similarity = env->GetFieldID(newExcCls, "similarity",    "I");
    jfieldID fid_rotation   = env->GetFieldID(newExcCls, "rotation",      "I");
    jfieldID fid_trans_x    = env->GetFieldID(newExcCls, "trans_x",       "I");
    jfieldID fid_trans_y    = env->GetFieldID(newExcCls, "trans_y",       "I");
    jfieldID fid_mm_count   = env->GetFieldID(newExcCls, "mm_count",      "I");

    if (fid_similarity == NULL ||
        fid_rotation   == NULL ||
        fid_trans_x    == NULL ||
        fid_trans_y    == NULL ||
        fid_mm_count   == NULL)
    {
        env->DeleteLocalRef(newExcCls);
        ThrowJavaException(env);

        return NULL;
    }

    env->SetIntField(result, fid_similarity, md.similarity);
    env->SetIntField(result, fid_rotation,   md.rotation);
    env->SetIntField(result, fid_trans_x,    md.trans_x);
    env->SetIntField(result, fid_trans_y,    md.trans_y);

    if (!extended_match_info)
    {
        env->SetIntField(result, fid_mm_count, 0);
    }
    else
    {
        // extented info

        env->SetIntField(result, fid_mm_count,    md.mm_count);

        jfieldID fid_mm = env->GetFieldID(newExcCls, "mm", "[[S");
        if (fid_mm == NULL)
        {
            env->DeleteLocalRef(newExcCls);
            ThrowJavaException(env);

            return NULL;
        }

        jobjectArray amm = (jobjectArray) env->GetObjectField(result, fid_mm);
        if (amm == NULL)
        {
            env->DeleteLocalRef(newExcCls);
            ThrowJavaException(env);

            return NULL;
        }

        for (int i = 0; i < 2; i++)
        {
            jobject obj = env->GetObjectArrayElement(amm, i);
            if (obj == NULL)
            {
                env->DeleteLocalRef(amm);

                env->DeleteLocalRef(newExcCls);
                ThrowJavaException(env);

                return NULL;
            }

            jshort* el = env->GetShortArrayElements((jshortArray)obj, NULL);

            if (el == NULL)
            {
                env->DeleteLocalRef(amm);
                env->DeleteLocalRef(obj);

                env->DeleteLocalRef(newExcCls);
                ThrowJavaException(env);

                return NULL;
            }

            memcpy(el, md.mm[i], sizeof(short)*md.mm_count);
            env->ReleaseShortArrayElements((jshortArray)obj, el, 0);

            env->DeleteLocalRef(obj);
        }

        env->DeleteLocalRef(amm);
    }

    env->DeleteLocalRef(newExcCls);

    return result;
}
/* Class:     VeriFinger_VeriFingerWrapper
 * Method:    VFIdentifyStart
 * Signature: ([BI)V

   public static native void VFIdentifyStart(byte[] test_features, int context) throws VeriFingerException;

 */

JNIEXPORT void JNICALL Java_VeriFinger_VeriFingerWrapper_VFIdentifyStart
  (JNIEnv * env, jclass cls, jbyteArray jtest_features, jint context)
{
    int res = VFE_OK;

    try
    {
        // get features
        jbyte* test_features = env->GetByteArrayElements(jtest_features, NULL);
        if (jtest_features == NULL || test_features == NULL)
            ThrowJavaIllegalArgumentException(env);
        else
        {
            res = VFIdentifyStart((const BYTE *)test_features, (HVFCONTEXT) context);

            if VFFailed(res)
                ThrowVeriFingerException(env, res);

            // release features
            env->ReleaseByteArrayElements(jtest_features, test_features, 0);
        }
    }
    catch(...)
    {
        ThrowJavaException(env);
    }
}

/*
 * Class:     VeriFinger_VeriFingerWrapper
 * Method:    VFIdentifyNext
 * Signature: ([BZI)LVeriFinger/VeriFingerMatchDetails;

   public static native VeriFingerMatchDetails VFIdentifyNext(byte[] sample_features, boolean extended_match_info,
                                                    int context) throws VeriFingerException, Exception;

 */
JNIEXPORT jobject JNICALL Java_VeriFinger_VeriFingerWrapper_VFIdentifyNext
  (JNIEnv * env, jclass jcls, jbyteArray jsample_features, jboolean extended_match_info, jint context)
{
    int res = VFE_OK;

    try
    {
        // get features
        jbyte* sample_features = env->GetByteArrayElements(jsample_features, NULL);
        if (sample_features == NULL || jsample_features == NULL)
            ThrowJavaIllegalArgumentException(env);
        else
        {
            VFMATCHDETAILSEX md;

            memset(&md, 0, sizeof(VFMATCHDETAILSEX));

            if (extended_match_info)
                md.size = sizeof(VFMATCHDETAILSEX);
            else
                md.size = sizeof(VFMATCHDETAILS);

            res = VFIdentifyNext((const BYTE *)sample_features, (VFMATCHDETAILS *) &md, (HVFCONTEXT) context);

            // release features
            env->ReleaseByteArrayElements(jsample_features, sample_features, 0);

            if (VFFailed(res) && (res != VFE_FAILED))
                ThrowVeriFingerException(env, res);

            return FillMatchingResult(env, md, extended_match_info);
        }
    }
    catch(...)
    {
        ThrowJavaException(env);
    }

    return NULL;
}


/*
 * Class:     VeriFinger_VeriFingerWrapper
 * Method:    VFIdentifyEnd
 * Signature: (I)V


  public static native void VFIdentifyEnd(int context) throws VeriFingerException;

 */

JNIEXPORT void JNICALL Java_VeriFinger_VeriFingerWrapper_VFIdentifyEnd
  (JNIEnv * env, jclass cls, jint context)
{
    int res = VFE_OK;

    try
    {
        res = VFIdentifyEnd((HVFCONTEXT)context);
        if VFFailed(res)
            ThrowVeriFingerException(env, res);
    }
    catch(...)
    {
        ThrowJavaException(env);
    }
}

/*
 * Class:     VeriFinger_VeriFingerWrapper
 * Method:    VFVerify
 * Signature: ([B[BZI)LVeriFinger/VeriFingerMatchDetails;

  public static native VeriFingerMatchDetails VFVerify(byte[] features1,
  byte[] features2, boolean extended_match_info, int context) throws VeriFingerException, Exception;

 */
JNIEXPORT jobject JNICALL Java_VeriFinger_VeriFingerWrapper_VFVerify
  (JNIEnv * env, jclass jcls, jbyteArray jfeatures1, jbyteArray jfeatures2, jboolean extended_match_info, jint context)
{
    int res = VFE_OK;

    try
    {
        // get features
        jbyte* features1 = env->GetByteArrayElements(jfeatures1, NULL);
        jbyte* features2 = env->GetByteArrayElements(jfeatures2, NULL);

        if (jfeatures1 == NULL || features1 == NULL ||
            jfeatures2 == NULL || features2 == NULL)
        {
            ThrowJavaIllegalArgumentException(env);
        }
        else
        {
            VFMATCHDETAILSEX md;

            memset(&md, 0, sizeof(VFMATCHDETAILSEX));

            if (extended_match_info)
                md.size = sizeof(VFMATCHDETAILSEX);
            else
                md.size = sizeof(VFMATCHDETAILS);

            res = VFVerify((const BYTE *)features1, (const BYTE *)features2, (VFMATCHDETAILS *) &md, (HVFCONTEXT) context);

            if (VFFailed(res) && (res != VFE_FAILED))
                ThrowVeriFingerException(env, res);

            // release features
            env->ReleaseByteArrayElements(jfeatures1, features1, 0);
            env->ReleaseByteArrayElements(jfeatures2, features2, 0);

            return FillMatchingResult(env, md, extended_match_info);
        }
    }
    catch(...)
    {
        ThrowJavaException(env);
    }

    return NULL;
}
