//#if defined(linux) || defined (__LINUX__)
//	#include "linux_include.h"
//#else
	#include "stdafx.h"
//#endif

#define VF_MAX_PARAMETER_LENGTH 1000

/*
 * Class:     VeriFinger_VeriFingerWrapper
 * Method:    getParameterType
 * Signature: (I)I
 */

JNIEXPORT jint JNICALL Java_VeriFinger_VeriFingerWrapper_getParameterType
  (JNIEnv * env, jclass cls, jint parameter)
{
    int res = VFE_OK;

    try
    {
        res = VFGetParameter(VFP_TYPE, (void*)parameter, NULL);

        if VFFailed(res)
            ThrowVeriFingerException(env, res);
    }
    catch(...)
    {
        ThrowJavaException(env);
    }

    return res;
}

/*
 * Class:     VeriFinger_VeriFingerWrapper
 * Method:    getParameter
 * Signature: (II)Ljava/lang/String;
 */

JNIEXPORT jstring JNICALL Java_VeriFinger_VeriFingerWrapper_getParameterValue
  (JNIEnv * env, jclass cls, jint parameter, jint context)
{
    int res = VFE_OK;

    char result[VF_MAX_PARAMETER_LENGTH];
    memset(result, 0, VF_MAX_PARAMETER_LENGTH);

    try
    {
        int ptype = VFGetParameter(VFP_TYPE, (void*)parameter, NULL);

        if VFFailed(ptype)
        {
            res = ptype;
            ThrowVeriFingerException(env, res);
            return env->NewStringUTF(result);
        }

        switch (ptype)
        {
            case VF_TYPE_BYTE:
                {
                    BYTE byteValue;

                    res = VFGetParameter(parameter, (void*)&byteValue, (HVFCONTEXT) context);
                    sprintf(result, "%d", (int) byteValue);
                    break;
                }

            case VF_TYPE_SBYTE:
                {
                    SBYTE sbyteValue;

                    res = VFGetParameter(parameter, (void*)&sbyteValue, (HVFCONTEXT) context);
                    sprintf(result, "%d", (int) sbyteValue);
                    break;
                }

            case VF_TYPE_WORD:
                {
                    WORD wordValue;

                    res = VFGetParameter(parameter, (void*)&wordValue, (HVFCONTEXT) context);
                    sprintf(result, "%d", (int) wordValue);
                    break;
                }

            case VF_TYPE_SHORT:
                {
                    SHORT shortValue;

                    res = VFGetParameter(parameter, (void*)&shortValue, (HVFCONTEXT) context);
                    sprintf(result, "%d", (int) shortValue);
                    break;
                }

            case VF_TYPE_DWORD:
                {
                    DWORD dwordValue;

                    res = VFGetParameter(parameter, (void*)&dwordValue, (HVFCONTEXT) context);
                    sprintf(result, "%u", dwordValue);
                    break;
                }

            case VF_TYPE_INT:
                {
                    int intValue;

                    res = VFGetParameter(parameter, (void*)&intValue, (HVFCONTEXT) context);
                    sprintf(result, "%d", intValue);
                    break;
                }

            case VF_TYPE_BOOL:
                {
                    BOOL boolValue;

                    res = VFGetParameter(parameter, (void*)&boolValue, (HVFCONTEXT) context);
                    if (boolValue)
                        sprintf(result, "true");
                    else
                        sprintf(result, "false");

                    break;
                }

            case VF_TYPE_CHAR:
                {
                    CHAR charValue;

                    res = VFGetParameter(parameter, (void*)&charValue, (HVFCONTEXT) context);
                    sprintf(result, "%c", charValue);
                    break;
                }

            case VF_TYPE_STRING:
                {
                    int l = VFGetParameter(parameter, NULL, NULL);
                    if (l > 0)
                    {
                        char* strValue = new char[l+1];

                        memset(strValue, 0, l+1);

                        res = VFGetParameter(parameter, (void*)strValue, (HVFCONTEXT) context);
                        sprintf(result, "%s", strValue);

                        delete[] strValue;
                    }
                    else
                        res = l;

                    break;
                }

            default:
                res = VFE_INVALID_PARAMETER;
        }

        if VFFailed(res)
            ThrowVeriFingerException(env, res);
    }
    catch(...)
    {
        ThrowJavaException(env);
    }

    return env->NewStringUTF(result);
}

/*
 * Class:     VeriFinger_VeriFingerWrapper
 * Method:    setParameter
 * Signature: (ILjava/lang/String;I)V
 */

JNIEXPORT void JNICALL Java_VeriFinger_VeriFingerWrapper_setParameterValue
  (JNIEnv * env, jclass cls, jint parameter, jstring jvalue, jint context)
{
    int res = VFE_OK;

    const char* value = env->GetStringUTFChars(jvalue, NULL);

    if (value == NULL || jvalue == NULL)
    {
        ThrowVeriFingerException(env, VFE_INVALID_ARGUMENT);
        return;
    }

    try
    {
        int ptype = VFGetParameter(VFP_TYPE, (void*)parameter, NULL);

        if VFFailed(ptype)
        {
            res = ptype;
            ThrowVeriFingerException(env, res);
            env->ReleaseStringUTFChars(jvalue, value);
            return;
        }

        switch (ptype)
        {
            case VF_TYPE_BYTE:
                {
                    int byteValue;

                    sscanf(value, "%d", &byteValue);

                    res = VFSetParameter(parameter, (DWORD)byteValue, (HVFCONTEXT) context);
                    break;
                }

            case VF_TYPE_SBYTE:
                {
                    int sbyteValue;

                    sscanf(value, "%d", &sbyteValue);
                    res = VFSetParameter(parameter, (DWORD)sbyteValue, (HVFCONTEXT) context);
                    break;
                }

            case VF_TYPE_WORD:
                {
                    int wordValue;

                    sscanf(value, "%d", &wordValue);
                    res = VFSetParameter(parameter, (DWORD)wordValue, (HVFCONTEXT) context);
                    break;
                }

            case VF_TYPE_SHORT:
                {
                    int shortValue;

                    sscanf(value, "%d", &shortValue);
                    res = VFSetParameter(parameter, (DWORD)shortValue, (HVFCONTEXT) context);
                    break;
                }

            case VF_TYPE_DWORD:
                {
                    int dwordValue;

                    sscanf(value, "%u", &dwordValue);
                    res = VFSetParameter(parameter, (DWORD)dwordValue, (HVFCONTEXT) context);
                    break;
                }

            case VF_TYPE_INT:
                {
                    int intValue;

                    sscanf(value, "%d", &intValue);
                    res = VFSetParameter(parameter, (DWORD)intValue, (HVFCONTEXT) context);
                    break;
                }

            case VF_TYPE_BOOL:
                {
                    BOOL boolValue = (_stricmp(value, "true") == 0);
                    res = VFSetParameter(parameter, (DWORD)boolValue, (HVFCONTEXT) context);

                    break;
                }

            case VF_TYPE_CHAR:
                {
                    CHAR charValue;

                    sscanf(value, "%c", &charValue);
                    res = VFSetParameter(parameter, (DWORD)charValue, (HVFCONTEXT) context);
                    break;
                }

            case VF_TYPE_STRING:
                {
                    res = VFSetParameter(parameter, (DWORD)value, (HVFCONTEXT) context);

                    break;
                }

            default:
                res = VFE_INVALID_PARAMETER;
        }

        if VFFailed(res)
            ThrowVeriFingerException(env, res);
    }
    catch(...)
    {
        ThrowJavaException(env);
    }

    env->ReleaseStringUTFChars(jvalue, value);
}
