/********************************************************\
|*                                                      *|
|*  support.h                                           *|
|*    common functions used in wrapper implementation   *|
|*                                                      *|
|*  Copyright (C) 2002-2006, Neurotechnologija          *|
|*                                                      *|
\********************************************************/

#ifndef _SUPPORT_H_
#define _SUPPORT_H_

inline char* VFErrorToString(int code)
{
    switch(code)
    {
        //General
    case VFE_OK:
            return "No error";
    case VFE_FAILED:
            return "Failed!";
    case VFE_OUT_OF_MEMORY:
            return "Out of memory";
    case VFE_NOT_INITIALIZED:
            return "Not initialized";
    case VFE_ARGUMENT_NULL:
            return "Argument NULL";
    case VFE_INVALID_ARGUMENT:
            return "Invalid argument";
    // Registration
    case VFE_NOT_REGISTERED:
            return "Not registered";
    case VFE_INVALID_SERIAL_NUMBER:
            return "Invalid serial number";
    case VFE_INVALID_REGISTRATION_KEY:
            return "Invalid registration key";
    case VFE_SCANNER_DRIVER_ERROR:
            return "Scanner driver error";
    case VFE_REGISTRATION_NOT_NEEDED:
            return "Registration not needed";
    case VFE_NO_SCANNER:
            return "No scanner";
    case VFE_MORE_THAN_ONE_SCANNER:
            return "More than one scanner";
    case VFE_LM_CONNECTION_ERROR:
            return "Communication with License Manager error";
    case VFE_LM_NO_MORE_LICENSES:
            return "License Manager error: no more licenses";
    // Parameters
    case VFE_INVALID_PARAMETER:
            return "Invalid parameter";
    case VFE_PARAMETER_READ_ONLY:
            return "Parameter read-only";
    // Features extraction
    case VFE_ILLEGAL_IMAGE_RESOLUTION:
            return "Illegal image resolution";
    case VFE_ILLEGAL_IMAGE_SIZE:
            return "Illegal image size";
    case VFE_LOW_QUALITY_IMAGE:
            return "Low quality image";
    // VeriFinger specific
    case VFE_INVALID_MODE:
            return "Invalid mode";
    }

    return "Error";
}

inline void ThrowVeriFingerException(JNIEnv* env, int code)
{
    jthrowable exc = env->ExceptionOccurred();
    if (exc == NULL)
    {
        jclass newExcCls = env->FindClass("VeriFinger/VeriFingerException");
        if (newExcCls != NULL)
        {
            jmethodID constr = env->GetMethodID(newExcCls, "<init>", "(ILjava/lang/String;)V");
            jthrowable exception = (jthrowable)env->NewObject(newExcCls, constr, code, env->NewStringUTF(VFErrorToString(code)));

            env->Throw(exception);

            env->DeleteLocalRef(newExcCls);
            env->DeleteLocalRef(exception);
        }
    }
}

inline void ThrowJavaException(JNIEnv* env, char* msg = "Unexpected error in native code - VeriFinger or it wrapper")
{
    jthrowable exc = env->ExceptionOccurred();
    if (exc == NULL)
    {
        jclass newExcCls = env->FindClass("java/lang/Exception");
        if (newExcCls != NULL)
        {
            if (msg == NULL)
                env->ThrowNew(newExcCls, "");
            else
                env->ThrowNew(newExcCls, msg);

            env->DeleteLocalRef(newExcCls);
        }
    }
}

inline void ThrowJavaIllegalArgumentException(JNIEnv* env, char* msg = NULL)
{
    jthrowable exc = env->ExceptionOccurred();

    if (exc == NULL) // throw exception only when no other exception
    {
        jclass newExcCls = env->FindClass("java/lang/IllegalArgumentException");
        if (newExcCls == NULL)
            return;

        if (msg == NULL)
            env->ThrowNew(newExcCls, "");
        else
            env->ThrowNew(newExcCls, msg);

        env->DeleteLocalRef(newExcCls);
    }
}

inline void ThrowJavaClassNotFoundException(JNIEnv* env, char* msg = NULL)
{
    jthrowable exc = env->ExceptionOccurred();

    if (exc == NULL) // throw exception only when no other exception
    {
        jclass newExcCls = env->FindClass("java/lang/ClassNotFoundException");
        if (newExcCls == NULL)
            return;

        if (msg == NULL)
            env->ThrowNew(newExcCls, "");
        else
            env->ThrowNew(newExcCls, msg);

        env->DeleteLocalRef(newExcCls);
    }
}

inline void ThrowJavaRuntimeException(JNIEnv* env, char* msg = NULL)
{
    jthrowable exc = env->ExceptionOccurred();

    if (exc == NULL) // throw exception only when no other exception
    {
        jclass newExcCls = env->FindClass("java/lang/RuntimeException");
        if (newExcCls == NULL)
            return;

        if (msg == NULL)
            env->ThrowNew(newExcCls, "");
        else
            env->ThrowNew(newExcCls, msg);

        env->DeleteLocalRef(newExcCls);
    }
}

#endif //_SUPPORT_H_
