/********************************************************\
|*                                                      *|
|*                  VeriFinger 5.0 SDK                  *|
|*                                                      *|
|*  Image.cpp                                           *|
|*    CImage class implementation                       *|
|*                                                      *|
|*  Copyright (C) 1998-2006 Neurotechnologija           *|
|*                                                      *|
\********************************************************/

#include <windows.h>
#include <stdio.h>
#include <string.h>
#include "image.h"

////////////////////////////////////////////////////////////////////////////
// TIFF reader implemantation
////////////////////////////////////////////////////////////////////////////

// tags

#define TIFF_TAG_NewSubfileType                         254
#define TIFF_TAG_SubfileType                            255
#define TIFF_TAG_ImageWidth                                     256
#define TIFF_TAG_ImageLength                            257
#define TIFF_TAG_BitsPerSample                          258
#define TIFF_TAG_Compression                            259
#define TIFF_TAG_VALUE_Uncompressed                             1
#define TIFF_TAG_VALUE_CCITT_1D                                 2
#define TIFF_TAG_VALUE_Group_3_FAX                              3
#define TIFF_TAG_VALUE_Group_4_FAX                              4
#define TIFF_TAG_VALUE_LZW                                              5
#define TIFF_TAG_VALUE_JPEG                                             6
#define TIFF_TAG_VALUE_PackBits                                 32773
#define TIFF_TAG_PhotometricInterpretation      262
#define TIFF_TAG_VALUE_WhiteIsZero                              0
#define TIFF_TAG_VALUE_BlackIsZero                              1
#define TIFF_TAG_VALUE_RGB                                              2
#define TIFF_TAG_VALUE_RGB_Palette                              3
#define TIFF_TAG_VALUE_Transparency_mask                4
#define TIFF_TAG_VALUE_CMYK                                             5
#define TIFF_TAG_VALUE_YCbCr                                    6
#define TIFF_TAG_VALUE_CIELab                                   8
#define TIFF_TAG_Threshholding                          263
#define TIFF_TAG_CellWidth                                      264
#define TIFF_TAG_CellLength                                     265
#define TIFF_TAG_FillOrder                                      266
#define TIFF_TAG_DocumentName                           269
#define TIFF_TAG_ImageDescription                       270
#define TIFF_TAG_Make                                           271
#define TIFF_TAG_Model                                          272
#define TIFF_TAG_StripOffsets                           273
#define TIFF_TAG_Orientation                            274
#define TIFF_TAG_SamplesPerPixel                        277
#define TIFF_TAG_RowsPerStrip                           278
#define TIFF_TAG_StripByteCounts                        279
#define TIFF_TAG_MinSampleValue                         280
#define TIFF_TAG_MaxSampleValue                         281
#define TIFF_TAG_XResolution                            282
#define TIFF_TAG_YResolution                            283
#define TIFF_TAG_PlanarConfiguration            284
#define TIFF_TAG_PageName                                       285
#define TIFF_TAG_XPosition                                      286
#define TIFF_TAG_YPosition                                      287
#define TIFF_TAG_FreeOffsets                            288
#define TIFF_TAG_FreeByteCounts                         289
#define TIFF_TAG_GrayResponseUnit                       290
#define TIFF_TAG_GrayResponseCurve                      291
#define TIFF_TAG_T4Options                                      292
#define TIFF_TAG_T6Options                                      293
#define TIFF_TAG_ResolutionUnit                         296
#define TIFF_TAG_PageNumber                                     297
#define TIFF_TAG_TransferFunction                       301
#define TIFF_TAG_Software                                       305
#define TIFF_TAG_DateTime                                       306
#define TIFF_TAG_Artist                                         315
#define TIFF_TAG_HostComputer                           316
#define TIFF_TAG_Predictor                                      317
#define TIFF_TAG_WhitePoint                                     318
#define TIFF_TAG_PrimaryChromaticities          319
#define TIFF_TAG_ColorMap                                       320
#define TIFF_TAG_HalftoneHints                          321
#define TIFF_TAG_TileWidth                                      322
#define TIFF_TAG_TileLength                                     323
#define TIFF_TAG_TileOffsets                            324
#define TIFF_TAG_TileByteCounts                         325
#define TIFF_TAG_InkSet                                         332
#define TIFF_TAG_InkNames                                       333
#define TIFF_TAG_NumberOfInks                           334
#define TIFF_TAG_DotRange                                       336
#define TIFF_TAG_TargetPrinter                          337
#define TIFF_TAG_ExtraSamples                           338
#define TIFF_TAG_SampleFormat                           339
#define TIFF_TAG_SMinSampleValue                        340
#define TIFF_TAG_SMaxSampleValue                        341
#define TIFF_TAG_TransferRange                          342
#define TIFF_TAG_JPEGProc                                       512
#define TIFF_TAG_JPEGInterchangeFormat          513
#define TIFF_TAG_JPEGInterchangeFormatLngth     514
#define TIFF_TAG_JPEGRestartInterval            515
#define TIFF_TAG_JPEGLosslessPredictors         517
#define TIFF_TAG_JPEGPointTransforms            518
#define TIFF_TAG_JPEGQTables                            519
#define TIFF_TAG_JPEGDCTables                           520
#define TIFF_TAG_JPEGACTables                           521
#define TIFF_TAG_YCbCrCoefficients                      529
#define TIFF_TAG_YCbCrSubSampling                       530
#define TIFF_TAG_YCbCrPositioning                       531
#define TIFF_TAG_ReferenceBlackWhite            532
#define TIFF_TAG_Copyright                                      33432

// TIFF types
#define TIFF_TYPE_Byte   1
#define TIFF_TYPE_Short  3
#define TIFF_TYPE_Long   4
#define TIFF_TYPE_SByte  6
#define TIFF_TYPE_SShort 8
#define TIFF_TYPE_SLong  9

// TIFF LWZ

#define TIFF_LZW_EoiCode           257
#define TIFF_LZW_ClearCode         256
#define TIFF_LZW_MaxTableEntry 8193

typedef struct TIFF_LZW_TableEntry
{
        t_uint16 Length;
        t_uint8  Value;
        t_uint16 OldCode;
} TIFF_LZW_TableEntry;


t_uint16 CImage::TIFFGetValueByOrder(t_uint16 value)
{
        if (m_tiff_byteorder)
                return (value);
        else
                return ((value << 8) + (value >> 8));
}

t_uint32 CImage::TIFFGetValueByOrder(t_uint32 value)
{
        if (m_tiff_byteorder)
                return (value);
        else
                return ((value << 24) +
                        ((value & 0xFF00) << 8) +
                                ((value & 0xFF0000) >> 8) +
                                ((value & 0xFF000000) >> 24));
}

Tag* CImage::TIFFGetTag(t_uint16 TagType)
{
        t_uint16 i;
        for (i = 0; i < m_tiff_TagsCount; i++)
                if (TagType == TIFFGetValueByOrder(m_tiff_Tags[i].Tag))
                        return (&m_tiff_Tags[i]);

        // default values

        return (NULL);
}

t_uint32 CImage::TIFFGetTagValue(t_uint16 TagType)
{
        Tag* t = TIFFGetTag(TagType);

        if (t != NULL)
        {
                if (TIFFGetValueByOrder(t->Count) == 1)
                {
                        switch(TIFFGetValueByOrder(t->Type))
                        {
                        case TIFF_TYPE_Byte:
                        case TIFF_TYPE_SByte:
                                return ((t_uint32)(t->Value.Byte[0]));

                        case TIFF_TYPE_Short:
                        case TIFF_TYPE_SShort:
                                return ((t_uint32)TIFFGetValueByOrder(t->Value.Short[0]));

                        case TIFF_TYPE_Long:
                        case TIFF_TYPE_SLong:
                                return ((t_uint32)TIFFGetValueByOrder(t->Value.Long));
                        }
                }
        }

        // default values

        switch(TagType)
        {
        case TIFF_TAG_BitsPerSample:
                return (1);
        case TIFF_TAG_Compression:
                return (1);
        case TIFF_TAG_FillOrder:
                return (1);
        case TIFF_TAG_MaxSampleValue:
                return (0); //Default is 2**(BitsPerSample) - 1.
        case TIFF_TAG_MinSampleValue:
                return (0);
        case TIFF_TAG_NewSubfileType:
                return (0);
        case TIFF_TAG_Orientation:
                return (1);
        case TIFF_TAG_PlanarConfiguration:
                return (1);
        case TIFF_TAG_ResolutionUnit:
                return (2);
        case TIFF_TAG_SamplesPerPixel:
                return (1);
        case TIFF_TAG_Threshholding:
                return (1);
        }

        return 0;
}

t_uint32* CImage::TIFFGetTagValues(t_uint16 TagType, FILE * file)
{
        Tag*      t    = TIFFGetTag(TagType);
        t_uint32* data = NULL;

        __try
        {
                if (t != NULL)
                {
                        if (TIFFGetValueByOrder(t->Count) == 0)
                                return (NULL); // no data

                        data = (t_uint32*)calloc(TIFFGetValueByOrder(t->Count), sizeof(t_uint32));
                        if (data == NULL)
                                return (NULL); // no memory

                        t_uint32 i;

                        switch(TIFFGetValueByOrder(t->Type))
                        {
                                case TIFF_TYPE_Byte:
                                case TIFF_TYPE_SByte:
                                        if (TIFFGetValueByOrder(t->Count) <= 4)
                                        {
                                                for (i = 0; i < TIFFGetValueByOrder(t->Count); ++i)
                                                        data[i] = (t_uint32)(t->Value.Byte[i]);
                                        }
                                        else
                                        {
                                                if (fseek(file, TIFFGetValueByOrder(t->Value.Long), SEEK_SET))
                                                        break;

                                                t_uint8 temp;
                                                for (i = 0; i < TIFFGetValueByOrder(t->Count); ++i)
                                                {
                                                        if (fread(&temp, sizeof(t_uint8), 1, file) != 1)
                                                                break;
                                                        data[i] = temp;
                                                }
                                        }
                                        return (data);

                                case TIFF_TYPE_Short:
                                case TIFF_TYPE_SShort:
                                        if (TIFFGetValueByOrder(t->Count) <= 2)
                                        {
                                                for (i = 0; i < TIFFGetValueByOrder(t->Count); ++i)
                                                        data[i] = (t_uint32)(TIFFGetValueByOrder(t->Value.Short[i]));
                                        }
                                        else
                                        {
                                                if (fseek(file, TIFFGetValueByOrder(t->Value.Long), SEEK_SET))
                                                        break;

                                                t_uint16 temp;
                                                for (i = 0; i < TIFFGetValueByOrder(t->Count); ++i)
                                                {
                                                        if (fread(&temp, sizeof(t_uint16), 1, file) != 1)
                                                                break;
                                                    data[i] = TIFFGetValueByOrder(temp);
                                                }
                                        }
                                        return (data);

                                case TIFF_TYPE_Long:
                                case TIFF_TYPE_SLong:
                                        if (TIFFGetValueByOrder(t->Count) <= 1)
                                        {
                                                data[0] = TIFFGetValueByOrder(t->Value.Long);
                                        }
                                        else
                                        {
                                                if (fseek(file, TIFFGetValueByOrder(t->Value.Long), SEEK_SET))
                                                        break;

                                                t_uint32 temp;
                                                for (i = 0; i < TIFFGetValueByOrder(t->Count); ++i)
                                                {
                                                        if (fread(&temp, sizeof(t_uint32), 1, file) != 1)
                                                                break;
                                                        data[i] = TIFFGetValueByOrder(temp);
                                                }
                                        }
                                        return (data);
                        }
                }
        }
        __except(EXCEPTION_EXECUTE_HANDLER){
        }

        free(data);
        return (NULL); // no tag or error
}

HRESULT CImage::TIFFReadHeaderAndTags(FILE * file)
{
        __try
        {
                rewind(file);

                size_t count;

                // read header
                t_uint8  temp[2];
                t_uint32 IDF_Address;

                count = fread(temp, sizeof(t_uint8), 2, file);

                if((count != 2) || (temp[1] != temp[0] && (temp[0] !='I' || temp[0] !='M')))
                        return (E_FAIL); // bad header

                if (temp[0] == 'I')
                        m_tiff_byteorder = 1;
                else
                        m_tiff_byteorder = 0;

                if (fseek(file, 2L, SEEK_CUR)) // skip version
                        return (E_FAIL);

                // read first IFD  address
                if (fread(&IDF_Address, sizeof(t_uint32), 1, file) != 1)
                        return (E_FAIL);

                // read tag count
                if (fseek(file, TIFFGetValueByOrder(IDF_Address), SEEK_SET))
                        return (E_FAIL);

                if (fread(&m_tiff_TagsCount, sizeof(t_uint16), 1, file) != 1)
                        return (E_FAIL);

                m_tiff_TagsCount = TIFFGetValueByOrder(m_tiff_TagsCount);
                if (m_tiff_TagsCount <= 0)
                        return (E_FAIL); // must be positive

                // read tags
                m_tiff_Tags = (Tag*)calloc(m_tiff_TagsCount, sizeof(Tag));
                if (m_tiff_Tags == NULL)
                        return (E_FAIL); // no memory

                t_uint16 i;
                for(i = 0; i < m_tiff_TagsCount; i++)
                        if (fread(&m_tiff_Tags[i], sizeof(Tag), 1, file) != 1)
                        {
                                free(m_tiff_Tags);
                                m_tiff_Tags = NULL;
                                return (E_FAIL);
                        }

                m_tiff_colormap = TIFFGetTagValues(TIFF_TAG_ColorMap, file);
                m_tiff_bitpersample = TIFFGetTagValues(TIFF_TAG_BitsPerSample, file);

                // get image size
                t_uint32 x = TIFFGetTagValue(TIFF_TAG_ImageWidth);
                t_uint32 y = TIFFGetTagValue(TIFF_TAG_ImageLength);

                if (x <= 0 || y <= 0)
                {
                        free(m_tiff_Tags);
                        m_tiff_Tags = NULL;
                        free(m_tiff_colormap);
                        m_tiff_colormap = NULL;
                        free(m_tiff_bitpersample);
                        m_tiff_bitpersample = NULL;
                        return (E_FAIL);
                }

                SetSize(x, y);
        }
        __except(EXCEPTION_EXECUTE_HANDLER){
                free(m_tiff_Tags);
                m_tiff_Tags = NULL;
                free(m_tiff_colormap);
                m_tiff_colormap = NULL;
                free(m_tiff_bitpersample);
                m_tiff_bitpersample = NULL;
                return (E_FAIL);
        }

        return S_OK;
}

HRESULT CImage::TIFFReadStipsInfo(FILE * file)
{
        // if (TIFFGetTagValue(TIFF_TAG_RowsPerStrip) == 0)
        //       return (E_FAIL);
    //
    // if TIFF_TAG_RowsPerStrip not found - it can be calculated

        m_tiff_offsets = TIFFGetTagValues(TIFF_TAG_StripOffsets, file);
        if (m_tiff_offsets == NULL)
                return (E_FAIL);

        m_tiff_counts = TIFFGetTagValues(TIFF_TAG_StripByteCounts, file);
        if (m_tiff_counts == NULL)
        {
                free(m_tiff_offsets);
                m_tiff_offsets = NULL;
                return (E_FAIL);
        }

        m_tiff_count = TIFFGetValueByOrder(TIFFGetTag(TIFF_TAG_StripByteCounts)->Count);
        if (m_tiff_count <= 0)
        {
                free(m_tiff_counts);
                m_tiff_offsets = NULL;

                free(m_tiff_counts);
                m_tiff_offsets = NULL;

                return (E_FAIL);
        }

        return (S_OK);
}

HRESULT CImage::TIFFReadTilesInfo(FILE * file)
{
        m_tiff_offsets = TIFFGetTagValues(TIFF_TAG_TileOffsets, file);
        if (m_tiff_offsets == NULL)
                return (E_FAIL);

        m_tiff_counts = TIFFGetTagValues(TIFF_TAG_TileByteCounts, file);
        if (m_tiff_counts == NULL)
        {
                free(m_tiff_offsets);
                m_tiff_offsets = NULL;
                return (E_FAIL);
        }

        m_tiff_count = TIFFGetValueByOrder(TIFFGetTag(TIFF_TAG_TileOffsets)->Count);
        if (m_tiff_count <= 0)
        {
                free(m_tiff_counts);
                m_tiff_offsets = NULL;

                free(m_tiff_counts);
                m_tiff_offsets = NULL;

                return (E_FAIL);
        }

        return (S_OK);
}

static HRESULT ReadDataFromFile(FILE * file, t_uint8* data, t_uint32 count, t_uint32 offset)
{
        if (fseek(file, offset, SEEK_SET))
                return (E_FAIL);

        if (fread(data, sizeof(t_uint8), count, file) != count)
                return (E_FAIL);

        return (S_OK);
}

static t_uint8 GetBit(t_uint8 value, t_uint8 bit)
{
        return ((value >> (7-bit)) & 0x01);
}

static t_uint8 Get4Bit(t_uint8 value, t_uint8 bit)
{
        if (bit)
                return (value & 0x0F);
        else
                return ((value >> (4)) & 0x0F);
}

static void FillOrderTable(t_uint8* table)
{
        t_uint8 i;
        for (i = 0; i < 0xFF; ++i)
                table[i] = ((i >> 7) & 0x01) +
                                   (((i >> 6) & 0x01) << 1) +
                                   (((i >> 5) & 0x01) << 2) +
                                   (((i >> 4) & 0x01) << 3) +
                                   (((i >> 3) & 0x01) << 4) +
                                   (((i >> 2) & 0x01) << 5) +
                                   (((i >> 1) & 0x01) << 6) +
                                   ((i        & 0x01) << 7);
}

static void ChangeFillOrder(t_uint8* table, t_uint8* data, t_uint32 l)
{
        t_uint32 i;
        for (i = 0; i < l; ++i)
                data[i] = table[data[i]];
}

static t_uint16 TIFFLZWGetNextCode(t_uint8* d_in, t_uint32* stream_byte, t_uint32* stream_bit, t_uint32* stream_codebits)
{
        t_uint16 code = 0;
        t_uint32 i;

        for (i = 0; i < *stream_codebits; ++i)
        {
                code <<= 1;

                if (d_in[*stream_byte] & 0x80)
                        code += 1;

                d_in[*stream_byte] <<= 1;

                ++(*stream_bit);
                if (*stream_bit > 7)
                {
                        ++(*stream_byte);
                        *stream_bit = 0;
                }
        }

        return (code);
}

static void TIFFLZWInitializeTable(TIFF_LZW_TableEntry* lzw_table)
{
        t_uint16 i;

        for (i = 256; i < TIFF_LZW_MaxTableEntry; ++i)
        {
                lzw_table[i].Length = 0;
                lzw_table[i].OldCode = TIFF_LZW_EoiCode;
        }
}

static void TIFFLZWWriteCode(t_uint8* d_out, t_uint32* writed, TIFF_LZW_TableEntry* lzw_table, t_uint16 Code, t_uint8 *firstchar, t_uint32 length)
{
        t_uint32 i, l;
        l = lzw_table[Code].Length;

        for (i = 0; i < l; ++i)
        {
                d_out[(*writed)+l-i-1] = lzw_table[Code].Value;

                if (lzw_table[Code].OldCode != TIFF_LZW_EoiCode)
                        Code = lzw_table[Code].OldCode;
                else
                        break;
        }

        *firstchar = lzw_table[Code].Value;
        (*writed) += l;
}

static int TIFFLZWIsInTable(TIFF_LZW_TableEntry* lzw_table, t_uint16 Code)
{
        return (lzw_table[Code].Length>0);
}

static void TIFFLZWAddStringToTable(TIFF_LZW_TableEntry* lzw_table, t_uint16 Code1, t_uint8 first_char, t_uint32* stream_codebits, t_uint16 entry)
{
        t_uint16 i;
        // search for free space
        for (i = entry; i < TIFF_LZW_MaxTableEntry; ++i)
        {
                if (lzw_table[i].Length == 0)
                        break;
        }

        lzw_table[i].Length = lzw_table[Code1].Length+1;
        lzw_table[i].OldCode = Code1;

        lzw_table[i].Value = first_char;

        *stream_codebits = 9;
        if (i >= 511-1)
                *stream_codebits = 10;
        if (i >= 1023-1)
                *stream_codebits = 11;
        if (i >= 2047-1)
                *stream_codebits = 12;
        //if (entry >= 8191-1)
        //      *stream_codebits = 13;
}

static HRESULT TIFFDecompress(t_uint8* d_in, t_uint8* d_out, t_uint32 l, t_uint32 BytesPerRow, t_uint32 compression)
{
        // common variables
        t_uint16 i;
        t_uint32 writed;

        // TIFF_TAG_VALUE_PackBits
        t_uint32 readed;
        t_uint32 r;
        t_int8   byte;

        // TIFF_TAG_VALUE_LZW
        t_uint16 Code;
        t_uint16 OldCode;
        t_uint32 stream_byte;
        t_uint32 stream_bit;
        t_uint32 stream_codebits;
        t_uint8 first_char;
        TIFF_LZW_TableEntry lzw_table[TIFF_LZW_MaxTableEntry];
        t_uint16 entry_count;

        for (i = 0; i < TIFF_LZW_MaxTableEntry; ++i)
        {
                lzw_table[i].Length = 0;
                lzw_table[i].OldCode = TIFF_LZW_EoiCode;
        }

        __try
        {

        switch (compression)
        {
        case TIFF_TAG_VALUE_Uncompressed:
                ::CopyMemory(d_out, d_in, l);
                break;

        case TIFF_TAG_VALUE_PackBits:

                writed = 0;
                readed = 0;
                r          = 0;

                while (writed < l)
                {
                        // decompress row
                        writed = r*BytesPerRow;

                        while (writed < (r+1)*BytesPerRow)
                        {
                                byte = d_in[readed]; ++readed;

                                if (byte >= 0 && byte <= 127)
                                {
                                        for (i = 0; i < byte+1; ++i)
                                        {
                                                d_out[writed] = d_in[readed];
                                                ++readed; ++writed;
                                                if (writed >= (r+1)*BytesPerRow)
                                                        break;
                                        }
                                }

                                if (writed >= (r+1)*BytesPerRow)
                                        break;

                                if (byte >= -127 && byte <= -1)
                                {
                                        for (i = 0; i < -byte+1; ++i)
                                        {
                                                d_out[writed] = d_in[readed];
                                                ++writed;
                                                if (writed >= (r+1)*BytesPerRow)
                                                        break;
                                        }
                                        ++readed;
                                }
                        }
                        ++r;
                }

                break;

        case TIFF_TAG_VALUE_LZW:

                stream_codebits = 9; // starting from 9 bit codes
                Code                    = 0;
                OldCode                 = 0;
                stream_byte             = 0;
                stream_bit              = 0;
                entry_count             = 258;

                for (i = 0; i < 256; ++i)
                {
                        lzw_table[i].Length = 1;
                        lzw_table[i].Value  = (t_uint8) i;
                }

                writed = 0;

                while (writed < l && (Code = TIFFLZWGetNextCode(d_in, &stream_byte, &stream_bit, &stream_codebits))
                           != TIFF_LZW_EoiCode)
                {
                        if (Code == TIFF_LZW_ClearCode || writed == 0)
                        {
                                TIFFLZWInitializeTable(lzw_table); entry_count = 258; stream_codebits = 9;
                                Code = TIFFLZWGetNextCode(d_in, &stream_byte, &stream_bit, &stream_codebits);
                                if (Code == TIFF_LZW_EoiCode)
                                        break;
                                TIFFLZWWriteCode(d_out, &writed, lzw_table, Code, &first_char, l);
                                OldCode = Code;
                        } /* end of ClearCode case */
                        else
                        {
                                if (TIFFLZWIsInTable(lzw_table, Code))
                                {
                                        TIFFLZWWriteCode(d_out, &writed, lzw_table, Code, &first_char, l);
                                        TIFFLZWAddStringToTable(lzw_table, OldCode, first_char, &stream_codebits, entry_count);
                                        entry_count++;
                                        OldCode = Code;
                                }
                                else
                                {
                                        TIFFLZWWriteCode(d_out, &writed, lzw_table, OldCode, &first_char, l);

                                        d_out[writed] = first_char; ++writed;

                                        //TIFFLZWAddStringToTable(lzw_table, OldCode, first_char, &stream_codebits, entry_count);
                                        //entry_count++;
                                        TIFFLZWAddStringToTable(lzw_table, OldCode, first_char, &stream_codebits, Code);
                                        entry_count = Code+1;
                                        OldCode = Code;
                                }
                        } /* end of not-ClearCode case */
                } /* end of while loop */

                return (S_OK);

        default:
                return (E_FAIL);
        }

        }
        __except(EXCEPTION_EXECUTE_HANDLER){
                return (S_OK);
        }

        return (S_OK);
}

HRESULT CImage::TIFFReadImage(FILE * file, t_uint32 BitsPerSample, int rgb)
{
        t_uint32 i, y, x, j, z, tag_photo, tag_fillorder, tag_compression;
        t_uint8 table[0xFF]; // for FillOrder

        t_uint8* data = NULL;
        t_uint8* uncompressed_data = NULL;

        int error = 0;

        __try
        {
                tag_photo        = TIFFGetTagValue(TIFF_TAG_PhotometricInterpretation);
                tag_fillorder    = TIFFGetTagValue(TIFF_TAG_FillOrder);
                tag_compression  = TIFFGetTagValue(TIFF_TAG_Compression);

                //if (tag_photo != TIFF_TAG_VALUE_WhiteIsZero &&
                //      tag_photo != TIFF_TAG_VALUE_BlackIsZero)
                //{
                //      return (E_FAIL); // bad TIFF_TAG_PhotometricInterpretation
                //}

                if (tag_fillorder == 2)
                        FillOrderTable(table);

                if (FAILED(TIFFReadStipsInfo(file)))
                {
                        if (FAILED(TIFFReadTilesInfo(file)))
                        {
                                return (E_FAIL); // no image!
                        }
                        else // tiles
                        {
                                t_uint32 TileWidth    = TIFFGetTagValue(TIFF_TAG_TileWidth);
                                t_uint32 TileLength   = TIFFGetTagValue(TIFF_TAG_TileLength);
                                t_uint32 BytesPerRow;

                                if (BitsPerSample == 1)
                                        BytesPerRow = (TileWidth+7)/8;

                                if (BitsPerSample == 4)
                                        BytesPerRow = (TileWidth+1)/2;

                                if (BitsPerSample == 8)
                                        BytesPerRow = TileWidth;

                                if (rgb)
                                        BytesPerRow = 3 * BytesPerRow;

                                t_uint32 TilePerX     = w/TileWidth + ((w%TileWidth)?1:0);
                                t_uint32 TilePerY     = l/TileLength + ((l%TileLength)?1:0);

                                // find max tile
                                t_uint32 maxtile = m_tiff_counts[0];
                                for (i = 1; i < m_tiff_count; ++i)
                                        if (maxtile < m_tiff_counts[i])
                                                maxtile = m_tiff_counts[i];

                                data = (t_uint8*) calloc(maxtile, sizeof(t_uint8));
                                uncompressed_data = (t_uint8*) calloc(TileLength*BytesPerRow, sizeof(t_uint8));

                                if (data != NULL)
                                for (i = 0; i < m_tiff_count; ++i)
                                {
                                        if (FAILED(ReadDataFromFile(file, data, m_tiff_counts[i], m_tiff_offsets[i])))
                                        {
                                                error = 1;
                                                break;
                                        }

                                        // fill order
                                        if (tag_fillorder == 2)
                                                ChangeFillOrder(table, data, m_tiff_counts[i]);

                                        // decompress

                                        if (FAILED(TIFFDecompress(data, uncompressed_data, BytesPerRow*TileLength, BytesPerRow, tag_compression)))
                                        {
                                                error = 1;
                                                break;
                                        }

                                        // put into buffer
                                        j = i/TilePerX*TileLength;
                                        z = i%TilePerX*TileWidth;

                                        if (rgb)
                                        {
                                                for (y = j; y < ((j+TileLength)>(t_uint32)l?(t_uint32)l:(j+TileLength)); ++y)
                                                        for (x = z; x < ((z+TileWidth)>(t_uint32)w?(t_uint32)w:(z+TileWidth)); ++x)
                                                                img[y][x] = (uncompressed_data[(y-j)*BytesPerRow+(x-z)*3]+uncompressed_data[(y-j)*BytesPerRow+(x-z)*3+1]+uncompressed_data[(y-j)*BytesPerRow+(x-z)*3+2])/3;
                                                continue;
                                        }

                                        if (BitsPerSample == 1)
                                        {
                                                for (y = j; y < ((j+TileLength)>(t_uint32)l?(t_uint32)l:(j+TileLength)); ++y)
                                                        for (x = z; x < ((z+TileWidth)>(t_uint32)w?(t_uint32)w:(z+TileWidth)); ++x)
                                                                img[y][x] = GetBit(uncompressed_data[(y-j)*BytesPerRow+(x-z)/8], (t_uint8) (x-z)%8)*0xFF;
                                                continue;
                                        }

                                        if (m_tiff_colormap == NULL)
                                        {
                                                if (BitsPerSample == 4)
                                                        for (y = j; y < ((j+TileLength)>(t_uint32)l?(t_uint32)l:(j+TileLength)); ++y)
                                                                for (x = z; x < ((z+TileWidth)>(t_uint32)w?(t_uint32)w:(z+TileWidth)); ++x)
                                                                        img[y][x] = Get4Bit(uncompressed_data[(y-j)*BytesPerRow+(x-z)/2], (t_uint8) (x-z)%2)*0x0F;

                                                if (BitsPerSample == 8)
                                                        for (y = j; y < ((j+TileLength)>(t_uint32)l?(t_uint32)l:(j+TileLength)); ++y)
                                                                for (x = z; x < ((z+TileWidth)>(t_uint32)w?(t_uint32)w:(z+TileWidth)); ++x)
                                                                        img[y][x] = uncompressed_data[(y-j)*BytesPerRow+(x-z)];
                                        }
                                        else
                                        {
                                                t_uint32 temp;
                                                t_uint32 rgbsize = 1 << BitsPerSample;

                                                if (BitsPerSample == 4)
                                                        for (y = j; y < ((j+TileLength)>(t_uint32)l?(t_uint32)l:(j+TileLength)); ++y)
                                                                for (x = z; x < ((z+TileWidth)>(t_uint32)w?(t_uint32)w:(z+TileWidth)); ++x)
                                                                {
                                                                        temp = Get4Bit(uncompressed_data[(y-j)*BytesPerRow+(x-z)/2], (t_uint8) (x-z)%2);
                                                                        img[y][x] = (t_uint8) ((m_tiff_colormap[temp]+ m_tiff_colormap[rgbsize+temp] + m_tiff_colormap[2*rgbsize+temp])/3/0x100);
                                                                }


                                                if (BitsPerSample == 8)
                                                        for (y = j; y < ((j+TileLength)>(t_uint32)l?(t_uint32)l:(j+TileLength)); ++y)
                                                                for (x = z; x < ((z+TileWidth)>(t_uint32)w?(t_uint32)w:(z+TileWidth)); ++x)
                                                                {
                                                                        temp = uncompressed_data[(y-j)*BytesPerRow+(x-z)];
                                                                        img[y][x] = (t_uint8) ((m_tiff_colormap[temp]+ m_tiff_colormap[rgbsize+temp] + m_tiff_colormap[2*rgbsize+temp])/3/0x100);
                                                                }
                                        }
                                }
                        }
                }
                else // strips
                {
                        t_uint32 RowsPerStrip = TIFFGetTagValue(TIFF_TAG_RowsPerStrip);

                        t_uint32 BytesPerRow;

                        if (BitsPerSample == 1)
                                BytesPerRow = (w+7)/8;

                        if (BitsPerSample == 4)
                                BytesPerRow = (w+1)/2;

                        if (BitsPerSample == 8)
                                BytesPerRow = w;

                        if (rgb)
                                        BytesPerRow = 3 * BytesPerRow;

                        if (RowsPerStrip == 0)
                                RowsPerStrip = m_tiff_counts[0]/BytesPerRow;

                        // find max strip
                        t_uint32 maxstrip = m_tiff_counts[0];
                        for (i = 1; i < m_tiff_count; ++i)
                                if (maxstrip < m_tiff_counts[i])
                                        maxstrip = m_tiff_counts[i];

                        data = (t_uint8*) calloc(maxstrip, sizeof(t_uint8));
                        uncompressed_data = (t_uint8*) calloc(RowsPerStrip*BytesPerRow, sizeof(t_uint8));

                        if (data != NULL)
                        for (i = 0; i < m_tiff_count; ++i)
                        {
                                if (FAILED(ReadDataFromFile(file, data, m_tiff_counts[i], m_tiff_offsets[i])))
                                {
                                        error = 1;
                                        break;
                                }

                                // fill order
                                if (tag_fillorder == 2)
                                        ChangeFillOrder(table, data, m_tiff_counts[i]);

                                // decompress
                                // put into buffer
                                if (i+1 == m_tiff_count)
                                        j = l-RowsPerStrip*i;
                                else
                                        j = RowsPerStrip;

                                if (FAILED(TIFFDecompress(data, uncompressed_data, BytesPerRow*j, BytesPerRow, tag_compression)))
                                {
                                        error = 1;
                                        break;
                                }

                                if (rgb)
                                {
                                        for (y = 0; y < j; ++y)
                                                for (x = 0; x < (t_uint32) w; ++x)
                                                        img[RowsPerStrip*i+y][x] = (uncompressed_data[y*BytesPerRow+x*3]+uncompressed_data[y*BytesPerRow+x*3+1]+uncompressed_data[y*BytesPerRow+x*3+2])/3;

                                        continue;
                                }

                                if (BitsPerSample == 1)
                                {
                                        for (y = 0; y < j; ++y)
                                                for (x = 0; x < (t_uint32) w; ++x)
                                                        img[RowsPerStrip*i+y][x] = GetBit(uncompressed_data[y*BytesPerRow+x/8], (t_uint8) x%8)*0xFF;
                                        continue;
                                }

                                if (m_tiff_colormap == NULL)
                                {
                                        if (BitsPerSample == 4)
                                                for (y = 0; y < j; ++y)
                                                        for (x = 0; x < (t_uint32) w; ++x)
                                                                img[RowsPerStrip*i+y][x] = Get4Bit(uncompressed_data[y*BytesPerRow+x/2], (t_uint8) x%2)*0x0F;

                                        if (BitsPerSample == 8)
                                                for (y = 0; y < j; ++y)
                                                        for (x = 0; x < (t_uint32) w; ++x)
                                                                img[RowsPerStrip*i+y][x] = uncompressed_data[y*BytesPerRow+x];
                                }
                                else
                                {
                                        t_uint32 temp;
                                        t_uint32 rgbsize = 1 << BitsPerSample;

                                        if (BitsPerSample == 4)
                                                for (y = 0; y < j; ++y)
                                                        for (x = 0; x < (t_uint32) w; ++x)
                                                        {

                                                                        temp = Get4Bit(uncompressed_data[y*BytesPerRow+x/2], (t_uint8) x%2);
                                                                        img[RowsPerStrip*i+y][x] = (t_uint8) ((m_tiff_colormap[temp]+ m_tiff_colormap[rgbsize+temp] + m_tiff_colormap[2*rgbsize+temp])/3/0x100);
                                                        }


                                        if (BitsPerSample == 8)
                                                for (y = 0; y < j; ++y)
                                                        for (x = 0; x < (t_uint32) w; ++x)
                                                        {

                                                                        temp = uncompressed_data[y*BytesPerRow+x];
                                                                        img[RowsPerStrip*i+y][x] = (t_uint8) ((m_tiff_colormap[temp]+ m_tiff_colormap[rgbsize+temp] + m_tiff_colormap[2*rgbsize+temp])/3/0x100);
                                                        }
                                }
                        }
                }

                // TIFF_TAG_VALUE_WhiteIsZero
                if (tag_photo == TIFF_TAG_VALUE_WhiteIsZero)
                        for (y = 0; y < (t_uint32) l; ++y)
                                for (x = 0; x < (t_uint32) w; ++x)
                                        img[y][x] = 0xFF-img[y][x];

                free(data);
                data = NULL;

                free(uncompressed_data);
                uncompressed_data = NULL;

                if (error)
                        return (E_FAIL);
                else
                        return (S_OK);
        }
        __except(EXCEPTION_EXECUTE_HANDLER){
                free(data);
                free(uncompressed_data);
                return (E_FAIL);
        }
}

HRESULT CImage::TIFFReadData(FILE * file)
{
        // detect type of image

        t_uint32 BitsPerSample   = TIFFGetTagValue(TIFF_TAG_BitsPerSample);
        t_uint32 SamplesPerPixel = TIFFGetTagValue(TIFF_TAG_SamplesPerPixel);
        t_uint32 PhotometricInterpretation = TIFFGetTagValue(TIFF_TAG_PhotometricInterpretation);

        // bilevel?
        if (BitsPerSample == 1 &&
                SamplesPerPixel == 1 &&
                (PhotometricInterpretation == TIFF_TAG_VALUE_WhiteIsZero ||
                 PhotometricInterpretation == TIFF_TAG_VALUE_BlackIsZero))
        {
                return TIFFReadImage(file, BitsPerSample, 0);
        }

        // grayscale?
        if ((BitsPerSample == 4 || BitsPerSample == 8) &&
                 SamplesPerPixel == 1 &&
                 (PhotometricInterpretation == TIFF_TAG_VALUE_WhiteIsZero ||
                  PhotometricInterpretation == TIFF_TAG_VALUE_BlackIsZero))
        {
                return TIFFReadImage(file, BitsPerSample, 0);
        }

        // color map?
        if ((BitsPerSample == 4 || BitsPerSample == 8) &&
                 m_tiff_colormap != NULL &&
                 TIFFGetTagValue(TIFF_TAG_PhotometricInterpretation) == TIFF_TAG_VALUE_RGB_Palette)
        {
                return TIFFReadImage(file, BitsPerSample, 0);
        }

        // true color? 8bit?
        if (m_tiff_bitpersample != NULL &&
                TIFFGetTagValue(TIFF_TAG_SamplesPerPixel) == 3 &&
                TIFFGetTagValue(TIFF_TAG_PhotometricInterpretation) == TIFF_TAG_VALUE_RGB)
        {
                if (m_tiff_bitpersample[0] == 8 && m_tiff_bitpersample[1] == 8 &&
                        m_tiff_bitpersample[2] == 8)

                        return TIFFReadImage(file, 8, 1);
        }

        return E_FAIL; // image type is wrong or unknown
}

HRESULT CImage::LoadTIF(LPCSTR fn)
{
        FILE * file;
        file = NULL;

        __try
        {
                file = fopen(fn,"rb");
                if(file==NULL){
                        return E_FAIL;
                }

                int error = 0;

                if (FAILED(TIFFReadHeaderAndTags(file)))
                        error = 1;
                else
                {
                        if (FAILED(TIFFReadData(file)))
                                error = 1;
                }

                if (error)
                {
                        SetSize(0,0);
                        return E_FAIL;
                }
        }
        __except(EXCEPTION_EXECUTE_HANDLER ){
                SetSize(0,0);
                return E_FAIL;
        }

        free(m_tiff_Tags);
        m_tiff_Tags = NULL;

        free(m_tiff_offsets);
        m_tiff_offsets = NULL;

        free(m_tiff_counts);
        m_tiff_counts = NULL;

        free(m_tiff_colormap);
        m_tiff_colormap = NULL;

        free(m_tiff_bitpersample);
        m_tiff_bitpersample = NULL;

        fclose(file);

        return S_OK;
}

////////////////////////////////////////////////////////////////////////////
// Bitmap
////////////////////////////////////////////////////////////////////////////
static inline int GetLeftShiftCount (DWORD dwVal)
{
    int nCount = 0;
    for (int i=0; i<sizeof (DWORD) * 8; i++) {
        if (dwVal & 1)
            nCount++;
        dwVal >>= 1;
    }
    return (8 - nCount);
}
////////////////////////////////////////////////////////////////////////////
static inline int GetRightShiftCount (DWORD dwVal)
{
    for (int i=0; i<sizeof (DWORD) * 8; i++) {
        if (dwVal & 1)
            return i;
        dwVal >>= 1;
    }
    return -1;
}
////////////////////////////////////////////////////////////////////////////


CImage::CImage()
{
        img = NULL;
        w = 0;
        l = 0;
        m_pBMI = (LPBITMAPINFO) new BYTE[sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD) * 256];
    m_pBMIGn = (LPBITMAPINFO) new BYTE[sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD) * 256];
        m_pBMI->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
    m_pBMI->bmiHeader.biPlanes = 1;
    m_pBMI->bmiHeader.biBitCount = 8;
    m_pBMI->bmiHeader.biCompression = BI_RGB;
    m_pBMI->bmiHeader.biSizeImage = 0;
    m_pBMI->bmiHeader.biXPelsPerMeter = 0;
    m_pBMI->bmiHeader.biYPelsPerMeter = 0;
    m_pBMI->bmiHeader.biClrUsed      = 256;
    m_pBMI->bmiHeader.biClrImportant = 0;
        int i;
		for(i = 0; i < 256; i++)
        {
                m_pBMI->bmiColors[i].rgbBlue = i;
                m_pBMI->bmiColors[i].rgbGreen = i;
                m_pBMI->bmiColors[i].rgbRed = i;
                m_pBMI->bmiColors[i].rgbReserved = 0;
        }
        m_pBMIGn->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
    m_pBMIGn->bmiHeader.biPlanes = 1;
    m_pBMIGn->bmiHeader.biBitCount = 8;
    m_pBMIGn->bmiHeader.biCompression = BI_RGB;
    m_pBMIGn->bmiHeader.biSizeImage = 0;
    m_pBMIGn->bmiHeader.biXPelsPerMeter = 0;
    m_pBMIGn->bmiHeader.biYPelsPerMeter = 0;
    m_pBMIGn->bmiHeader.biClrUsed      = 256;
    m_pBMIGn->bmiHeader.biClrImportant = 0;
        BYTE gr = 235,flag = 0;
        for(i = 0; i < 256; i++)
        {
                m_pBMIGn->bmiColors[i].rgbBlue = i;
                m_pBMIGn->bmiColors[i].rgbGreen = gr;
                m_pBMIGn->bmiColors[i].rgbRed = i;
                m_pBMIGn->bmiColors[i].rgbReserved = 0;
                if(gr != 255){
                        if(flag <13){
                                flag++;
                        }
                        else{
                                gr++;
                                flag = 0;
                        }
                }
        }

        m_tiff_byteorder = 1;
        m_tiff_TagsCount = 0;
        m_tiff_Tags              = NULL;
        m_tiff_count     = 0;
        m_tiff_offsets   = NULL;
        m_tiff_counts    = NULL;
        m_tiff_colormap  = NULL;
        m_tiff_bitpersample = NULL;
}

CImage::~CImage()
{
        SetSize(0,0);
        delete [] m_pBMI;
        delete [] m_pBMIGn;

        // for TIFF reader
        free(m_tiff_Tags);              m_tiff_Tags = NULL;
        free(m_tiff_offsets);   m_tiff_offsets = NULL;
        free(m_tiff_counts);    m_tiff_counts = NULL;
        free(m_tiff_colormap);  m_tiff_colormap = NULL;
        free(m_tiff_bitpersample);  m_tiff_bitpersample = NULL;
}

HRESULT CImage::Load(LPCSTR filename,float ratio)
{
        CHAR ext[4];
        GetFileExt(filename,ext);
        ext[3] = 0;
        HRESULT result;

        if(_stricmp(ext,"dat") == 0) result = LoadDAT(filename);
        if(_stricmp(ext,"bmp") == 0) result = LoadBMP(filename);
        if(_stricmp(ext,"tif") == 0) result = LoadTIF(filename);

        if(FAILED(result))
                return E_FAIL;
        /*
        if(ratio == 1)
           return S_OK;

        BYTE ** itmp = img;
        img = NULL;
        SetSize(int(w / ratio), int(l / ratio));
        for(int i,j = 0; j < l; j++)
                for(i = 0; i < w; i++)
                        img[j][i] = itmp[int(j * ratio)][int(i * ratio)];  */
        return S_OK;
}

HRESULT CImage::GetFileExt(LPCSTR fn, CHAR *ext)
{
        int n = strlen(fn);
        ext[2] = fn[n-1];
        ext[1] = fn[n-2];
        ext[0] = fn[n-3];
        return S_OK;
}

HRESULT CImage::LoadDAT(LPCSTR fn)
{
        FILE * file;
        file = fopen(fn,"rb");
        if(file==NULL){
                return E_FAIL;
        }
        DWORD dwV,x,y;
        __try{
                fread(&dwV,sizeof(DWORD),1,file);
                fread(&dwV,sizeof(DWORD),1,file);
                fread(&dwV,sizeof(DWORD),1,file);
                fread(&y,sizeof(DWORD),1,file);
                fread(&x,sizeof(DWORD),1,file);
                SetSize(x,y);
                for(dwV = 0; dwV < DWORD(l);dwV++)
                        if(fread(img[dwV],sizeof(BYTE),w,file) != DWORD(w))
                        {
                                SetSize(0,0);
                                fclose(file);
                                return E_FAIL;
                        }
                fread(&dwV,sizeof(DWORD),1,file);
        }
        __except(EXCEPTION_EXECUTE_HANDLER){
                SetSize(0,0);
                fclose(file);
                return E_FAIL;
        }
        fclose(file);
        return S_OK;
}

HRESULT CImage::SaveBMP(LPCSTR path)
{
        BITMAPFILEHEADER hdr;
        DWORD dwTmp;
        hdr.bfType = 0x4d42;
        m_pBMI->bmiHeader.biClrUsed = 256;
    hdr.bfSize = (DWORD) (sizeof(BITMAPFILEHEADER) +
                 m_pBMI->bmiHeader.biSize + m_pBMI->bmiHeader.biClrUsed
                 * sizeof(RGBQUAD) + m_pBMI->bmiHeader.biSizeImage);
    hdr.bfReserved1 = 0;
    hdr.bfReserved2 = 0;
        hdr.bfOffBits = (DWORD) sizeof(BITMAPFILEHEADER) +
                   m_pBMI->bmiHeader.biSize + m_pBMI->bmiHeader.biClrUsed
                 * sizeof(RGBQUAD);
        HANDLE hf = CreateFile(path,
                   GENERIC_READ | GENERIC_WRITE,
                   (DWORD) 0,
                    NULL,
                   CREATE_ALWAYS,
                   FILE_ATTRIBUTE_NORMAL,
                   (HANDLE) NULL);
    if (hf == INVALID_HANDLE_VALUE)
                return E_FAIL;
        if (!WriteFile(hf, &hdr, sizeof(hdr),
        &dwTmp, NULL))
    {
                CloseHandle(hf);
                return E_FAIL;
    }
        if (!WriteFile(hf, m_pBMI, m_pBMI->bmiHeader.biSize
                  + m_pBMI->bmiHeader.biClrUsed * sizeof (RGBQUAD),
                  &dwTmp, NULL))
        {
                CloseHandle(hf);
                return E_FAIL;
    }
        if (!WriteFile(hf, m_img.m_row, m_pBMI->bmiHeader.biSizeImage,
                &dwTmp,NULL))
    {
                CloseHandle(hf);
                return E_FAIL;
    }
        CloseHandle(hf);
        return S_OK;
}

HRESULT CImage::LoadBMP(LPCSTR fn)
{
        HANDLE ibmp = LoadImage(
                NULL,
                fn,
                IMAGE_BITMAP,
                0,0,
                LR_DEFAULTCOLOR|LR_CREATEDIBSECTION|LR_DEFAULTSIZE|LR_LOADFROMFILE);
        if(ibmp == NULL){
                return E_FAIL;
        }
        DIBSECTION ds;
        if(!GetObject(ibmp,sizeof(ds),&ds)){
                DeleteObject(ibmp);
                return E_FAIL;
        }
        SetSize(ds.dsBm.bmWidth,ds.dsBm.bmHeight);
        BYTE ** buf = img;
        int x,y;
        DWORD rmask, gmask, bmask;
    int rright, gright, bright;
        int rleft, gleft, bleft;
        BITMAPINFO bmi;
        WORD wColor;
        WORD* pwBits;
    DWORD* pdwBits;
        DWORD dwColor;
        BYTE* pbBits,*pBuffer,r, g, b;
    HDC hdc;

    if (ds.dsBmih.biSizeImage == 0)
        ds.dsBmih.biSizeImage = ((((ds.dsBmih.biWidth * ds.dsBmih.biBitCount) + 31) & ~31) >> 3) * ds.dsBmih.biHeight;

        int nPad = ds.dsBmih.biSizeImage/ds.dsBmih.biHeight - (((ds.dsBmih.biWidth *
        ds.dsBmih.biBitCount) + 7) / 8);

        switch(ds.dsBmih.biBitCount)
        {
        case 1:
    case 4:
    case 8:
        hdc = ::GetDC (NULL);
        pBuffer = new BYTE[(ds.dsBmih.biWidth+(4-ds.dsBmih.biWidth%4)) * 3];

        ::ZeroMemory (&bmi, sizeof (bmi));
        bmi.bmiHeader.biSize = sizeof (BITMAPINFOHEADER);
        bmi.bmiHeader.biWidth = ds.dsBmih.biWidth;
        bmi.bmiHeader.biHeight = ds.dsBmih.biHeight;
        bmi.bmiHeader.biPlanes = 1;
        bmi.bmiHeader.biBitCount = 24;
        bmi.bmiHeader.biCompression = BI_RGB;

        for (y=0; y<ds.dsBmih.biHeight; y++) {
            ::GetDIBits (hdc, (HBITMAP) ibmp, y, 1, pBuffer, &bmi,
                DIB_RGB_COLORS);
            pbBits = pBuffer;
            for (x=0; x<ds.dsBmih.biWidth; x++) {
                b = *pbBits++;
                g = *pbBits++;
                r = *pbBits++;
                                buf[l-y-1][x] = (b+r+g)/3;
            }
        }

        delete pBuffer;
        ::ReleaseDC (NULL, hdc);
        break;
        case 16:
        if (ds.dsBmih.biCompression == BI_BITFIELDS) {
            rmask = ds.dsBitfields[0];
            gmask = ds.dsBitfields[1];
            bmask = ds.dsBitfields[2];
        }
        else {
            rmask = 0x7C00;
            gmask = 0x03E0;
            bmask = 0x001F;
        }

        rright = GetRightShiftCount (rmask);
        gright = GetRightShiftCount (gmask);
        bright = GetRightShiftCount (bmask);

        rleft = GetLeftShiftCount (rmask);
        gleft = GetLeftShiftCount (gmask);
        bleft = GetLeftShiftCount (bmask);

        pwBits = (WORD*) ds.dsBm.bmBits;
        for (y=0; y<ds.dsBmih.biHeight; y++) {
            for (x=0; x<ds.dsBmih.biWidth; x++) {
                wColor = *pwBits++;
                b = (BYTE) (((wColor & (WORD) bmask) >> bright) << bleft);
                g = (BYTE) (((wColor & (WORD) gmask) >> gright) << gleft);
                r = (BYTE) (((wColor & (WORD) rmask) >> rright) << rleft);
                                buf[l-y-1][x] = (b+r+g)/3;
            }
            pwBits = (WORD*) (((BYTE*) pwBits) + nPad);
        }
        break;

        case 24:
                pbBits = (BYTE*) ds.dsBm.bmBits;
        for (y=0; y<ds.dsBmih.biHeight; y++) {
            for (x=0; x<ds.dsBmih.biWidth; x++) {
                b = *pbBits++;
                g = *pbBits++;
                r = *pbBits++;
                                buf[l-y-1][x] = (b+r+g)/3;
            }
            pbBits += nPad;
        }
                break;
        case 32:
                if (ds.dsBmih.biCompression == BI_BITFIELDS) {
            rmask = ds.dsBitfields[0];
            gmask = ds.dsBitfields[1];
            bmask = ds.dsBitfields[2];
        }
        else {
            rmask = 0x00FF0000;
            gmask = 0x0000FF00;
            bmask = 0x000000FF;
        }

        rright = GetRightShiftCount (rmask);
        gright = GetRightShiftCount (gmask);
        bright = GetRightShiftCount (bmask);

        pdwBits = (DWORD*) ds.dsBm.bmBits;
        for (y=0; y<ds.dsBmih.biHeight; y++) {
            for (x=0; x<ds.dsBmih.biWidth; x++) {
                dwColor = *pdwBits++;
                b = (BYTE) ((dwColor & bmask) >> bright);
                g = (BYTE) ((dwColor & gmask) >> gright);
                r = (BYTE) ((dwColor & rmask) >> rright);
                buf[l-y-1][x] = (b+r+g)/3;
            }
            pdwBits = (DWORD*) (((BYTE*) pdwBits) + nPad);
        }
                break;
        default:
                DeleteObject(ibmp);
                SetSize(0,0);
                return E_FAIL;
        }
        DeleteObject(ibmp);
        return S_OK;
}

HRESULT CImage::SetSize(int x, int y)
{
        if( x == w && y == l) return S_OK;
        div_t d = div(x,4);
        img = NULL;
        w = 0;
        l = 0;
        m_pBMI->bmiHeader.biHeight = 0;
        m_pBMI->bmiHeader.biWidth = 0;
        m_pBMIGn->bmiHeader.biHeight = 0;
        m_pBMIGn->bmiHeader.biWidth = 0;
        if(!m_img.Set(y,d.rem?x+(4-d.rem):x))return E_FAIL;
        img = m_img.GetBuffer();
        w = x;
        l = y;
        m_pBMI->bmiHeader.biHeight = l;
        m_pBMI->bmiHeader.biWidth = w;
        m_pBMIGn->bmiHeader.biHeight = l;
        m_pBMIGn->bmiHeader.biWidth = w;
        m_pBMI->bmiHeader.biSizeImage = m_pBMI->bmiHeader.biHeight*m_img.m_sx;
        m_pBMIGn->bmiHeader.biSizeImage = m_pBMIGn->bmiHeader.biHeight*m_img.m_sx;
        return S_OK;
}

void CImage::Draw(HDC dc, BOOL fGrayScale,float Ratio)
{
        if(!img)return;
        StretchDIBits(dc,0,0,int(w*Ratio),int(l*Ratio),0,0,w,l,m_img.m_row,fGrayScale?m_pBMI:m_pBMIGn,DIB_RGB_COLORS,SRCCOPY);
}

HRESULT CImage::Copy(CImage *img)
{
        if(w != img->w || l != img->l)
                if(FAILED(img->SetSize(w,l)))return E_FAIL;
        m_img.Clone(&(img->m_img));
        return S_OK;
}


HRESULT CImage::FromHandle(HBITMAP ibmp)
{
        HDC hdc;
        BITMAPINFO srcbi;
        srcbi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
        srcbi.bmiHeader.biBitCount = 0;
        hdc = ::GetDC (NULL);
        if(!GetDIBits(hdc,ibmp,0,0,NULL,&srcbi,0)){
                ::ReleaseDC (NULL, hdc);
                return E_FAIL;
        }


        srcbi.bmiHeader.biBitCount = 24;
        srcbi.bmiHeader.biClrImportant = 0;
        srcbi.bmiHeader.biClrUsed = 0;
        srcbi.bmiHeader.biCompression = BI_RGB;
        srcbi.bmiHeader.biSizeImage = 0;
        srcbi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);

        BYTE *crcimgbuf;

        HBITMAP srchbmp = CreateDIBSection(hdc,&srcbi,DIB_RGB_COLORS,(VOID **)&crcimgbuf,NULL,0);
        if(!srchbmp){
                ::ReleaseDC (NULL, hdc);
                return E_FAIL;
        }
        if(!GetDIBits(hdc,ibmp,0,srcbi.bmiHeader.biHeight,crcimgbuf,&srcbi,DIB_RGB_COLORS)){
                DeleteObject(srchbmp);
                ::ReleaseDC (NULL, hdc);
                return E_FAIL;
        }
        ::ReleaseDC (NULL, hdc);
        DIBSECTION ds;
        if(!GetObject(srchbmp,sizeof(ds),&ds)){
                DeleteObject(srchbmp);
                return E_FAIL;
        }
        SetSize(ds.dsBm.bmWidth,ds.dsBm.bmHeight);
        BYTE ** buf = img;
        int x,y;
        DWORD rmask, gmask, bmask;
    int rright, gright, bright;
        int rleft, gleft, bleft;
        BITMAPINFO bmi;
        WORD wColor;
        WORD* pwBits;
    DWORD* pdwBits;
    DWORD dwColor;
        BYTE* pbBits,*pBuffer,r, g, b;

    if (ds.dsBmih.biSizeImage == 0)
        ds.dsBmih.biSizeImage = ((((ds.dsBmih.biWidth * ds.dsBmih.biBitCount) + 31) & ~31) >> 3) * ds.dsBmih.biHeight;

        int nPad = ds.dsBmih.biSizeImage/ds.dsBmih.biHeight - (((ds.dsBmih.biWidth *
        ds.dsBmih.biBitCount) + 7) / 8);

        switch(ds.dsBmih.biBitCount)
        {
        case 1:
    case 4:
    case 8:
        hdc = ::GetDC (NULL);
        pBuffer = new BYTE[ds.dsBmih.biWidth * 3];

        ::ZeroMemory (&bmi, sizeof (bmi));
        bmi.bmiHeader.biSize = sizeof (BITMAPINFOHEADER);
        bmi.bmiHeader.biWidth = ds.dsBmih.biWidth;
        bmi.bmiHeader.biHeight = ds.dsBmih.biHeight;
        bmi.bmiHeader.biPlanes = 1;
        bmi.bmiHeader.biBitCount = 24;
        bmi.bmiHeader.biCompression = BI_RGB;

        for (y=0; y<ds.dsBmih.biHeight; y++) {
            ::GetDIBits (hdc, (HBITMAP) ibmp, y, 1, pBuffer, &bmi,
                DIB_RGB_COLORS);
            pbBits = pBuffer;
            for (x=0; x<ds.dsBmih.biWidth; x++) {
                b = *pbBits++;
                g = *pbBits++;
                r = *pbBits++;
                                buf[l-y-1][x] = (b+r+g)/3;
            }
        }

        delete pBuffer;
        ::ReleaseDC (NULL, hdc);
        break;
        case 16:
        if (ds.dsBmih.biCompression == BI_BITFIELDS) {
            rmask = ds.dsBitfields[0];
            gmask = ds.dsBitfields[1];
            bmask = ds.dsBitfields[2];
        }
        else {
            rmask = 0x7C00;
            gmask = 0x03E0;
            bmask = 0x001F;
        }

        rright = GetRightShiftCount (rmask);
        gright = GetRightShiftCount (gmask);
        bright = GetRightShiftCount (bmask);

        rleft = GetLeftShiftCount (rmask);
        gleft = GetLeftShiftCount (gmask);
        bleft = GetLeftShiftCount (bmask);

        pwBits = (WORD*) ds.dsBm.bmBits;
        for (y=0; y<ds.dsBmih.biHeight; y++) {
            for (x=0; x<ds.dsBmih.biWidth; x++) {
                wColor = *pwBits++;
                b = (BYTE) (((wColor & (WORD) bmask) >> bright) << bleft);
                g = (BYTE) (((wColor & (WORD) gmask) >> gright) << gleft);
                r = (BYTE) (((wColor & (WORD) rmask) >> rright) << rleft);
                                buf[l-y-1][x] = (b+r+g)/3;
            }
            pwBits = (WORD*) (((BYTE*) pwBits) + nPad);
        }
        break;

        case 24:
                pbBits = (BYTE*) ds.dsBm.bmBits;
        for (y=0; y<ds.dsBmih.biHeight; y++) {
            for (x=0; x<ds.dsBmih.biWidth; x++) {
                b = *pbBits++;
                g = *pbBits++;
                r = *pbBits++;
                                buf[l-y-1][x] = (b+r+g)/3;
            }
            pbBits += nPad;
        }
                break;
        case 32:
                if (ds.dsBmih.biCompression == BI_BITFIELDS) {
            rmask = ds.dsBitfields[0];
            gmask = ds.dsBitfields[1];
            bmask = ds.dsBitfields[2];
        }
        else {
            rmask = 0x00FF0000;
            gmask = 0x0000FF00;
            bmask = 0x000000FF;
        }

        rright = GetRightShiftCount (rmask);
        gright = GetRightShiftCount (gmask);
        bright = GetRightShiftCount (bmask);

        pdwBits = (DWORD*) ds.dsBm.bmBits;
        for (y=0; y<ds.dsBmih.biHeight; y++) {
            for (x=0; x<ds.dsBmih.biWidth; x++) {
                dwColor = *pdwBits++;
                b = (BYTE) ((dwColor & bmask) >> bright);
                g = (BYTE) ((dwColor & gmask) >> gright);
                r = (BYTE) ((dwColor & rmask) >> rright);
                buf[l-y-1][x] = (b+r+g)/3;
            }
            pdwBits = (DWORD*) (((BYTE*) pdwBits) + nPad);
        }
                break;
        default:
                SetSize(0,0);
                DeleteObject(srchbmp);
                return E_FAIL;
        }
        DeleteObject(srchbmp);
        return S_OK;
}

HBITMAP CImage::ToHandle(bool green)
{
        BYTE * buff;
        HBITMAP hbmp = CreateDIBSection(
                NULL,
        green?m_pBMIGn:m_pBMI,
                DIB_RGB_COLORS,
                (VOID**)&buff,
                NULL,0);
        if(hbmp == NULL)return NULL;
        memcpy(buff,m_img.m_row,sizeof(BYTE)*m_img.m_sx*m_img.m_sy);
        return hbmp;
}
