using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Neurotec.Biometrics.VeriFingerDemo.Properties;
using System.Globalization;

namespace Neurotec.Biometrics.VeriFingerDemo
{
	public partial class EnrollmentOptionsForm : Form
	{
		#region Private static readonly fields

		private static readonly KeyValuePair<uint, string>[] vfeModes =
			{
				new KeyValuePair<uint, string>(VFExtractor.ModeGeneral, "General"),
				new KeyValuePair<uint, string>(VFExtractor.ModeDigitalPersonaUareU, "DigitalPersona U.are.U"),
				new KeyValuePair<uint, string>(VFExtractor.ModeBiometrikaFX2000, "BiometriKa FX2000"),
				new KeyValuePair<uint, string>(VFExtractor.ModeBiometrikaFX3000, "BiometriKa FX3000"),
				new KeyValuePair<uint, string>(VFExtractor.ModeKeytronicSecureDesktop, "Keytronic SecureDesktop"),
				new KeyValuePair<uint, string>(VFExtractor.ModeIdentixTouchView, "Identix TouchView"),
				new KeyValuePair<uint, string>(VFExtractor.ModeIdentixDfr2090, "Identix DFR2090"),
				new KeyValuePair<uint, string>(VFExtractor.ModePreciseBiometrics100CS, "PreciseBiometrics 100CS"),
				new KeyValuePair<uint, string>(VFExtractor.ModeUpekTouchChip, "Upek TouchChip"),
				new KeyValuePair<uint, string>(VFExtractor.ModeIdenticatorTechnologyDF90, "IdenticatorTechnology DF90"),
				new KeyValuePair<uint, string>(VFExtractor.ModeAuthentecAFS2, "Authentec AF-S2"),
				new KeyValuePair<uint, string>(VFExtractor.ModeAuthentecAes4000, "Authentec AES4000"),
				new KeyValuePair<uint, string>(VFExtractor.ModeAuthentecAes2501B, "Authentec AES2501B"),
				new KeyValuePair<uint, string>(VFExtractor.ModeAtmelFingerchip, "Atmel FingerChip"),
				new KeyValuePair<uint, string>(VFExtractor.ModeBmfBlp100, "BMF BLP100"),
				new KeyValuePair<uint, string>(VFExtractor.ModeSecugenHamster, "Secugen Hamster"),
				new KeyValuePair<uint, string>(VFExtractor.ModeEthentica, "Ethentica"),
				new KeyValuePair<uint, string>(VFExtractor.ModeCrossmatchVerifier300, "CrossMatch Verifier 300"),
				new KeyValuePair<uint, string>(VFExtractor.ModeNitgenFingkeyHamster, "Nitgen Fingkey Hamster"),
				new KeyValuePair<uint, string>(VFExtractor.ModeTestechBioI, "Testech Bio-i"),
				new KeyValuePair<uint, string>(VFExtractor.ModeDigentIzzix, "Digent Izzix"),
				new KeyValuePair<uint, string>(VFExtractor.ModeStartekFM200, "Startek FM200"),
				new KeyValuePair<uint, string>(VFExtractor.ModeFujitsuMBF200, "Fujitsu MBF200"),
				new KeyValuePair<uint, string>(VFExtractor.ModeFutronicFS80, "Futronic FS80"),
				new KeyValuePair<uint, string>(VFExtractor.ModeLighTuningLttC500, "LighTuning LTT-C500"),
				new KeyValuePair<uint, string>(VFExtractor.ModeTacomaCmos, "Tacoma CMOS")
			};
		private static readonly Dictionary<uint, int> vfeModesToIndexes = new Dictionary<uint, int>();
		private static readonly KeyValuePair<VfeTemplateSize, string>[] vfeTemplateSizes =
			{
				new KeyValuePair<VfeTemplateSize, string>(VfeTemplateSize.Small, "Small"),
				new KeyValuePair<VfeTemplateSize, string>(VfeTemplateSize.Large, "Large")
			};
		private static readonly Dictionary<VfeTemplateSize, int> vfeTemplateSizesToIndexes = new Dictionary<VfeTemplateSize, int>();
		private static readonly KeyValuePair<VfeReturnedImage, string>[] vfeReturnedImages =
			{
				new KeyValuePair<VfeReturnedImage, string>(VfeReturnedImage.None, "None"),
				new KeyValuePair<VfeReturnedImage, string>(VfeReturnedImage.Binarized, "Binarized"),
				new KeyValuePair<VfeReturnedImage, string>(VfeReturnedImage.Skeletonized, "Skeletonized")
			};
		private static readonly Dictionary<VfeReturnedImage, int> vfeReturnedImagesToIndexes = new Dictionary<VfeReturnedImage, int>();

		#endregion

		#region Static constructor

		static EnrollmentOptionsForm()
		{
			int i;
			i = 0;
			foreach (KeyValuePair<uint, string> vfeMode in vfeModes)
			{
				vfeModesToIndexes.Add(vfeMode.Key, i++);
			}
			i = 0;
			foreach (KeyValuePair<VfeTemplateSize, string> vfeTemplateSize in vfeTemplateSizes)
			{
				vfeTemplateSizesToIndexes.Add(vfeTemplateSize.Key, i++);
			}
			i = 0;
			foreach (KeyValuePair<VfeReturnedImage, string> vfeReturnedImage in vfeReturnedImages)
			{
				vfeReturnedImagesToIndexes.Add(vfeReturnedImage.Key, i++);
			}
		}

		#endregion

		#region Public constructor

		public EnrollmentOptionsForm()
		{
			InitializeComponent();

			vfeQualityThresholdPercentLabel.Text = CultureInfo.CurrentCulture.NumberFormat.PercentSymbol;

			vfeModeComboBox.BeginUpdate();
			foreach (KeyValuePair<uint, string> vfeMode in vfeModes)
			{
				vfeModeComboBox.Items.Add(vfeMode.Value);
			}
			vfeModeComboBox.EndUpdate();

			vfeTemplateSizeComboBox.BeginUpdate();
			foreach (KeyValuePair<VfeTemplateSize, string> vfeTemplateSize in vfeTemplateSizes)
			{
				vfeTemplateSizeComboBox.Items.Add(vfeTemplateSize.Value);
			}
			vfeTemplateSizeComboBox.EndUpdate();

			vfeReturnedImageComboBox.BeginUpdate();
			foreach (KeyValuePair<VfeReturnedImage, string> vfeReturnedImage in vfeReturnedImages)
			{
				vfeReturnedImageComboBox.Items.Add(vfeReturnedImage.Value);
			}
			vfeReturnedImageComboBox.EndUpdate();

			vfeGeneralizationFarComboBox.BeginUpdate();
			vfeGeneralizationFarComboBox.Items.Add(0.001.ToString("P1"));
			vfeGeneralizationFarComboBox.Items.Add(0.0001.ToString("P2"));
			vfeGeneralizationFarComboBox.Items.Add(0.00001.ToString("P3"));
			vfeGeneralizationFarComboBox.EndUpdate();

			UpdateSettings();
		}

		#endregion

		#region Private methods

		private void UpdateSettings()
		{
			Settings settings = Settings.Default;
			minimalMinutiaCountNumericUpDown.Value = settings.FPMinimalMinutiaCount;
			generalizationTemplatesNumericUpDown.Value = settings.FPGeneralizationTemplates;
			vfeUseQualityCheckBox.Checked = Data.VFExtractor.UseQuality;
			vfeQualityThresholdNumericUpDown.Value = Data.QualityToPercent(Data.VFExtractor.QualityThreshold);
			vfeModeComboBox.SelectedIndex = vfeModesToIndexes[Data.VFExtractor.Mode];
			vfeTemplateSizeComboBox.SelectedIndex = vfeTemplateSizesToIndexes[Data.VFExtractor.TemplateSize];
			vfeReturnedImageComboBox.SelectedIndex = vfeReturnedImagesToIndexes[Data.VFExtractor.ReturnedImage];
			vfeGeneralizationFarComboBox.Text = Data.MatchingThresholdToString(Data.VFExtractor.GeneralizationThreshold);
			vfeGeneralizationMaximalRotationNumericUpDown.Value = Data.MaximalRotationToDegrees(Data.VFExtractor.GeneralizationMaximalRotation);
		}

		#endregion

		private void EnrollmentOptionsForm_FormClosing(object sender, FormClosingEventArgs e)
		{
			if (DialogResult == DialogResult.OK)
			{
				string error = null;
				try
				{
					int vfeGeneralizationThreshold = Data.MatchingThresholdFromString(vfeGeneralizationFarComboBox.Text);
				}
				catch
				{
					vfeGeneralizationFarComboBox.Select();
					error = "Generalization FAR is not valid";
				}
				if (error != null)
				{
					MessageBox.Show(error, Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
					e.Cancel = true;
				}
			}
		}

		private void EnrollmentOptionsForm_FormClosed(object sender, FormClosedEventArgs e)
		{
			Settings settings = Settings.Default;
			if (DialogResult == DialogResult.OK)
			{
				settings.FPMinimalMinutiaCount = (int)minimalMinutiaCountNumericUpDown.Value;
				settings.FPGeneralizationTemplates = (int)generalizationTemplatesNumericUpDown.Value;
				Data.VFExtractor.UseQuality = vfeUseQualityCheckBox.Checked;
				Data.VFExtractor.QualityThreshold = Data.QualityFromPercent((int)vfeQualityThresholdNumericUpDown.Value);
				Data.VFExtractor.Mode = vfeModes[vfeModeComboBox.SelectedIndex].Key;
				Data.VFExtractor.TemplateSize = vfeTemplateSizes[vfeTemplateSizeComboBox.SelectedIndex].Key;
				Data.VFExtractor.ReturnedImage = vfeReturnedImages[vfeReturnedImageComboBox.SelectedIndex].Key;
				Data.VFExtractor.GeneralizationThreshold = Data.MatchingThresholdFromString(vfeGeneralizationFarComboBox.Text);
				Data.VFExtractor.GeneralizationMaximalRotation = Data.MaximalRotationFromDegrees((int)vfeGeneralizationMaximalRotationNumericUpDown.Value);
			}
			else
			{
				settings.Reload();
				Data.VFExtractor.Reset();
				Data.UpdateVfe();
			}
			Data.UpdateVfeSettings();
			settings.Save();
		}

		private void defaultButton_Click(object sender, EventArgs e)
		{
			Data.ResetSetting("SearchForDuplicates");
			Data.ResetSetting("FPUseMinimalMinutiaCount");
			Data.ResetSetting("FPMinimalMinutiaCount");
			Data.ResetSetting("FPUseGeneralization");
			Data.ResetSetting("FPGeneralizationTemplates");
			Data.VFExtractor.Reset();
			UpdateSettings();
		}
	}
}
