using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Neurotec.Biometrics.VeriFingerDemo.Properties;

namespace Neurotec.Biometrics.VeriFingerDemo
{
	public partial class FPScannersOptionsForm : Form
	{
		#region Private fields

		private FPScannerMan scannerMan;

		#endregion

		#region Public constructor

		public FPScannersOptionsForm()
		{
			InitializeComponent();

			scannerMan = new FPScannerMan(this);
			scannerMan.ScannerAdded += new FPScannerManScannerEventHandler(scannerMan_ScannerAdded);
			scannerMan.ScannerRemoved += new FPScannerManScannerEventHandler(scannerMan_ScannerRemoved);
			foreach (FPScanner scanner in scannerMan.Scanners)
			{
				scannersCheckedListBox.Items.Add(scanner);
			}

			UpdateSettings();
		}

		#endregion

		#region Private methods

		private void UpdateSettings()
		{
			Settings settings = Settings.Default;
			(settings.UseSelectedFPScanners ? selectedRadioButton : allRadioButton).Checked = true;
			string[] values = settings.SelectedFPScanners == null ? new string[0]
				: settings.SelectedFPScanners.Split(Path.PathSeparator);
			scannersCheckedListBox.BeginUpdate();
			int count = scannersCheckedListBox.Items.Count;
			for (int i = 0; i != count; i++)
			{
				bool isChecked = Array.IndexOf(values, ((FPScanner)scannersCheckedListBox.Items[i]).ID) != -1;
				scannersCheckedListBox.SetItemChecked(i, isChecked);
			}
			scannersCheckedListBox.EndUpdate();
		}

		private void scannerMan_ScannerAdded(object sender, FPScannerManScannerEventArgs e)
		{
			scannersCheckedListBox.Items.Add(e.Scanner);
		}

		private void scannerMan_ScannerRemoved(object sender, FPScannerManScannerEventArgs e)
		{
			scannersCheckedListBox.Items.Remove(e.Scanner);
		}

		#endregion

		private void FPScannersForm_FormClosed(object sender, FormClosedEventArgs e)
		{
			Settings settings = Settings.Default;
			if (DialogResult == DialogResult.OK)
			{
				settings.UseSelectedFPScanners = selectedRadioButton.Checked;
				if (scannersCheckedListBox.CheckedItems.Count == 0)
				{
					settings.SelectedFPScanners = null;
				}
				else
				{
					StringBuilder value = new StringBuilder();
					int i = 0;
					foreach (FPScanner scanner in scannersCheckedListBox.CheckedItems)
					{
						if (i != 0) value.Append(Path.PathSeparator);
						value.Append(scanner.ID);
						i++;
					}
					settings.SelectedFPScanners = value.ToString();
				}
				settings.Save();
			}
			else
			{
				settings.Reload();
			}
			scannerMan.Dispose();
			scannerMan = null;
		}

		private void radioButton_CheckedChanged(object sender, EventArgs e)
		{
			scannersCheckedListBox.Enabled = selectedRadioButton.Checked;
		}

		private void defaultButton_Click(object sender, EventArgs e)
		{
			Data.ResetSetting("UseSelectedFPScanners");
			Data.ResetSetting("SelectedFPScanners");
			UpdateSettings();
		}
	}
}
