using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Diagnostics;

namespace Neurotec.Biometrics.VeriFingerDemo
{
	public partial class RocCalculationForm : Form
	{
		#region Private fields

		private GIDistributionsView giDistributionsView;
		private int lastMT;
		private MatchListType rocType;
		private MatchList matchList;
		private DateTime timeStarted;
		private bool finished;
		private GIDistribution giDistribution;
		private int matchedCount, seCount;
		private int matchSpeed;
		private TimeSpan seAveTime;

		#endregion

		#region Public constructor

		public RocCalculationForm()
		{
			InitializeComponent();

			giDistributionsView = new GIDistributionsView();
			giDistributionsView.Dock = DockStyle.Fill;
			Controls.Add(giDistributionsView);
			giDistributionsView.BringToFront();

			lastMT = Data.VFMatcher.MatchingThreshold;
			Data.VFMatcher.MatchingThreshold = 0;
		}

		#endregion

		#region Public properties

		public MatchListType RocType
		{
			get
			{
				return rocType;
			}
			set
			{
				rocType = value;
			}
		}

		#endregion

		private void backgroundWorker_DoWork(object sender, DoWorkEventArgs e)
		{
			int count = Data.Database.Records.Count;
			GIDistributionBuilder gidb = new GIDistributionBuilder();
			DateTime timeLastProgressReported = DateTime.Now;
			int matchedCount = 0, seCount = 0;
			Stopwatch sesw = new Stopwatch();
			Stopwatch sw = new Stopwatch();
			for (int i1 = 0; i1 != count; i1++)
			{
				if (backgroundWorker.CancellationPending) break;
				sesw.Start();
				Data.VFMatcher.IdentifyStart(Data.Database.Records[i1].Template);
				sesw.Stop();
				for (int i2 = 0; i2 != count; i2++)
				{
					if (backgroundWorker.CancellationPending)
					{
						i1 = count - 1;
						break;
					}
					MatchPair matchPair = matchList.GetMatchPair(i1, i2);
					if (matchPair != MatchPair.None)
					{
						sw.Start();
						int score = Data.VFMatcher.IdentifyNext(Data.Database.Records[i2].Template);
						sw.Stop();
						gidb.Add(score, matchPair == MatchPair.Genuine);
						matchedCount++;
						DateTime now = DateTime.Now;
						if ((now - timeLastProgressReported).TotalSeconds >= 0.3)
						{
							backgroundWorker.ReportProgress(matchedCount);
							timeLastProgressReported = now;
						}
					}
				}
				sesw.Start();
				Data.VFMatcher.IdentifyEnd();
				sesw.Stop();
				seCount++;
			}
			giDistribution = gidb.ToGIDistribution(false);
			this.matchedCount = matchedCount;
			this.seCount = seCount;
			if (matchedCount != 0)
			{
				matchSpeed = (int)Math.Round(matchedCount / sw.Elapsed.TotalSeconds);
			}
			if (seCount != 0)
			{
				seAveTime = TimeSpan.FromTicks((long)Math.Round((double)sesw.Elapsed.Ticks / seCount));
			}
		}

		private void backgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
		{
			double p = (double)e.ProgressPercentage / matchList.Count;
			progressBar.Value = (int)Math.Round(p * 100);
			TimeSpan timeElapsed = DateTime.Now - timeStarted;
			timeElapsedLabel.Text = Data.TimeToString(timeElapsed);
			TimeSpan timeRemaining = TimeSpan.FromTicks((long)Math.Round((double)timeElapsed.Ticks * (1 - p) / p));
			timeRemainingLabel.Text = Data.TimeToString(timeRemaining);
		}

		private void backgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
		{
			finished = true;
			closeButton.Text = "Close";
			closeButton.Enabled = true;
			timeElapsedLabel.Text = Data.TimeToString(DateTime.Now - timeStarted);
			timeRemainingLabel.Text = null;
			progressBar.Visible = false;
			if (e.Error != null)
			{
				MessageBox.Show(string.Format("Error: {0}", e.Error.Message), Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
			else
			{
				string text = string.Format("ROC calculation {0}", matchedCount == matchList.Count ? "complete" : "stopped");
				if (matchedCount == 0)
				{
					text += ". Nothing matched";
				}
				else
				{
					text += string.Format(". {0} template pair(s) matched. Matching speed: {1} template pair(s) per second. Average start/end time: {2}",
						matchedCount, matchSpeed, Data.TimeToString(seAveTime));
					if (matchedCount == matchList.Count)
					{
						giDistributionsView.Items.Add(new GIDistributionsViewItem(giDistribution));
					}
				}
				MessageBox.Show(text, Text, MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
		}

		private void RocCalculationForm_FormClosing(object sender, FormClosingEventArgs e)
		{
			if (!finished)
			{
				closeButton.Enabled = false;
				backgroundWorker.CancelAsync();
				e.Cancel = true;
			}
		}

		private void RocCalculationForm_Shown(object sender, EventArgs e)
		{
			matchList = new MatchList(Data.Database, rocType);
			finished = false;
			timeStarted = DateTime.Now;
			backgroundWorker.RunWorkerAsync();
		}

		private void RocCalculationForm_FormClosed(object sender, FormClosedEventArgs e)
		{
			Data.VFMatcher.MatchingThreshold = lastMT;
		}
	}
}
